<?php
/**
 * Created by PhpStorm.
 * @author mihovil.bubnjar
 * @date 08.08.2016
 * @time 15:44
 */

class Shopware_Plugins_Frontend_SicoCreditPlus_Bootstrap extends \Shopware_Components_Plugin_Bootstrap {

	/**
	 * Saves data between "saveOrder" and "saveOrderAttributes" to move attached CreditPlusPayment data to the correct order
	 * @var array
	 */
	protected $aLastOrderData = array();

	/**
	 * Gibt die Versionsnummer des Plugins aus.
	 * Ist relevant für die Update Anweisungen.
	 *
	 * @return string Versionsnummer des Plugins
	 */
	public function getVersion() {
		return '1.3.8';
	}

	/**
	 * Gibt den Namen des Plugins aus.
	 *
	 * @return string Der Name des Plugins
	 */
	public function getLabel() {
		return 'CreditPlus Finanzierung';
	}

	/**
	 * Gibt parent::getCapabilities + enable und disable auf true aus
	 *
	 * @return array Verfügbare Funktionen als Index, true als Wert
	 */
	public function getCapabilities() {
		$aReturn = parent::getCapabilities();
		$aReturn['enable'] = true;
		$aReturn['disable'] = true;
		return $aReturn;
	}

	/**
	 * Gibt ein paar nette Informationen für das Backend aus.
	 * @return array
	 */
	public function getInfo() {
		$aReturn = parent::getInfo();
		$aReturn['author'] = 'SinkaCom AG';
		$aReturn['link'] = 'http://www.sinkacom.de/';
		try {
			$aPluginInfo = json_decode(file_get_contents( __DIR__.'/plugin.json'), true);
			$aChangeLog = $aPluginInfo['changelog'];
			// Will be read from user session soon
			$sLocale = 'de_DE';
			if ( array_key_exists($sLocale, $aChangeLog) ) {
				$aChangeLog = $aChangeLog[$sLocale];
			} else {
				$aChangeLog = $aChangeLog['de_DE'];
			}
			if ( array_keys($aPluginInfo['installationManual'], $sLocale) ) {
				$sInstallationManual = $aPluginInfo['installationManual'][$sLocale];
			} else {
				$sInstallationManual = $aPluginInfo['installationManual']['de_DE'];
			}
			if ( array_keys($aPluginInfo['description'], $sLocale) ) {
				$sDescription = $aPluginInfo['description'][$sLocale];
			} else {
				$sDescription = $aPluginInfo['description']['de_DE'];
			}
			$aReturn['changelog'] = $aChangeLog;
			$aReturn['copyright'] = $aPluginInfo['copyright'];
			$aReturn['description'] = $sDescription;
			$aReturn['installationManual'] = $sInstallationManual;
		} catch ( \Exception $e ) {
			// No changelog, no info, no nothing
		}
		return $aReturn;
	}

	/**
	 * Führt diverse Funktionen bei der Installation des Plugins aus.
	 * @return array|bool
	 * @throws \Exception
	 */
	public function install() {
		parent::install();
		$this->addPayment();
		$this->updatePaymentStates();
		$this->createConfig();
		$this->addControllers();
		$this->addModels();
		$this->addFrontendControllerExtensions();
		$this->addLoggingSubscribes();
		$this->installMailTemplates();
		$this->installDefaultFinancingOptions();
		$this->addBackendMenu();
		$this->addBackendControllerExtensions();
		$this->addNotifiers();
		$this->addSnippets();
		$this->addCronJobs();
		return $this->defaultSuccess();
	}

	/**
	 * Legt eine Menge von Konfigurationseinträgen im Backend an.
	 * Die meisten sind mit Default Werten
	 * gefüllt, einige müssen jedoch geändert werden.
	 */
	protected function createConfig() {
		$oForm = $this->Form();
		$oForm->setElement(
			'boolean',
			'bTestMode',
			array(
				'label' => 'Test Modus',
				'value' => true,
				'position' => 1,
			)
		);
		$oForm->setElement(
			'boolean',
			'bLogActive',
			array(
				'label' => 'Protokoll führen',
				'description' => 'Dies sollte nur bei vermuteten Fehlern aktiv sein, da es personenbezogene Daten speichert.',
				'value' => true,
				'position' => 2,
			)
		);
		$oForm->setElement(
			'text',
			'sCPDealer',
			array(
				'label' => 'Händlernummer',
				'value' => '500276',
				'position' => 3,
			)
		);
		$oForm->setElement(
			'text',
			'sDealerName',
			array(
				'label' => 'Händler Name',
				'value' => 'WebFin Demo Hdl',
				'position' => 4,
			)
		);
		$oForm->setElement(
			'text',
			'sSoapUser',
			array(
				'label' => 'SOAP API Benutzer',
				'value' => 'testwebshop-500276',
				'position' => 5,
			)
		);
		$oForm->setElement(
			'text',
			'sSoapPass',
			array(
				'label' => 'SOAP API Passwort',
				'value' => '',
				'position' => 6,
			)
		);
		$oForm->setElement(
			'select',
			'sSoapType',
			array(
				'label' => 'SOAP API Passwort Art',
				'store' => array(
					array('PasswordDigest', 'PasswordDigest'),
				),
				'value' => 'PasswordDigest',
				'position' => 7,
			)
		);
		$oForm->setElement(
			'text',
			'sPartnerName',
			array(
				'label' => 'Partner Name',
				'value' => '',
				'required' => false,
				'position' => 8,
			)
		);
		$oForm->setElement(
			'text',
			'sWSDL',
			array(
				'label' => 'SOAP API WSDL',
				'value' => 'https://kreditrechner-long-test.creditplus.de/webshopService/cxf/Webshop?wsdl',
				'position' => 9,
			)
		);
		$oForm->setElement(
			'select',
			'sCPShownAs',
			array(
				'label' => 'Anzeigen als',
				'store' => array(
					array('popup', 'Popup'),
					array('iframe', 'iframe'),
					array('new_window', 'Neues Fenster'),
				),
				'value' => 'popup',
				'position' => 10,
			)
		);
		$oForm->setElement(
			'number',
			'sMinRate',
			array(
				'label' => 'Mindestrate',
				'value' => 25.00,
				'minValue' => 25.00,
				'maxValue' => 9999.99,
				'position' => 11,
			)
		);
		$oForm->setElement(
			'number',
			'sMinBasketPrice',
			array(
				'label' => 'Mindest Warenkorbwert',
				'value' => 150.00,
				'minValue' => 150.00,
				'maxValue' => 999999.99,
				'position' => 12,
			)
		);
		$oForm->setElement(
			'number',
			'sMaxBasketPrice',
			array(
				'label' => 'Maximaler Warenkorbwert',
				'value' => 40000.00,
				'minValue' => 150.00,
				'maxValue' => 999999.99,
				'position' => 13,
			)
		);
		$oForm->setElement(
			'select',
			'sTransactionMode',
			array(
				'label' => 'Zahlungszeitpunkt',
				'store' => array(
					array('postorder', 'Nach Bestellabschluss'),
					array('inorder', 'Vor Bestellabschluss'),
				),
				'value' => 'postorder',
				'position' => 14,
			)
		);
		$oForm->setElement(
			'select',
			'sTransactionNumber',
			array(
				'label' => 'Transaktionsnummer',
				'description' => 'Der hier ausgewählte Wert wird als Bestellnummer an CreditPlus übertragen. Er taucht dann auch bei den Kunden bei den Lastschriften zur Finanzierung auf.',
				'store' => array(
					array('ordernumber', 'Bestellnummer'),
					array('cpnumber', 'Transaktionsnummer'),
				),
				'value' => 'ordernumber',
				'position' => 15,
			)
		);
		$oForm->setElement(
			'text',
			'sSignatureSalt',
			array(
				'label' => 'Shared Secret (für Signaturen)',
				'value' => '',
				'position' => 16,
			)
		);
		$oForm->setElement(
			'select',
			'sBasketFinancingMode',
			array(
				'label' => 'Warenkorbfinanzierung',
				'store' => array(
					// array('most-expensive', 'Teuerster Zins'),
					// array('weighted-majority', 'Gewichteter Zins'),
					// array('number-majority', 'Häufigster Zins'),
					array('cheapest', 'Günstigster Zins'),
				),
				'value' => 'cheapest',
				'position' => 17,
			)
		);
		$oForm->setElement(
			'boolean',
			'bShowDetails',
			array(
				'label' => 'Ratentabelle und günstigste Rate auf Details anzeigen',
				'value' => true,
				'position' => 18,
			)
		);
		$oForm->setElement(
			'select',
			'sRateTableMode',
			array(
				'label' => 'Ratentabelle Anzeigeort',
				'store' => array(
					array(0, 'In den Tabs'),
					array(1, 'In einer Modal Box')
				),
				'value' => 0,
				'position' => 19,
		));
		$oForm->setElement(
			'boolean',
			'bShowBasket',
			array(
				'label' => 'Ratentabelle auf Warenkorb anzeigen',
				'value' => true,
				'position' => 20,
			)
		);
		$oForm->setElement(
			'boolean',
			'bShowPayment',
			array(
				'label' => 'Ratentabelle auf Zahlungsseite anzeigen',
				'value' => true,
				'position' => 21,
			)
		);
		$this->createStatusMailTargets();
		$oForm->setElement(
			'select',
			'sMailTemplate',
			array(
				'label' => 'Mailvorlage für Bestellung',
				'description' => 'Wenn Sie hier "Shopware Standard" auswählen, müssen Sie in Text und HTML Variante die entsprechenden Blöcke aus der Installationsdokumentation eintragen.',
				'store' => array(
					array('sORDERForCreditPlus', 'CreditPlus Standard'),
					array('sORDER', 'Shopware Standard'),
				),
				'value' => 'sORDERForCreditPlus',
				'position' => 29,
			)
		);
		$oForm->setElement(
			'boolean',
			'bSendMailOnBackendStatechange',
			array(
				'label' => 'E-Mail Formular bei Statuswechsel im Backend öffnen',
				'value' => false,
				'position' => 30,
			)
		);
	}

	protected function createStatusMailTargets() {

		$oForm = $this->Form();
		// Added in 1.2.3 - Mails on status change
		$oForm->setElement(
			'text',
			'sEmailState20',
			array(
				'label' => 'Empfänger E-Mail Adresse für Status Offen',
				'value' => '',
				'position' => 22,
			)
		);
		$oForm->setElement(
			'text',
			'sEmailState24n',
			array(
				'label' => 'Empfänger E-Mail Adresse für Status Vorläufig genehmigt',
				'value' => '',
				'position' => 23,
			)
		);
		$oForm->setElement(
			'text',
			'sEmailState24a',
			array(
				'label' => 'Empfänger E-Mail Adresse für Status Lieferfreigabe',
				'value' => '',
				'position' => 24,
			)
		);
		$oForm->setElement(
			'text',
			'sEmailState25',
			array(
				'label' => 'Empfänger E-Mail Adresse für Status Warten auf Unterlagen',
				'value' => '',
				'position' => 25,
			)
		);
		$oForm->setElement(
			'text',
			'sEmailState92',
			array(
				'label' => 'Empfänger E-Mail Adresse für Status Abgelehnt',
				'value' => '',
				'position' => 26,
			)
		);
		$oForm->setElement(
			'text',
			'sEmailState95',
			array(
				'label' => 'Empfänger E-Mail Adresse für Status Storniert',
				'value' => '',
				'position' => 27,
			)
		);
		$oForm->setElement(
			'text',
			'sEmailState99',
			array(
				'label' => 'Empfänger E-Mail Adresse für Status Ausgezahlt',
				'value' => '',
				'position' => 28,
			)
		);
	}

	/**
	 * Fügt die Zahlungsart hinzu, wenn diese noch nicht existiert
	 */
	protected function addPayment() {
		$oDB = Shopware()->Db();
		$aResult = $oDB->fetchAssoc('SELECT id FROM s_core_paymentmeans WHERE name = :payment',array('payment' => 'sccp_financing'));
		if ( !$aResult ) {
			$aPayment = array(
				'name' => 'sccp_financing',
				'description' => 'CreditPlus Finanzierung',
				'template' => 'sccp_financing.tpl',
				'class' => 'SicoCreditPlus\\Models\\PaymentMethod\\SicoFinancing',
				'hide' => 0,
				'additionaldescription' => 'Bezahlen Sie Ihren Einkauf schnell und bequem über monatlich niedrige Raten mit der CreditPlus Finanzierung.',
				'debitPercent' => 0.00,
				'active' => 1
			);
			$this->createPayment($aPayment, 'CreditPlus Finanzierung');
		}
	}

	/**
	 * Adds CreditPlus payment states to the database.
	 * Since the corresponding Model of Shopware (\Shopware\Models\Order\Status) does NOT support insertion
	 * this is done the old SQL way.
	 * @see \SicoCreditPlus\Components\SicoCreditPlusHelper::getCreditPlusPaymentStates() Provides the list of payment states
	 *
	 * @throws \Zend_Db_Adapter_Exception If flush does not work
	 */
	protected function updatePaymentStates() {
		$this->registerCustomModels();
		$oDB = Shopware()->Db();
		$bNameExists = property_exists('Shopware\Models\Order\Status','name');
		if ( !$bNameExists ) {
			$aFields = $oDB->fetchAll('SELECT * FROM s_core_states LIMIT 1');
			if ( $aFields[0] ) {
				if ( !isset($aFields[0]['name']) ) {
					$oDB->exec('ALTER TABLE s_core_states ADD COLUMN name VARCHAR(55) DEFAULT \'\' AFTER id;');
				}
			}
		}
		$aResults = $oDB->fetchAll('SELECT id, name FROM s_core_states WHERE name LIKE :creditplusstates', array('creditplusstates' => 'creditplus_%'), Zend_Db::FETCH_ASSOC);
		$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$aInsertStates = $oCPHelper->getCreditPlusPaymentStates();
		$aReplacements = $oCPHelper->getCreditPlusPaymentStateReplacements();
		/** @var string[] $aReplacementsIDtoName Array in form of 37 => newstatename */
		$aReplacementsIDtoName = array();
		/** @var int[] $aInsertedIDs Array in form of newstatename => 51 */
		$aNameToID = array();
		// Do not insert already known states
		// Remember old states to be remapped, then deleted
		// Remember state ids from database
		if ( $aResults ) {
			foreach ( $aResults as $aResult ) {
				if ( isset($aInsertStates[$aResult['name']]) ) {
					unset($aInsertStates[$aResult['name']]);
				} elseif ( isset($aReplacements[$aResult['name']]) ) {
					$aReplacementsIDtoName[$aResult['id']] = $aReplacements[$aResult['name']];
				}
				$aNameToID[$aResult['name']] = $aResult['id'];
			}
		}

		// Neue Status einpflegen
		if ( $aInsertStates ) {
			$oSelect = $oDB->select();
			$oSelect->from(array('s' => 's_core_states'), ('id'))
				->order('id DESC')
				->limit(1)
			;
			$oQuery = $oSelect->query();
			$oQuery->execute();
			$iMaxID = (int)$oQuery->fetchColumn(0);
			foreach ( $aInsertStates as $aInsertState ) {
				$iMaxID++;
				$aInsertBinding = array(
					'id' => $iMaxID,
					'name' => $aInsertState['name'],
					'description' => $aInsertState['description'],
					'group' => $aInsertState['group'],
					'position' => $aInsertState['position'],
					'mail' => $aInsertState['mail']
				);
				$oDB->insert('s_core_states', $aInsertBinding);
				$aNameToID[$aInsertState['name']] = $iMaxID;
			}
		}
		if ( $aReplacementsIDtoName ) {
			$oQueryUpdate = $oDB->prepare('UPDATE s_order SET cleared = :newstate WHERE cleared = :oldstate');
			$oQueryDelete = $oDB->prepare('DELETE FROM s_core_states WHERE id = :oldstate');
			foreach ( $aReplacementsIDtoName as $iID => $sNewState ) {
				$iNewState = $aNameToID[$sNewState];
				$oQueryUpdate->execute(array('newstate' => $iNewState, 'oldstate' => $iID));
				$oQueryDelete->execute(array('oldstate' => $iID));
			}
		}
	}

	/**
	 * Fügt dem Shopware Shop die zwei Controller für das Backend hinzu,
	 * welche für die Konfiguration notwendig sind.
	 *
	 * Fügt im Frontend die Trigger URL hinzu,
	 * welche für die Kommunikation Shop <-> CreditPlus notwendig ist.
	 *
	 * Fügt eine Helper Funktion für das Backend hinzu,
	 * welche bei diversen Änderungen und Buttons im CreditPlus Tab der
	 * Bestellung benutzt wird.
	 *
	 * @throws \Exception
	 */
	protected function addControllers() {
		$this->registerController('Frontend', 'Sicotrigger');
		$this->registerController('Backend', 'SicoCreditPlusProductgroups');
		$this->registerController('Backend', 'SicoCreditPlusOfferedOptions');
		$this->registerController('Backend', 'SicoCreditPlusOrder');
	}

	/**
	 * For 4.0.1 compatibility, taken from 5.2.x code
	 *
	 * @param string $module
	 * @param string $name
	 * @param string $listener
	 *
	 * @return Shopware_Components_Plugin_Bootstrap
	 * @throws Exception
	 */
	public function registerController($module, $name, $listener = 'getDefaultControllerPath') {
		if ( method_exists('\Shopware_Components_Plugin_Bootstrap', 'registerController') ) {
			return parent::registerController($module, $name, $listener);
		}
		if (empty($module)) {
			throw new Exception('Register controller requires a module name');
		}
		if (empty($name)) {
			throw new Exception('Register controller requires a controller name');
		}
		$this->subscribeEvent(
			'Enlight_Controller_Dispatcher_ControllerPath_' . $module . '_' . $name,
			$listener
		);

		return $this;
	}

	/**
	 * For 4.0.1 compatibility, taken from 5.2.x code
	 *
	 * @param Enlight_Event_EventArgs $arguments
	 * @throws Exception
	 * @return string
	 */
	public function getDefaultControllerPath(Enlight_Event_EventArgs $arguments) {
		if ( method_exists('\Shopware_Components_Plugin_Bootstrap', 'getDefaultControllerPath') ) {
			return parent::getDefaultControllerPath($arguments);
		}
		$eventName = $arguments->getName();
		$eventName = str_replace('Enlight_Controller_Dispatcher_ControllerPath_', '', $eventName);

		$parts = explode('_', $eventName);

		$module = $parts[0];
		$controller = $parts[1];

		$path = $this->Path() . 'Controllers/' . ucfirst($module) . '/' . ucfirst($controller) . '.php';

		if (!file_exists($path)) {
			throw new Enlight_Exception('Controller "' . $controller . '" can\'t load failure');
		}

		//register plugin model directory
		if (file_exists($this->Path() . 'Models')) {
			$this->registerCustomModels();
		}

		//register plugin views directory
		if (file_exists($this->Path() . 'Views')) {
			Shopware()->Template()->addTemplateDir(
				$this->Path() . 'Views/'
			);
		}

		//register plugin snippet directory
		if (file_exists($this->Path() . 'Snippets')) {
			Shopware()->Snippets()->addConfigDir(
				$this->Path() . 'Snippets/'
			);
		}

		//register plugin component directory
		if (file_exists($this->Path() . 'Components')) {
			Shopware()->Loader()->registerNamespace(
				'Shopware_Components',
				$this->Path() . 'Components/'
			);
		}

		return $path;
	}

	/**
	 * Legt in der Datenbank die Tabellen zu den Elementen an.
	 * Wichtig: Wenn bei updateSchema nicht der Parameter true angegeben wird, wird die ganze Datenbank geleert!
	 */
	protected function addModels() {
		$this->registerCustomModels();

		$oModelManager = Shopware()->Models();
		$oSchemaTool = new Doctrine\ORM\Tools\SchemaTool($oModelManager);
		$oSchemaTool->updateSchema(
			array(
				$oModelManager->getClassMetadata('SicoCreditPlus\Models\CreditPlusPayment\CreditPlusPayment'),
				$oModelManager->getClassMetadata('SicoCreditPlus\Models\OfferedOption\OfferedOption'),
				$oModelManager->getClassMetadata('SicoCreditPlus\Models\Productgroup\Productgroup'),
				$oModelManager->getClassMetadata('SicoCreditPlus\Models\RequestQueue\RequestQueue'),
			),
			//Do NOT!!! touch this true value
			true
		);
	}

	/**
	 * Update des Plugins
	 *
	 * @param string $sOldVersion
	 *
	 * @return array|bool
	 * @throws \Doctrine\ORM\OptimisticLockException
	 * @throws Zend_Db_Adapter_Exception
	 */
	public function update($sOldVersion) {
		$aReturn = array( 'success' => true );
		if ( version_compare($sOldVersion, '0.0.1', '<=') ) {
			// No install/registration things changed
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '0.0.2', '<=') ) {
			// No install/registration things changed
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '0.0.3', '<=') ) {
			// No install/registration things changed
			$aReturn = $this->addSuccess($aReturn, $this->defaultSuccess());
		}
		if ( version_compare($sOldVersion, '0.0.4', '<=') ) {
			$this->subscribeEvent(
				'Enlight_Controller_Action_PostDispatch_Backend_PluginManager',
				'afterPluginManager'
			);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config', 'proxy')));
		}
		if ( version_compare($sOldVersion, '0.0.5', '<=') ) {
			$this->subscribeEvent(
				'Enlight_Controller_Action_PreDispatch_Frontend_Checkout',
				'beforeCheckout'
			);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config', 'proxy')));
		}
		if ( version_compare($sOldVersion, '0.0.6', '<=') ) {
			// Added "inorder" to store
			$this->Form()->setElement(
				'select',
				'sTransactionMode',
				array(
					'label' => 'Zahlungszeitpunkt',
					'store' => array(
						array('postorder', 'Nach Bestellabschluss'),
						array('inorder', 'Vor Bestellabschluss'),
					),
					'value' => 'postorder',
					'position' => 13,
				)
			);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config')));
		}
		if ( version_compare($sOldVersion, '0.0.7', '<=') ) {
			// Backend order windows updated
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'template')));
		}

		if ( version_compare($sOldVersion, '1.0.0', '<=') ) {
			// Backend order windows + OfferedOption assignment updated
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'template')));
		}
		if ( version_compare($sOldVersion, '1.0.1', '<=') ) {
			// Backend default options and required fields
			$this->Form()->setElement(
				'text',
				'sCPDealer',
				array(
					'label' => 'Händlernummer',
					'value' => '500276',
					'position' => 3,
				)
			);
			$this->Form()->setElement(
				'text',
				'sDealerName',
				array(
					'label' => 'Händler Name',
					'value' => 'WebFin Demo Hdl',
					'position' => 4,
				)
			);
			$this->Form()->setElement(
				'text',
				'sSoapUser',
				array(
					'label' => 'SOAP API Benutzer',
					'value' => 'testwebshop-500276',
					'position' => 5,
				)
			);
			$this->Form()->setElement(
				'text',
				'sSoapPass',
				array(
					'label' => 'SOAP API Passwort',
					'value' => '',
					'position' => 6,
				)
			);
			$this->Form()->setElement(
				'text',
				'sPartnerName',
				array(
					'label' => 'Partner Name',
					'value' => '',
					'required' => false,
					'position' => 8,
				)
			);
			$this->Form()->setElement(
				'text',
				'sSignatureSalt',
				array(
					'label' => 'Shared Secret (für Signaturen)',
					'value' => '',
					'position' => 16,
				)
			);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend')));
		}
		if ( version_compare($sOldVersion, '1.0.2', '<=') ) {
			// Only API backend changes which don't require cache updates
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.0.3', '<=') ) {
			// Backend labelling has been changed, invalidate backend caches just to make sure it now gets read out correctly.
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend')));
		}
		if ( version_compare($sOldVersion, '1.0.4', '<=') ) {
			// Added Cache clear on Config Change
			$this->subscribeEvent(
				'Enlight_Controller_Action_PreDispatch_Backend_Config',
				'beforeBackendConfig'
			);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config','backend')));
		}
		if ( version_compare($sOldVersion, '1.0.5', '<=') ) {
			// Only changes in PHP Code for the cart page
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.0.6', '<=') ) {
			// Only changes in PHP Code for the cart page
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.0.7', '<=') ) {
			// Add new payment states for overview
			$this->updatePaymentStates();
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config','backend')));
			$this->subscribeEvent(
				'Enlight_Controller_Action_PreDispatch_Backend_Order',
				'beforeBackendOrder'
			);
		}
		if ( version_compare($sOldVersion, '1.0.8', '<=') ) {
			// Add 4.3 style "Set transactionID" function
			if ( version_compare($this->Config()->get('version'),'5.0.0','<') ) {
				$this->subscribeEvent(
					'Shopware_Modules_Order_SaveOrder_FilterSQL',
					'filterSaveOrderOrderFilterSQL'
				);
			}
		}

		if ( version_compare($sOldVersion, '1.1.0', '<=') ) {
			$oModelManager = Shopware()->Models();
			$oMailRep = $oModelManager->getRepository('Shopware\Models\Mail\Mail');
			$oMail = $oMailRep->findOneBy(array('name' => 'sORDERForCreditPlus'));
			$sMailContent = $oMail->getContent();
			$sMailContent = str_replace('{$billingaddress.street}','{$billingaddress.street}{if $billingaddress.streetnumber} {$billingaddress.streetnumber}{/if}', $sMailContent);
			$sMailContent = str_replace('{$shippingaddress.street}','{$shippingaddress.street}{if $shippingaddress.streetnumber} {$shippingaddress.streetnumber}{/if}', $sMailContent);
			$oMail->setContent($sMailContent);
			$sMailContentHTML = $oMail->getContentHtml();
			$sMailContentHTML = str_replace('{$billingaddress.street}','{$billingaddress.street}{if $billingaddress.streetnumber} {$billingaddress.streetnumber}{/if}', $sMailContentHTML);
			$sMailContentHTML = str_replace('{$shippingaddress.street}','{$shippingaddress.street}{if $shippingaddress.streetnumber} {$shippingaddress.streetnumber}{/if}', $sMailContentHTML);
			$oMail->setContentHtml($sMailContentHTML);
			$oModelManager->flush($oMail);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config','template','theme')));
		}
		if ( version_compare($sOldVersion, '1.1.1', '<=') ) {
			// Added template blocks
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config', 'frontend', 'template', 'theme')));
		}
		if ( version_compare($sOldVersion, '1.1.2', '<=') ) {
			// Just a bugfix for 4.3.x shops
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.1.3', '<=') ) {
			// Just a bugfix for 5.x.x shops
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.1.4', '<=') ) {
			// Bugfix for detail page
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.1.5', '<=') ) {
			$this->Form()->setElement(
				'text',
				'sWSDL',
				array(
					'label' => 'SOAP API WSDL',
					'value' => 'https://kessdemo.creditplus.de/ws_webshop/cxf/Webshop?wsdl',
					'position' => 9,
				)
			);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config', 'frontend', 'template', 'theme', 'backend')));
		}
		if ( version_compare($sOldVersion, '1.1.6', '<=') ) {
			// Bugfix for cheapest rate selection
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.1.7', '<=') ) {
			// Update for Shopware 5.3.x Shop and new default theme JS
			// Refreshing frontend/template caches for 5.x Shops
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('frontend', 'template', 'theme')));
		}

		if ( version_compare($sOldVersion, '1.2.0', '<=') ) {
			// Bugfix for paying after changing the order state
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.1', '<=') ) {
			// Bugfix for cancellation after order being declined
			$this->updateDefaultFinancings();
			$this->Form()->setElement(
				'number',
				'sMaxBasketPrice',
				array(
					'label' => 'Maximaler Warenkorbwert',
					'value' => 40000.00,
					'minValue' => 150.00,
					'maxValue' => 999999.99,
					'position' => 13,
				)
			);
		}
		if ( version_compare($sOldVersion, '1.2.2', '<=') ) {
			// Added new Mail templates
			$this->installMailTemplates();
			$this->createStatusMailTargets();
			$this->subscribeEvent(
				'SicoCreditPlus_Financing_Status_Changed',
				'onSicoCreditPlusFinancingStatusChanged'
			);
		}
		if ( version_compare($sOldVersion, '1.2.3', '<=') ) {
			// Changed EAN handling on createCreditOffer
			// Added update mail on first time finish
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.4', '<=') ) {
			// Changed template on details page
			// Added ajax update for financing tab
			// Updated E-Mail templates (colors)
			$this->updateMailTemplates();
			$this->updateSnippets();
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('frontend', 'template', 'theme')));
		}
		if ( version_compare($sOldVersion, '1.2.5', '<=') ) {
			$this->Form()->setElement(
				'select',
				'sMailTemplate',
				array(
					'label' => 'Mailvorlage für Bestellung',
					'description' => 'Wenn Sie hier "Shopware Standard" auswählen, müssen Sie in Text und HTML Variante die entsprechenden Blöcke aus der Installationsdokumentation eintragen.',
					'store' => array(
						array('sORDERForCreditPlus', 'CreditPlus Standard'),
						array('sORDER', 'Shopware Standard'),
					),
					'value' => 'sORDERForCreditPlus',
					'position' => 28,
				)
			);
			$this->Form()->setElement(
				'select',
				'sCPShownAs',
				array(
					'label' => 'Anzeigen als',
					'store' => array(
						array('popup', 'Popup'),
						array('iframe', 'iframe'),
						array('new_window', 'Neues Fenster'),
					),
					'value' => 'popup',
					'position' => 10,
				)
			);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'config')));
		}
		if ( version_compare($sOldVersion, '1.2.6', '<=') ) {
			$this->updateSnippets();
			$this->addSnippets();
			$this->Form()->setElement(
				'boolean',
				'bSendMailOnBackendStatechange',
				array(
					'label' => 'E-Mail Formular bei Statuswechsel im Backend öffnen',
					'value' => false,
					'position' => 29,
				)
			);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'config', 'frontend', 'template', 'theme')));
		}
		if ( version_compare($sOldVersion, '1.2.7', '<=') ) {
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config')));
		}
		if ( version_compare($sOldVersion, '1.2.8', '<=') ) {
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config')));
		}
		if ( version_compare($sOldVersion, '1.2.9', '<=') ) {
			// Update Class MetaData for Request Queue
			$this->addModels();

			$this->createCronJob(
				'CreditPlus Status Wiederholung',
				'Shopware_CronJob_SicoCreditPlusRequestQueue',
				300,
				1,
				true
			);
			$this->subscribeEvent(
				'Shopware_CronJob_SicoCreditPlusRequestQueue',
				'onCronJobRequestQueue'
			);

			$this->createCronJob(
				'CreditPlus Status Fehler',
				'Shopware_CronJob_SicoCreditPlusDailyError',
				86400,
				1,
				true
			);
			$bUpdated = false;
			if ( method_exists($this, 'get') ) {
				/** @var \Doctrine\DBAL\Connection $oDB */
				try {
					$oDB = $this->get('dbal_connection');
				} catch (\Exception $oEx ) {
					$oDB = Shopware()->Db();
				}
				if ( $oDB && ( ($oDB instanceof \Doctrine\DBAL\Connection) || ($oDB instanceof \Enlight_Components_Db_Adapter_Pdo_Mysql) ) ) {
					try {
						$oDB->update('s_crontab',
							array(
								'inform_mail' => '{$sConfig.sMAIL}',
								'inform_template' => 'sCreditPlusRetryErrors',
							),
							array(
								'action' => 'Shopware_CronJob_SicoCreditPlusDailyError'
							)
						);
						$bUpdated = true;
					} catch ( \Zend_Db_Adapter_Exception $oEx ) {
						// This will make the $bUpdated remain on false
					}
				}
			}
			if ( $bUpdated === false ) {
				try {
					Shopware()->Db()->query( "UPDATE s_crontab SET inform_mail = ?, inform_template = ? WHERE action = ?", array(
						'{$sConfig.sMAIL}',
						'sCreditPlusRetryErrors',
						'Shopware_CronJob_SicoCreditPlusDailyError'
					) );
				} catch ( \Zend_Db_Adapter_Exception $oZDAE ) {
					// We tried and failed.
				}
			}
			$this->subscribeEvent(
				'Shopware_CronJob_SicoCreditPlusDailyError',
				'onCronJobDailyError'
			);
			$this->installMailTemplates();
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'config', 'template', 'theme')));
		}
		if ( version_compare($sOldVersion, '1.2.10', '<=') ) {
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.11', '<=') ) {
			$this->addSnippets();
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'config', 'template')));
		}
		if ( version_compare($sOldVersion, '1.2.12', '<=') ) {
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.13', '<=') ) {
			// Only signature checking improved
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.14', '<=') ) {
			// Only signature checking improved
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.15', '<=') ) {
			// Removed debug output to developer
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.16', '<=') ) {
			// Changed logging from file based to shop specific
			$this->createConfig();
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'config')));
		}
		if ( version_compare($sOldVersion, '1.2.17', '<=') ) {
			// Only code
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.18', '<=') ) {
			// Only code
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.19', '<=') ) {
			// Added transaction number select box
			$this->createConfig();
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'config')));
		}
		if ( version_compare($sOldVersion, '1.2.20', '<=') ) {
			// Only code
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.21', '<=') ) {
			$this->updateBackendStaticPaymentStatusSnippets();
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'config', 'template')));
		}
		if ( version_compare($sOldVersion, '1.2.22', '<=') ) {
			// Only code
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.2.24', '<=') ) {
			$element = $this->Form()->getElement('sTransactionMode');
			$options = $element->getOptions();
			unset($options['store'][1]);
			$element->setOptions($options);
			$this->Form()->setElement(
				'select',
				'sTransactionMode',
				$options);
		}
		if ( version_compare($sOldVersion, '1.2.25', '<=') ) {
			// Add inorder again
			$element = $this->Form()->getElement('sTransactionMode');
			$options = $element->getOptions();
			$options['store'][1] = array('inorder', 'Vor Bestellabschluss');
			$element->setOptions($options);
			$this->Form()->setElement(
				'select',
				'sTransactionMode',
				$options);
			$oShopware = $this->Application();
			$oSnippets = $oShopware->Snippets();
			$oTranslation = $oSnippets->getNamespace('frontend/plugins/payment/creditplus');
			$oTranslation->get('FinishTextToPaymentInorder', 'Die Bestellung ist zwar abgeschlossen, aber Sie müssen jetzt noch einen Identitätsnachweis zur Finanzierung durchgehen. Diesen erreichen Sie per Klick auf:<br /><br /><a class="sccp-continue-to-payment" href="http://retry.me/">Jetzt Bestätigen!</a><br />', true);

			$this->registerCustomModels();
			$oShopware->Hooks()->getProxyFactory()->clearCache();
			$oModelManager = $oShopware->Models();
			$oSchemaTool = new Doctrine\ORM\Tools\SchemaTool($oModelManager);
			$oSchemaTool->updateSchema(
				array(
					$oModelManager->getClassMetadata('SicoCreditPlus\Models\CreditPlusPayment\CreditPlusPayment'),
				),
				//Do NOT!!! touch this true value
				true
			);
			$this->installMailTemplates(); // Install sORDERForCreditPlusInOrder
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'config')));
		}
		if ( version_compare($sOldVersion, '1.3.0', '<=') ) {
			$oShopware = $this->Application();
			// Add new snippet for payment status 20 finisher
			$oSnippets = $oShopware->Snippets();
			$oTranslation = $oSnippets->getNamespace('frontend/plugins/payment/creditplus');
			$oTranslation->get('FinishTextWaitInorder', 'Die Bestellung ist zwar abgeschlossen, aber Sie müssen jetzt noch auf die Kreditentscheidung warten. Diese wird ihnen gesondert per E-Mail zugestellt.', true);
			$this->updateMailTemplates();
		}
		if ( version_compare($sOldVersion, '1.3.1', '<=') ) {
			$oShopware = $this->Application();
			// Add new snippet for payment status 20 finisher
			$oSnippets = $oShopware->Snippets();
			$oTranslation = $oSnippets->getNamespace('frontend/plugins/payment/creditplus');
			$oTranslation->get('FinishTextWaitInorder', 'Ihre Bestellung ist abgeschlossen. Eine Finanzierungsentscheidung konnte leider noch nicht erfolgen.<br />Sie erhalten von uns innerhalb eines Werktages die Finanzierungsentscheidung per E-Mail mitgeteilt.', true);
			$oTranslation->get('FinishTextToPaymentInorder', 'Ihre Bestellung ist abgeschlossen. Bitte bestätigen Sie Ihre Identität per Klick auf:<br /><br /><a class="sccp-continue-to-payment" href="http://retry.me/">Jetzt Bestätigen!</a><br />', true);
			$this->updateMailTemplates();
		}
		if ( version_compare($sOldVersion, '1.3.2', '<=') ) {
			// Backend updated, rebuild the cache
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('backend', 'config')));
		}
		if ( version_compare($sOldVersion, '1.3.3', '<=') ) {
			// Only code changed in 1.3.4
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.3.4', '<=') ) {
			// Only code changed in 1.3.5
			$this->defaultSuccess();
		}
		if ( version_compare($sOldVersion, '1.3.5', '<=') ) {
		    // Frontend changed in 1.3.6
			$this->createConfig();
			$oShopware = $this->Application();
			$oSnippets = $oShopware->Snippets();
			$oTranslation = $oSnippets->getNamespace('frontend/plugins/payment/creditplus');
			$oTranslation->get('DetailsFinancingCheapestRateModal', '<img class="financing-mini-info--logo" src="###LOGOLINK###"/><span class="financing-mini-info--text">Schon ab ###PRICE### für ###MONTHS### Monate <a href="javascript:void(0);">finanzieren</a>.</span>', true);
			$aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('frontend', 'config')));
		}
		if ( version_compare($sOldVersion, '1.3.6', '<=') ) {
			// Only code changed in 1.3.7
            $this->createConfig(); // Updated sorting numbers on config elements
            $aReturn = $this->addSuccess($aReturn, array('invalidateCache' => array('config'))); // Config was rearranged
		}
		if ( version_compare($sOldVersion, '1.3.7', '<=') ) {
			// Enhanced logging on saving orders
			$this->addLoggingSubscribes();
		}


		// If we were calling the parent, this might run the "install" script.
		return $aReturn;
	}

	/**
	 * Creates a backwards compatible function call
	 *
	 * @param string $name Name of the cronjob
	 * @param string $action Event fired on execution
	 * @param int $interval Seconds between runs
	 * @param int $active Active on creation
	 * @param bool $disableOnError Disable on error - only available after Shopware 5.3.x, will be ignored on older versions
	 */
	public function createCronJob($name, $action, $interval = 86400, $active = 1, $disableOnError = true) {
		if ( $this->existsCronJob($action) ) {
			// CronJob already exists, skip
			return;
		}
		try {
			$oMethod = new \ReflectionMethod('Shopware_Components_Plugin_Bootstrap', 'createCronJob');
			if ( $oMethod->getNumberOfParameters() == 5 ) {
				parent::createCronJob($name, $action, $interval, $active, $disableOnError);
			} else {
				parent::createCronJob($name, $action, $interval, $active);
			}
		} catch ( \ReflectionException $oRE ) {
			parent::createCronJob($name, $action, $interval, $active);
		}
	}

	/**
	 * Checks whether a cronjob already exists.
	 * @param string $sAction CronJob Action called
	 *
	 * @return bool True if cronjob exists, false if not
	 */
	protected function existsCronJob($sAction = '') {
		if ( $sAction === '' ) {
			// Empty = error, don't create a cronjob then
			return true;
		} else {
			$sCronName = $sAction;
			if ( substr($sAction,0,17) !== 'Shopware_CronJob_' ) {
				$sCronName = 'Shopware_CronJob_'.$sAction;
			}
			/** @var \Doctrine\DBAL\Connection $oDB */
			try {
				$oDB = $this->get('dbal_connection');
			} catch (\Exception $oEx ) {
				$oDB = Shopware()->Db();
			}
			if ( $oDB && ( $oDB instanceof \Doctrine\DBAL\Connection ) ) {
				$iCount = $oDB->fetchColumn('SELECT COUNT(*) FROM s_crontab WHERE action = :action', array('action' => $sCronName));
				if ( $iCount > 0 ) { return true; }
			} elseif ( $oDB && ( $oDB instanceof \Enlight_Components_Db_Adapter_Pdo_Mysql ) ) {
				$iCount = $oDB->fetchOne('SELECT COUNT(*) FROM s_crontab WHERE action = :action', array('action' => $sCronName));
				if ( $iCount > 0 ) { return true; }
			}
		}
		return false;
	}


	/**
	 * Deinstallation des Plugins
	 *
	 * @return array|bool
	 *
	 * @throws \Doctrine\ORM\OptimisticLockException If flush does not work
	 */
	public function uninstall() {
		$this->deactivatePayment();
		return $this->defaultSuccess();
	}

	/**
	 * Standard Wert um nach dem Installieren, Deinstallieren, Aktivieren, Deaktivieren die Templates neu zu bauen
	 * @return array
	 */
	private function defaultSuccess() {
		return array('success' => true, 'invalidateCache' => array('frontend', 'template', 'theme'));
	}

	/**
	 * @param array $aReturn
	 * @param array $aAdded
	 *
	 * @return array
	 */
	private function addSuccess( $aReturn, $aAdded ) {
		// Add success with boolean logic
		if ( isset($aAdded['success']) ) {
			$aReturn['success'] = $aReturn['success'] && $aAdded['success'];
		}
		if ( isset($aAdded['invalidateCache']) ) {
			foreach ( $aAdded['invalidateCache'] as $sCacheToClear ) {
				if ( !is_array($aReturn['invalidateCache']) ) { $aReturn['invalidateCache'] = array(); }
				if ( !in_array($sCacheToClear,$aReturn['invalidateCache']) ) {
					$aReturn['invalidateCache'][] = $sCacheToClear;
				}
			}
		}
		return $aReturn;
	}

	/**
	 * Deaktivierung des Plugins
	 *
	 * @return bool|array
	 *
	 * @throws \Doctrine\ORM\OptimisticLockException If flush does not work
	 */
	public function disable() {
		$this->deactivatePayment();
		return $this->defaultSuccess();
	}

	/**
	 * Aktivierung des Plugins
	 *
	 * @return bool|array
	 * @throws \Doctrine\ORM\OptimisticLockException
	 */
	public function enable() {
		$this->activatePayment();
		return $this->defaultSuccess();
	}

	/**
	 * Zahlungsart deaktivieren
	 * @throws \Doctrine\ORM\OptimisticLockException If flush does not work
	 */
	protected function deactivatePayment() {
		$oModelManager = Shopware()->Models();
		/** @var \Shopware\Models\Payment\Repository $oPaymentRepository */
		$oPaymentRepository = $oModelManager->getRepository('Shopware\Models\Payment\Payment');
		/** @var \Shopware\Models\Payment\Payment $oPayment */
		$oPayment = $oPaymentRepository->findOneBy(array('name' => 'sccp_financing'));
		$oPayment->setActive(false);
		$oModelManager->persist($oPayment);
		$oModelManager->flush();
	}

	/**
	 * Zahlungsart aktivieren
	 * @throws \Doctrine\ORM\OptimisticLockException If flush does not work
	 */
	protected function activatePayment() {
		$oModelManager = Shopware()->Models();
		/** @var \Shopware\Models\Payment\Repository $oPaymentRepository */
		$oPaymentRepository = $oModelManager->getRepository('Shopware\Models\Payment\Payment');
		/** @var \Shopware\Models\Payment\Payment $oPayment */
		$oPayment = $oPaymentRepository->findOneBy(array('name' => 'sccp_financing'));
		$oPayment->setActive(true);
		$oModelManager->persist($oPayment);
		$oModelManager->flush();
	}

	/**
	 * Fügt die echten Model Ordner dieses Plugins hinzu
	 */
	protected function registerCustomModels() {
		parent::registerCustomModels();

		Shopware()->Loader()->registerNamespace(
			'SicoCreditPlus\Lib',
			$this->Path() . 'Lib/'
		);
		Shopware()->Loader()->registerNamespace(
			'SicoCreditPlus\Models',
			$this->Path() . 'Models/'
		);
		Shopware()->Loader()->registerNamespace(
			'SicoCreditPlus\Components',
			$this->Path() . 'Components/'
		);

		try {
			$oApplication = $this->Application();
			if ( method_exists($oApplication, 'ModelAnnotations') ) {
				$oApplication->ModelAnnotations()->addPaths(array(
					$this->Path() . 'Lib/',
					$this->Path() . 'Components/'
				));
			}
			// Big bad 4.3.x hack - add our Models to the DriverChain
			$oDriverConfig = Shopware()->Models()->getConfiguration();
			/** @var Doctrine\Common\Persistence\Mapping\Driver\MappingDriverChain $oCacheDriver */
			$oCacheDriver = $oDriverConfig->getMetadataDriverImpl();
			$sDriverClass = get_class($oCacheDriver);
			if ( ($sDriverClass == 'Doctrine\\Common\\Persistence\\Mapping\\Driver\\MappingDriverChain') || ($sDriverClass == 'Doctrine\\ORM\\Mapping\\Driver\\DriverChain') ) {
				$aDrivers = $oCacheDriver->getDrivers();
				list( , $oDriver ) = each( $aDrivers );
				$oCacheDriver->addDriver( $oDriver, 'SicoCreditPlus\\Models\\' );
			}
		} catch ( \Exception $oEx ) {
			// It's probably not a 4.3.x Shop anyway
		}

	}

	/**
	 * Fügt den Tempalte Ordner dieses Plugins hinzu.
	 * @param \Enlight_View_Default $oView
	 */
	protected function registerTemplates($oView = null) {
		if ( $oView === null ) {
			$oView = Shopware()->Template();
		}
		$oView->addTemplateDir(
			$this->Path() . 'Views/'
		);
		if ( version_compare(Shopware()->Config()->get('version'),'5.0.0','<') ) {
			$oView->extendsTemplate('frontend_emotion/account/order_item.tpl');
			$oView->extendsTemplate('frontend_emotion/checkout/cart.tpl');
			$oView->extendsTemplate('frontend_emotion/checkout/confirm_payment.tpl');
			$oView->extendsTemplate('frontend_emotion/checkout/finish.tpl');
			$oView->extendsTemplate('frontend_emotion/detail/data.tpl');
			$oView->extendsTemplate('frontend_emotion/detail/index.tpl');
			$oView->extendsTemplate('frontend_emotion/detail/tabs.tpl');
			$oView->extendsTemplate('frontend_emotion/index/header.tpl');
			$oView->extendsTemplate('frontend_emotion/sicotrigger/dispatch.tpl');
			$oView->extendsTemplate('frontend_emotion/sicotrigger/show_popup.tpl');
		}
	}

	/**
	 * Fügt diversen Frontend Actions weitere Daten über Hooks hinzu
	 */
	protected function addFrontendControllerExtensions() {
		$this->subscribeEvent(
			'Enlight_Controller_Action_PreDispatch_Frontend_Checkout',
			'beforeCheckout'
		);
		$this->subscribeEvent(
			'Enlight_Controller_Action_PostDispatch_Frontend_Detail',
			'afterDetailIndex'
		);
		$this->subscribeEvent(
			'Enlight_Controller_Action_PostDispatch_Frontend_Checkout',
			'afterBasketCart'
		);
		$this->subscribeEvent(
			'Enlight_Controller_Action_PostDispatch_Frontend_Account',
			'afterFrontendAccount'
		);
		$this->subscribeEvent(
			'Theme_Compiler_Collect_Plugin_Less',
			'onCollectLessFiles'
		);
		$this->subscribeEvent(
			'Shopware_Modules_Order_SaveOrder_FilterParams',
			'saveOrderFilterParams'
		);
		$this->subscribeEvent(
			'Shopware_Modules_Order_SendMail_Create',
			'onOrderSendMailCreate'
		);
		$this->subscribeEvent(
			'Shopware_Modules_Admin_GetOpenOrderData_FilterResult',
			'onModulesAdminGetOpenOrderDataFilterResult'
		);
		$this->subscribeEvent(
			'Shopware_Modules_Admin_GetPaymentMeans_DataFilter',
			'onModulesAdminGetPaymentMeansDataFilter'
		);
		$this->subscribeEvent(
			'sBasket::sAddArticle::after',
			'afterBasketAddArticle'
		);
		if ( version_compare($this->Config()->get('version'),'5.0.0','<') ) {
			$this->subscribeEvent(
				'Shopware_Modules_Order_SaveOrder_FilterSQL',
				'filterSaveOrderOrderFilterSQL'
			);
		}
	}

	/**
	 * Fügt diversen Logging Stellen
	 */
	protected function addLoggingSubscribes() {
		$this->subscribeEvent(
			'sOrder::sSaveOrder::before',
			'loggingBeforeSaveOrder'
		);
		$this->subscribeEvent(
			'Shopware_Modules_Order_GetOrdernumber_FilterOrdernumber',
			'loggingAfterOrderNumberIncrement'
		);
	}

	public function loggingBeforeSaveOrder(Enlight_Hook_HookArgs $oHookArgs) {
		/** @var \sOrder $oOrderModule */
		$oOrderModule = $oHookArgs->getSubject();
		$this->registerCustomModels();
		$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$oLogger = $oCPHelper->getLogger();
		$sPaymentID = $oOrderModule->sUserData['additional']['user']['paymentID'];
		if (!empty($oOrderModule->sUserData['additional']['payment']['id'])) {
			$sPaymentID = $oOrderModule->sUserData['additional']['payment']['id'];
		}
		$oModelManager = Shopware()->Models();
		/** @var \Shopware\Models\Payment\Payment $oPayment */
		$oPayment = $oModelManager->find(\Shopware\Models\Payment\Payment::class, $sPaymentID);
		$sBookingID = $oOrderModule->bookingId;
		if ( $oPayment->getName() === 'sccp_financing' ) {
			$sSessionId = Shopware()->Session()->get('sessionId');
			$sTransactionMode = Shopware()->Config()->getByNamespace('SicoCreditPlus','sTransactionMode');
			if ( $sTransactionMode === 'inorder' && !$sBookingID ) {
				$oDB = Shopware()->Db();
				$aBind = array('sessionId' => $sSessionId);

				$oSelect = $oDB->select();
				$oSelect->from(array('o' => 's_order'),array('id' => 'id'))
					->joinLeft(array('ob' => 's_order_basket'),'ob.sessionID = o.temporaryID',[])
					->where('ob.sessionID = :sessionId')
					->limit(1)
					->bind($aBind);

				$oQuery = $oSelect->query();
				if ( $oQuery->execute() ) {
					$aResult = $oQuery->fetch();

					if ( !$aResult || !$aResult['id']) {
						// Wenn der Warenkorb bereits bearbeitet wurde, nichts mehr daran tun.
						// Dann war es auch keiner von unserer Zahlungsart.
						if ( $oLogger ) {
							$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::loggingBeforeSaveOrder', ['sMessage' => 'Warenkorb wurde bereits bearbeitet und steht nicht mehr zur Verfügung.', 'sSessionId' => $sSessionId, 'sTransactionMode' => $sTransactionMode]);
						}
						return;
					}

					/** @var \Shopware\Models\Order\Order $oOrder */
					$oOrder = Shopware()->Models()->find('Shopware\Models\Order\Order', $aResult['id']);
					if ( $oOrder ) {
						$sBookingID = $oOrder->getTransactionId();
						if ( $sBookingID ) {
							$oOrderModule->bookingId = $sBookingID;
						}
					}
				}
			}
			if ( $oLogger ) {
				$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::loggingBeforeSaveOrder', ['sMessage' => 'Vorgang sOrder::sSaveOrder wurde gestartet. Es ist eine CreditPlus Bestellung mit Transaktionsnummer "'.$sBookingID.'".', 'Zugehöriger Basket' => $oOrderModule->sBasketData, 'Benutzer' => $oOrderModule->sUserData]);
			}
		} else {
			if ( $oLogger ) {
				$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::loggingBeforeSaveOrder', ['sMessage' => 'Vorgang sOrder::sSaveOrder wurde gestartet. Es ist keine CreditPlus Bestellung mit Transaktionsnummer "'.$sBookingID.'".', 'Zugehöriger Basket' => $oOrderModule->sBasketData, 'Benutzer' => $oOrderModule->sUserData]);
			}
		}
	}

	public function loggingAfterOrderNumberIncrement(Enlight_Event_EventArgs $oEventArgs) {
		$this->registerCustomModels();
		$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$oLogger = $oCPHelper->getLogger();
		/** @var sOrder $oOrderModule */
		$oOrderModule = $oEventArgs->get('subject');
		$sPaymentID = $oOrderModule->sUserData['additional']['user']['paymentID'];
		if (!empty($oOrderModule->sUserData['additional']['payment']['id'])) {
			$sPaymentID = $oOrderModule->sUserData['additional']['payment']['id'];
		}
		$oModelManager = Shopware()->Models();
		if ( $oLogger ) {
			/** @var \Shopware\Models\Payment\Payment $oPayment */
			$oPayment = $oModelManager->find(\Shopware\Models\Payment\Payment::class, $sPaymentID);
			if ( $oPayment->getName() === 'sccp_financing' ) {
				$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::loggingAfterOrderNumberIncrement', ['sMessage' => 'Bestellnummer bei CreditPlus Bestellung wurde erhöht.', 'CPNummer' => $oOrderModule->bookingId, 'OrderNumber' => $oEventArgs->getReturn()]);
			} else {
				$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::loggingAfterOrderNumberIncrement', ['sMessage' => 'Bestellnummer bei nicht-CreditPlus Bestellung wurde erhöht.', 'Transaktionsnummer' => $oOrderModule->bookingId, 'OrderNumber' => $oEventArgs->getReturn()]);
			}
		}
	}

	protected function addNotifiers() {
		$this->subscribeEvent(
			'SicoCreditPlus_Financing_Status_Changed',
			'onSicoCreditPlusFinancingStatusChanged'
		);
	}

	protected function addCronJobs() {
		$this->createCronJob(
			'CreditPlus Status Wiederholung',
			'Shopware_CronJob_SicoCreditPlusRequestQueue',
			300,
			1,
			true
		);
		$this->subscribeEvent(
			'Shopware_CronJob_SicoCreditPlusRequestQueue',
			'onCronJobRequestQueue'
		);
		$this->createCronJob(
			'CreditPlus Status Fehler',
			'Shopware_CronJob_SicoCreditPlusDailyError',
			86400,
			1,
			true
		);
		$bUpdated = false;
		if ( method_exists($this, 'get') ) {
			/** @var \Doctrine\DBAL\Connection $oDB */
			try {
				$oDB = $this->get('dbal_connection');
			} catch (\Exception $oEx ) {
				$oDB = Shopware()->Db();
			}
			if ( $oDB && ( ($oDB instanceof \Doctrine\DBAL\Connection) || ($oDB instanceof \Enlight_Components_Db_Adapter_Pdo_Mysql) ) ) {
				try {
					$oDB->update('s_crontab',
						array(
							'inform_mail' => '{$sConfig.sMAIL}',
							'inform_template' => 'sCreditPlusRetryErrors',
						),
						array(
							'action' => 'Shopware_CronJob_SicoCreditPlusDailyError'
						)
					);
					$bUpdated = true;
				} catch ( Zend_Db_Adapter_Exception $oEx ) {
					// This will make the $bUpdated remain on false
				}
			}
		}
		if ( $bUpdated === false ) {
			try {
				Shopware()->Db()->query( "UPDATE s_crontab SET inform_mail = ?, inform_template = ? WHERE action = ?", array(
					'{$sConfig.sMAIL}',
					'sCreditPlusRetryErrors',
					'Shopware_CronJob_SicoCreditPlusDailyError'
				) );
			} catch ( \Zend_Db_Adapter_Exception $oZDAE ) {
				// We tried and failed.
			}
		}
		$this->subscribeEvent(
			'Shopware_CronJob_SicoCreditPlusDailyError',
			'onCronJobDailyError'
		);
	}

	protected function updateBackendStaticPaymentStatusSnippets(){
		$oShopware = Shopware();
		$oDB = $oShopware->Db();
		$oSnippets = Shopware()->Snippets();
		$oBackendStaticPaymentStatus = $oSnippets->getNamespace('backend/static/payment_status');
		$aBackendTranslations = array(
			'creditplus_referred' => 'Antrag ausgefüllt',
			'creditplus_accepted' => 'Antrag angenommen',
			'creditplus_declined_soft' => 'Weich abgelehnt',
			'creditplus_declined_hard' => 'Abgelehnt',
			'creditplus_approved' => 'Lieferfreigabe',
			'creditplus_approved_and_dispatched' => 'Ausgeliefert',
			'creditplus_processing_payment' => 'Auszahlung',
			'creditplus_paid' => 'Ausgezahlt',
			'creditplus_cancelled' => 'Storniert',
			'creditplus_error' => 'Fehler',
			'creditplus_docs_received' => 'Warten auf Unterlagen',
			'review_necessary' => 'Überprüfung notwendig'
		);
		foreach($aBackendTranslations as $sName => $sDefault){
			$sCurrentSnippet = $oBackendStaticPaymentStatus->get($sName, $sDefault,true);
			if($sCurrentSnippet == $sName){
				$oDB->exec("DELETE FROM s_core_snippets WHERE name = '{$sName}' AND namespace = 'backend/static/payment_status'");
				$oBackendStaticPaymentStatus->set($sName,$sDefault);
				$oBackendStaticPaymentStatus->write();
			}
		}

	}

	protected function addSnippets() {
		$oSnippets = Shopware()->Snippets();

		// Frontend Translations
		$oTranslation = $oSnippets->getNamespace('frontend/plugins/payment/creditplus');
		$oTranslation->get('sccppangvtext_text','Der Kaufpreis entspricht dem Nettokreditbetrag. Diese Angaben stellen zugleich das 2/3-Beispiel gemäß § 6a Abs. 4 PAngV dar. Kreditvermittlung erfolgt alleine für die CreditPlus Bank AG, Augustenstraße 7, 70178 Stuttgart. Bonität vorausgesetzt.<br /><br />Gilt nur für ausgewählte Produkte.', true);
		$oTranslation->get('DetailsFinancingDescription', 'Bezahlen Sie Ihren Einkauf schnell und bequem über monatlich niedrige Raten mit der CreditPlus Finanzierung. Einfach CreditPlus Finanzierung als Zahlungsart bei der Bestellung auswählen.', true);
		$oTranslation->get('DetailsFinancingCheapestRate', '<img class="financing-mini-info--logo" src="###LOGOLINK###"/><span class="financing-mini-info--text">Schon ab ###PRICE### für ###MONTHS### Monate <a href="javascript:void(0);" onclick="jQuery(\'a[data-tabname=\\\'sico-creditplus\\\']\').click(); jQuery(\'html, body\').animate({scrollTop: jQuery(\'[data-tabname=\\\'sico-creditplus\\\']\').offset().top}, 500);">finanzieren</a>.</span>', true );
		$oTranslation->get('ErrorSicoPaymentExcludedBySession', 'Leider können wir Ihrer Finanzierungsanfrage nicht entsprechen.<br />
Die Entscheidung über Ihre Anfrage basiert auf einer automatisierten Verarbeitung Ihrer personenbezogenen Daten, die der Bewertung einzelner Persönlichkeitsmerkmale dienen.<br />
Bitte wählen Sie eine andere Zahlungsart aus.', true);
		$oTranslation->get('FinishTextToPayment', 'Die Bestellung ist zwar abgeschlossen, aber Sie müssen jetzt noch die Antragsstrecke zur Finanzierung durchgehen. Diese erreichen Sie per Klick auf:<br /><br /><a class="sccp-continue-to-payment" href="http://retry.me/">Jetzt Finanzieren!</a><br />', true);
		$oTranslation->get('FinishTextToPaymentInorder', 'Ihre Bestellung ist abgeschlossen. Bitte bestätigen Sie Ihre Identität per Klick auf:<br /><br /><a class="sccp-continue-to-payment" href="http://retry.me/">Jetzt Bestätigen!</a><br />', true);
		$oTranslation->get('FinishTextWaitInorder', 'Ihre Bestellung ist abgeschlossen. Eine Finanzierungsentscheidung konnte leider noch nicht erfolgen.<br />Sie erhalten von uns innerhalb eines Werktages die Finanzierungsentscheidung per E-Mail mitgeteilt.', true);
		$oTranslation->get('sccpeverythingfinished_text','<p>Vielen Dank für Ihre Finanzierungsanfrage! Damit Sie Ihre Ware schnellstmöglich erhalten, folgen Sie bitte den Anweisungen der CreditPlus Bank.<br />Bitte beachten Sie, dass der Warenversand erst nach Eingang aller erforderlichen Unterlagen bei der CreditPlus Bank erfolgen kann.</p>', true);
		$oTranslation->get('sccpabortinfo_text','<p>Der Finanzierungsantrag wurde von Ihnen abgebrochen. Sie können die Finanzierung jederzeit <a href="http://retry.me/" style="text-decoration: underline;">hier</a> abschließen oder <a href="http://replace.me/" style="text-decoration: underline;">die Zahlungsart ändern</a>.</p>', true);
		$oTranslation->get('ErrorCancelNotAvailable', 'Die Bestellung ist aufgrund des Status bei CreditPlus nicht bei CreditPlus stornierbar.<br />Die Bestellung im Shop wurde jedoch auf den Status storniert gesetzt.', true);
		$oTranslation->get('ErrorCancelAlreadyCancelled', 'Die Bestellung ist bereits storniert.', true);
		$oTranslation->get('missing_billing', 'Rechnungsadresse fehlt.', true );
		$oTranslation->get('missing_billing_firstname', 'Vorname bei Rechnungsadresse fehlt.', true );
		$oTranslation->get('missing_billing_lastname', 'Nachname bei Rechnungsadresse fehlt.', true );
		$oTranslation->get('missing_billing_zipcode', 'PLZ bei Rechnungsadresse fehlt.', true );
		$oTranslation->get('missing_billing_city', 'Stadt bei Rechnungsadresse fehlt.', true );
		$oTranslation->get('missing_billing_street', 'Straße bei Rechnungsadresse fehlt.', true );
		$oTranslation->get('missing_shipping_firstname', 'Vorname bei Lieferadresse fehlt.', true );
		$oTranslation->get('missing_shipping_lastname', 'Nachname bei Lieferadresse fehlt.', true );
		$oTranslation->get('missing_shipping_zipcode', 'PLZ bei Lieferadresse fehlt.', true );
		$oTranslation->get('missing_shipping_city', 'Stadt bei Lieferadresse fehlt.', true );
		$oTranslation->get('missing_shipping_street', 'Straße bei Lieferadresse fehlt.', true );
		$oTranslation->get('sccpmobilehint', 'Es stehen weitere Laufzeiten für die Finanzierung zur Verfügung.', true);

		// Backend Translations
		$aBackendTranslations = array(
			'creditplus_referred' => 'Antrag ausgefüllt',
			'creditplus_accepted' => 'Antrag angenommen',
			'creditplus_declined_soft' => 'Weich abgelehnt',
			'creditplus_declined_hard' => 'Abgelehnt',
			'creditplus_approved' => 'Lieferfreigabe',
			'creditplus_approved_and_dispatched' => 'Ausgeliefert',
			'creditplus_processing_payment' => 'Auszahlung',
			'creditplus_paid' => 'Ausgezahlt',
			'creditplus_cancelled' => 'Storniert',
			'creditplus_error' => 'Fehler',
			'creditplus_docs_received' => 'Warten auf Unterlagen',
			'review_necessary' => 'Überprüfung notwendig'
		);
		$oBackendTranslation = $oSnippets->getNamespace('backend/base/model/payment_status/snippets');
		$oBackendStaticPaymentStatus = $oSnippets->getNamespace('backend/static/payment_status');
		foreach($aBackendTranslations as $sName => $sDefault){
			$oBackendTranslation->get($sName, $sDefault,true);
			$oBackendStaticPaymentStatus->get($sName, $sDefault,true);
		}
	}

	/**
	 * @param Enlight_Event_EventArgs $oEventArgs
	 *
	 * @throws Enlight_Exception If Mail cannot be created
	 * @throws \Doctrine\ORM\ORMException If a Doctrine Database Error occurs
	 * @throws \Doctrine\ORM\OptimisticLockException If flush does not work
	 * @throws \Doctrine\ORM\TransactionRequiredException If multiple statements need to be executed simultaneously
	 */
	public function onSicoCreditPlusFinancingStatusChanged( \Enlight_Event_EventArgs $oEventArgs ) {
		/** @var int|\Shopware\Models\Order\Order $oOrder */
		$oOrder = $oEventArgs->get('oOrder');
		if ( is_int($oOrder) ) {
			$oOrder = Shopware()->Models()->find('Shopware\Models\Order\Order', $oOrder);
		}
		/** @var array $aOrder Order as array */
		$aOrder = $oEventArgs->get('aOrder');
		$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$oContractData = $oCPHelper->getContractData($oOrder);
		$aMail = $this->getStatusMailName($oContractData->state, $oContractData->finallyApproved, $oContractData->deliveryDone);
		if ( $aMail['mail'] ) {
			$sNewStateText = $oCPHelper->getCreditPlusPaymentState($oContractData->state, $oContractData->finallyApproved, $oContractData->deliveryDone, $oContractData->informationRequest);
			$oPaymentState = $oCPHelper->getPaymentStateByName($sNewStateText);
			if ( method_exists($oPaymentState,'getDescription') ) {
				$sNewStateText = $oPaymentState->getDescription();
			} else {
				$oTranslation = Shopware()->Snippets()->getNamespace('backend/base/model/payment_status/snippets');
				$sNewStateText = $oTranslation->get($oPaymentState->getName());
			}
			/** @var \DateTime $oOrderTime */
			$oOrderTime = $aOrder['orderTime'];
			if ( !$oOrderTime ) {
				$oOrderTime = \DateTime::createFromFormat('Y-m-d H:i:s', $aOrder['ordertime']);
			}
			$iOrderTime = $oOrderTime->getTimestamp();
			$aContext = array(
				'aOrder' => $aOrder,
				'sOrderNumber' => $aOrder['number'],
				'sOrderDay' => date('d.m.Y', $iOrderTime),
				'sOrderTime' => date('H:i', $iOrderTime),
				'sccp_newstate' => $sNewStateText,
			);
			$oMail = Shopware()->TemplateMail()->createMail($aMail['mail'], $aContext);
			$oMail->addTo($aMail['recipient']);
			$oMail->send();
		}
	}

	/**
	 * @param string $sNewState New State
	 * @param bool $bFinallyApproved Finally approved set
	 * @param bool $bDeliveryDone Delivery done set
	 *
	 * @return array Indexed array with index 'mail' for the correct template and index 'recipient' for the target
	 */
	protected function getStatusMailName($sNewState = '', $bFinallyApproved = false, $bDeliveryDone = false) {
		$aTarget = array('mail' => '', 'recipient' => '');
		$oConfig = Shopware()->Config();
		$aMatches = array();
		if ( $sNewState == '20' ) {
			$aTarget['mail'] = 'sCreditPlusState20';
			$aTarget['recipient'] = $oConfig->getByNamespace('SicoCreditPlus', 'sEmailState20');
		} elseif ( $sNewState == '24' ) {
			$aTarget['mail'] = 'sCreditPlusState24';
			if ( $bDeliveryDone ) {
				// Delivery mail is not handled yet
				$aTarget['mail'] .= 'd';
				$aTarget['recipient'] = '';
			} elseif ( $bFinallyApproved ) {
				$aTarget['mail'] .= 'a';
				$aTarget['recipient'] = $oConfig->getByNamespace('SicoCreditPlus', 'sEmailState24a');
			} else {
				$aTarget['mail'] .= 'n';
				$aTarget['recipient'] = $oConfig->getByNamespace('SicoCreditPlus', 'sEmailState24a');
			}
		} elseif ( ($sNewState == '21') || ($sNewState == '22') || ($sNewState == '23') || ($sNewState == '25') ) {
			$aTarget['mail'] = 'sCreditPlusState25';
			$aTarget['recipient'] = $oConfig->getByNamespace('SicoCreditPlus', 'sEmailState25');
		} elseif ( ($sNewState == '92') || ($sNewState == '93') ) {
			$aTarget['mail'] = 'sCreditPlusState92';
			$aTarget['recipient'] = $oConfig->getByNamespace('SicoCreditPlus', 'sEmailState92');
		} elseif ( $sNewState == '95' ) {
			$aTarget['mail'] = 'sCreditPlusState95';
			$aTarget['recipient'] = $oConfig->getByNamespace('SicoCreditPlus', 'sEmailState95');
		} elseif ( $sNewState == '99' ) {
			$aTarget['mail'] = 'sCreditPlusState99';
			$aTarget['recipient'] = $oConfig->getByNamespace('SicoCreditPlus', 'sEmailState99');
		}
		if ( ($aTarget['mail'] == '') || ($aTarget['recipient'] == '') || (preg_match_all('/^([A-Za-z0-9]+)([.-_]([A-Za-z0-9]+))*@([A-Za-z]+)([.-]([A-Za-z0-9]+))*[.-]([A-Za-z0-9]+)$/', $aTarget['recipient'], $aMatches) == 0) ) {
			return array('mail' => '', 'recipient' => '');
		}
		return $aTarget;
	}

	/**
	 * Fügt dem Start des Backends unsere Preloader Anweisungen hinzu.
	 * Sorgt dafür, dass die CreditPlus Elemente geladen werden.
	 * @param Enlight_Controller_ActionEventArgs $oEventArgs
	 */
	public function afterBackend(\Enlight_Controller_ActionEventArgs $oEventArgs ) {
		$oSubject = $oEventArgs->getSubject();
		$oRequest = $oEventArgs->getRequest();
		$oResponse = $oEventArgs->getResponse();
		$oView = $oSubject->View();

		if ( !$oRequest->isDispatched() || $oResponse->isException() || !$oView->hasTemplate() ) {
			return;
		}

		if (
			(strpos($oRequest->getRequestUri(),'backend/base') !== false)
			&& (($sFile = $oRequest->getParam('file')) && ($sFile == 'bootstrap' ))
		) {
			// We are playing "Seven up" for the Shop root (where the theme files lie) :)
			$sPluginDir = dirname(__FILE__);
			$sBaseDir = dirname(dirname(dirname(dirname(dirname(dirname($sPluginDir))))));
			$sPaymentStatusFile = $sBaseDir.'/themes/Backend/ExtJs/backend/base/model/payment_status.js';
			if ( is_file($sPaymentStatusFile) && ($sPaymentStatusFileContent = file_get_contents($sPaymentStatusFile)) ) {
				if ( strpos($sPaymentStatusFileContent,'{block name="backend/base/model/payment_status/snippets"}') !== false ) {
					$this->registerTemplates($oView);
					$oView->extendsTemplate('backend/base/model/extends.payment_status.js');
				}
			}
			$sPreloaderFile = $sBaseDir.'/themes/Backend/ExtJs/backend/base/component/Shopware.component.Preloader.js';
			if ( is_file($sPreloaderFile) && ($sPreloaderFileContent = file_get_contents($sPreloaderFile)) ) {
				if ( strpos($sPreloaderFileContent,'{block name="backend/base/component/preloader"}') !== false ) {
					$this->registerTemplates($oView);
					$oView->extendsTemplate('backend/base/component/extends.Shopware.component.Preloader.js');
				} else {
					echo $this->getPreloaderFileContent($sPluginDir);
				}
			} else {
				echo $this->getPreloaderFileContent($sPluginDir);
			}
		}
	}

	/**
	 * Simple function to get the file content prepared for echo in function afterBackend
	 *
	 * @param string $sPluginDir File Path to this plugin
	 *
	 * @return string File Content without Smarty tags
	 *
	 * @see afterBackend
	 */
	private function getPreloaderFileContent( $sPluginDir ) {
		$sContent = file_get_contents($sPluginDir.'/Views/backend/base/component/extends.Shopware.component.Preloader.js');

		$sContent = str_replace('//{extends file="parent:backend/base/component/Shopware.component.Preloader.js"}','',$sContent);
		$sContent = str_replace('//{block name="backend/base/component/preloader" append}','',$sContent);
		$sContent = str_replace('//{/block}','',$sContent);

		return $sContent;
	}

	/**
	 * Fügt Backend Menüs für CreditPlus hinzu
	 */
	private function addBackendMenu() {

		// Menüstruktur

		// Vaterpunkt im Backend - führt direkt auf Produktgruppen
		$oParentItem = $this->createMenuItem(array(
			'label' => 'CreditPlus Finanzierung',
			'controller' => 'SicoCreditPlusProductgroups',
			'class' => 'sprite-credit-cards',
			'action' => 'index',
			'active' => 1,
			'parent' => $this->Menu()->findOneBy(array('label' => 'Einstellungen'))
		));

		// Produktgruppen
		$this->createMenuItem(array(
			'label' => 'Produktgruppen',
			'controller' => 'SicoCreditPlusProductgroups',
			'class' => 'sprite-blue-folders-stack',
			'action' => 'index',
			'active' => 1,
			'parent' => $oParentItem
		));

		// Zinstabelle
		$this->createMenuItem(array(
			'label' => 'Zinstabelle',
			'controller' => 'SicoCreditPlusOfferedOptions',
			'class' => 'sprite-ui-scroll-pane-list',
			'action' => 'index',
			'active' => 1,
			'parent' => $oParentItem
		));
	}

	/**
	 * Fügt diverse Hook Events hinzu, welche für das Backend relevant sind
	 */
	private function addBackendControllerExtensions() {
		$this->subscribeEvent(
			'Enlight_Controller_Action_PostDispatch_Backend_Base',
			'afterBackend'
		);

		$this->subscribeEvent(
			'Enlight_Controller_Action_PreDispatch_Backend_Order',
			'beforeBackendOrder'
		);

		$this->subscribeEvent(
			'Enlight_Controller_Action_PostDispatch_Backend_Order',
			'afterBackendOrder'
		);

		$this->subscribeEvent(
			'Enlight_Controller_Action_PostDispatch_Backend_PluginManager',
			'afterPluginManager'
		);

		$this->subscribeEvent(
			'Enlight_Controller_Action_PreDispatch_Backend_Config',
			'beforeBackendConfig'
		);
	}

	/**
	 * @param Enlight_Controller_ActionEventArgs $oArgs
	 *
	 * @throws Zend_Cache_Exception
	 */
	public function beforeBackendConfig( \Enlight_Controller_ActionEventArgs $oArgs ) {
		/** @var \Enlight_Controller_Request_RequestHttp $oRequest */
		$oRequest = $oArgs->getRequest();
		$sAction = $oRequest->getActionName();
		if ( $sAction == 'saveForm' ) {
			$sPluginName = $oRequest->getParam('name');
			if ( $sPluginName == 'SicoCreditPlus' ) {
				if ( version_compare(Shopware()->Config()->get('version'),'4.3.0','>=') ) {
					$this->clearCache430($oRequest);
				} else {
					$this->clearCache400($oRequest);
				}
			}
		}
	}

	/**
	 * @param Enlight_Controller_Request_RequestHttp $oRequest
	 */
	private function clearCache430($oRequest) {
		$aSuccess = array('success' => true, 'invalidateCache' => array('config'), 'elements' => array() );
		/** @var array[] $aElements */
		$aElements = $oRequest->getParam('elements');
		/** @var Shopware_Components_Config $oConfig */
		$oConfig = $this->get('config');
		/** @var \Shopware\Components\Model\ModelManager $oModelManager */
		$oModelManager = $this->get('models');
		/** @var \Shopware\Models\Shop\Repository $oShopRepository */
		$oShopRepository = $oModelManager->getRepository('Shopware\Models\Shop\Shop');
		$oDefaultShop = $oShopRepository->getDefault();
		$iShopID = $oDefaultShop->getId();
		foreach ( $aElements as $aElement ) {
			if ( in_array($aElement['name'],array('sTransactionMode','sCPShownAs','bShowDetails','bShowBasket','bShowPayment')) ) {
				// If new value != old value for current shop
				foreach ( $aElement['values'] as $aValue ) {
					if ( $iShopID == $aValue['shopId'] ) {
						if ( $aValue['value'] != $oConfig->getByNamespace('SicoCreditPlus', $aElement['name']) ) {
							$aSuccess = $this->addSuccess($aSuccess, array( 'invalidateCache' => array('template', 'theme') ) );
						}
					}
				}
			}
		}
		/** @var \Shopware\Components\CacheManager $oCacheManager */
		$oCacheManager = $this->get('shopware.cache_manager');
		foreach ( $aSuccess['invalidateCache'] as $sCacheName ) {
			$sClearCacheFunction = 'clear'.ucfirst($sCacheName).'Cache';
			if ( method_exists($oCacheManager,$sClearCacheFunction) ) {
				// Translates to $oCacheManager->clearTemplateCache() and other functions
				$oCacheManager->$sClearCacheFunction();
			}
		}
	}

	/**
	 * @param Enlight_Controller_Request_RequestHttp $oRequest
	 *
	 * @throws Zend_Cache_Exception
	 */
	private function clearCache400($oRequest) {
		$aSuccess = array('success' => true, 'invalidateCache' => array('Shopware_Config', 'Shopware_Plugin'), 'elements' => array() );
		/** @var array[] $aElements */
		$aElements = $oRequest->getParam('elements');
		/** @var Shopware_Components_Config $oConfig */
		$oConfig = Shopware()->Config();
		/** @var \Shopware\Components\Model\ModelManager $oModelManager */
		$oModelManager = Shopware()->Models();
		/** @var \Shopware\Models\Shop\Repository $oShopRepository */
		$oShopRepository = $oModelManager->getRepository('Shopware\Models\Shop\Shop');
		$oDefaultShop = $oShopRepository->getActiveDefault();
		$iShopID = $oDefaultShop->getId();
		foreach ( $aElements as $aElement ) {
			if ( in_array($aElement['name'],array('sTransactionMode','sCPShownAs','bShowDetails','bShowBasket','bShowPayment')) ) {
				// If new value != old value for current shop
				foreach ( $aElement['values'] as $aValue ) {
					if ( $iShopID == $aValue['shopId'] ) {
						if ( $aValue['value'] != $oConfig->getByNamespace('SicoCreditPlus', $aElement['name']) ) {
							$aSuccess = $this->addSuccess($aSuccess, array( 'invalidateCache' => array('template', 'theme') ) );
						}
					}
				}
			}
		}
		$oShopware = Shopware();
		if ( method_exists($oShopware, 'Cache') ) {
			/** @var Zend_Cache_Core $oCacheManager */
			$oCacheManager = $oShopware->Cache();
			$oCacheManager->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, $aSuccess['invalidateCache']);
		}
	}

	/**
	 * Diese Funktion baut auf dem Detailseiten Template die Finanzierungstabelle, sowie die günstigste Rate zusammen.
	 *
	 * @param Enlight_Controller_ActionEventArgs $args
	 *
	 * @throws Enlight_Exception
	 * @throws \Doctrine\ORM\NonUniqueResultException
	 */
	public function afterDetailIndex(\Enlight_Controller_ActionEventArgs $args) {
		/** @var $oController Shopware_Controllers_Frontend_Detail */
		$oController = $args->getSubject();
		$oRequest = $args->getRequest();
		$oResponse = $args->getResponse();
		$oView = $oController->View();
		$aArticle = $oView->sArticle;

		// Standard Shopware Ausschlüsse
		if (!$oRequest->isDispatched() || $oResponse->isException() || !$oView->hasTemplate()) {
			return;
		}
		$this->registerCustomModels();
		$this->registerTemplates($oView);

		/** @var \Shopware\Models\Article\Repository $oArticleRepository */
		$oArticleRepository = Shopware()->Models()->getRepository('\Shopware\Models\Article\Detail');
		$oArticleDetailsQuery = $oArticleRepository->getDetailsByIdsQuery($aArticle['articleDetailsID']);

		/** @var \Shopware\Models\Article\Detail $oArticleDetails */
		$oArticleDetails = $oArticleDetailsQuery->getOneOrNullResult(\Doctrine\ORM\AbstractQuery::HYDRATE_OBJECT);
		$oHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();

		$oWSCurrency = $oHelper->getWebshopCurrency();

		if ( $aArticle['price'] ) {
			$dPrice = $this->reFloatPrice($aArticle['price']);
			$aMonths = $oHelper->getFinancingMonths($oArticleDetails, null, $dPrice);
		} else {
			$aMonths = $oHelper->getFinancingMonths($oArticleDetails);
		}

		$oTranslation = Shopware()->Snippets()->getNamespace('frontend/plugins/payment/creditplus');
		$sFinancingDescription = $oTranslation->get('DetailsFinancingDescription');

		$oConfig = Shopware()->Config();
		$aMarker = array(
			'###MINDESTBESTELLWERT###',
			'###JAHRESZINS###'
		);
		$sMinimalOrderSum = number_format($oConfig->getByNamespace('SicoCreditPlus', 'sMinBasketPrice'), $oWSCurrency->decimals, $oWSCurrency->decimalSeparator, $oWSCurrency->thousandSeparator);
		$dCheapestInterestRate = $oHelper->getCheapestInterestRate($aMonths);
		$aReplacements = array(
			($oWSCurrency->side == 'left')?$oWSCurrency->sign.' '.$sMinimalOrderSum:$sMinimalOrderSum.' '.$oWSCurrency->sign,
			number_format($dCheapestInterestRate, $oWSCurrency->decimals, $oWSCurrency->decimalSeparator, $oWSCurrency->thousandSeparator)
		);

		$sFinancingDescription = str_replace($aMarker, $aReplacements, $sFinancingDescription);
		$bShowPAngVText = ($oTranslation->get('sccppangvtext_text') != '');
		$bShowDetails = $oConfig->getByNamespace('SicoCreditPlus', 'bShowDetails');

		$oCheapestMonth = $oHelper->getCheapestRate($aMonths);

		$iRateTableShowMode = intval($oConfig->getByNamespace('SicoCreditPlus', 'sRateTableMode'));
		if ( $iRateTableShowMode == 1 ) {
			$sCheapestRateText = $oTranslation->get('DetailsFinancingCheapestRateModal');
		} else {
			$sCheapestRateText = $oTranslation->get('DetailsFinancingCheapestRate');
		}

		$aCheapestMarker = array(
			'###PRICE###',
			'###MONTHS###',
			'###LOGOLINK###'
		);
		$sBasePath = $oRequest->getBasePath();
		if ( substr($sBasePath,-1) !== '/' ) {
			$sBasePath .= '/';
		}
		$aCheapestReplacements = array(
			$oCheapestMonth->monthlyRate,
			$oCheapestMonth->months,
			$sBasePath.'engine/Shopware/Plugins/Community/Frontend/SicoCreditPlus/Views/frontend/_public/src/img/creditplus_logo_small.png'
		);
		$sCheapestRateText = str_replace($aCheapestMarker, $aCheapestReplacements, $sCheapestRateText);
		if ( $bShowDetails ) {
			$oView->Template()->assign('aSicoRateTable', $aMonths);
			$oView->Template()->assign('sSicoFinancingDescription', $sFinancingDescription);
			$oView->Template()->assign('sSicoCheapestRateText', $aMonths?$sCheapestRateText:'');
			$oView->Template()->assign('bShowPAngVText', $bShowPAngVText);
			$oView->Template()->assign('iRateTableMode', $iRateTableShowMode);
		} else {
			$oView->Template()->assign('aSicoRateTable', array());
			$oView->Template()->assign('sSicoFinancingDescription', '');
			$oView->Template()->assign('sSicoCheapestRateText', '');
			$oView->Template()->assign('bShowPAngVText', false);
			$oView->Template()->assign('iRateTableMode', $iRateTableShowMode);
		}
	}

	/**
	 * Converts the string based price into a float/double value.
	 *
	 * @param string $sPrice
	 *
	 * @return double
	 */
	protected function reFloatPrice( $sPrice = '' ) {
		$dReturn = 0.00;
		if ( (is_string($sPrice)) && ($sPrice != '') ) {
			if ( substr_count($sPrice, ',') == 1 ) {
				$aPrice = explode(',', $sPrice);
			} elseif ( substr_count($sPrice, '.') == 1 ) {
				$aPrice = explode('.', $sPrice);
			} else {
				$aPrice = explode(',', $sPrice);
			}
			$iLength = count($aPrice);
			$sNewPrice = '';
			foreach ( $aPrice as $iPricePart => $sPricePart ) {
				if ( $iPricePart == ($iLength-1) ) {
					$sNewPrice .= '.';
				}
				$sNewPrice .= preg_replace('/([^0-9]+)/','', $sPricePart);
			}
			$dReturn = (float)$sNewPrice;
		}
		return $dReturn;
	}

	/**
	 * PreDispatch Hook, der verhindert, dass eine nicht "bezahlte" Bestellung im Fall "vor Bestellabschluss" durchgeht.
	 *
	 * @param Enlight_Controller_ActionEventArgs $oEventArgs
	 *
	 * @throws Zend_Db_Statement_Exception If the database fails
	 * @throws \Doctrine\ORM\ORMException Unspecified exception in ORM
	 * @throws \Doctrine\ORM\OptimisticLockException If flush fails
	 * @throws \Doctrine\ORM\TransactionRequiredException If flush requires more statements to be executed simultaneously*@throws Exception
	 * @throws Exception If a redirect exception occurs
	 */
	public function beforeCheckout( \Enlight_Controller_ActionEventArgs $oEventArgs ) {
		$oController = $oEventArgs->getSubject();
		$oView = $oController->View();
		$oRequest = $oEventArgs->getRequest();
		$oResponse = $oEventArgs->getResponse();
		$sActionName = $oRequest->getActionName();

		// Standard Shopware Ausschlüsse
		if (!$oRequest->isDispatched() || $oResponse->isException() || !$oView->hasTemplate()) {
			return;
		}

		//in chrome the iframe has another session cookie than the outer side.
		//If the user will returned from the contract Input than this means the contract is declined
		//
		$referrer = $oRequest->getHeader('referer');
		if(is_string($referrer) && $sActionName != 'finish'){
			if(strpos($referrer,'sicotrigger/dispatch/data') !== false){
				Shopware()->Session()->offsetSet('bSicoFinancingExcluded', true);
			}
		}

		$sSessionId = Shopware()->Session()->get('sessionId');
		$sTransactionMode = Shopware()->Config()->getByNamespace('SicoCreditPlus','sTransactionMode');

		$this->registerCustomModels();
		$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$oLogger = $oCPHelper->getLogger();
		if ( $sActionName === 'finish' ) {
			if ( $oLogger ) {
				$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::beforeCheckout', ['sMessage' => 'Logging vor dem Return wenn Fehler existieren.','sSessionId' => $sSessionId, 'sTransactionMode' => $sTransactionMode]);
			}
		}

		if ( ($sActionName !== 'finish') || ($sSessionId === null) || ($sTransactionMode == 'postorder') ) {
			// Gehört nicht zum Abschluss oder wird in einen Fehler laufen oder Zahlung nach Abschluss
			return;
		}
		$oDB = Shopware()->Db();
		$aBind = array('sessionId' => $sSessionId);

		$oSelect = $oDB->select();
		$oSelect->from(array('o' => 's_order'),array('id' => 'id'))
				->joinLeft(array('ob' => 's_order_basket'),'ob.sessionID = o.temporaryID',[])
				->where('ob.sessionID = :sessionId')
				->limit(1)
				->bind($aBind);

		$oQuery = $oSelect->query();
		if ( $oQuery->execute() ) {
			$aResult = $oQuery->fetch();

			if ( !$aResult || !$aResult['id']) {
				// Wenn der Warenkorb bereits bearbeitet wurde, nichts mehr daran tun.
				// Dann war es auch keiner von unserer Zahlungsart.
				if ( $oLogger ) {
					$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::beforeCheckout', ['sMessage' => 'Warenkorb wurde bereits bearbeitet und steht nicht mehr zur Verfügung.', 'sSessionId' => $sSessionId, 'sTransactionMode' => $sTransactionMode]);
				}
				return;
			}

			/** @var \Shopware\Models\Order\Order $oOrder */
			$oOrder = Shopware()->Models()->find('Shopware\Models\Order\Order', $aResult['id']);

			// Wenn Bestellung mit sccp_financing gezahlt wird, der Modus auf "Vor Bestellabschluss" steht (siehe oben),
			// dann TransaktionsID prüfen und wenn diese leer ist oder der Vertrag noch nicht ausgefüllt wurde:
			// Weiterleiten auf Zahlungsseite.
			if ( $oOrder->getPayment()->getName() == 'sccp_financing' ) {
				// Unsere Klassen anmelden
				$this->registerCustomModels();
				if ( $oOrder->getTransactionId() == '' ) {
					$oOrder->setTransactionId('CP'.date('YmdHis'));
					Shopware()->Models()->flush($oOrder);
					if ( $oLogger ) {
						$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::beforeCheckout', ['sMessage' => 'Weiterleitung zur Zahlungsseite (neue CP Nummer '.$oOrder->getTransactionId().')', 'sSessionId' => $sSessionId, 'sTransactionMode' => $sTransactionMode]);
					}
					$this->redirectToPaymentPage($oOrder, $oController);
				} else {
					$oContractData = $oCPHelper->getContractData($oOrder);
					if ( $oContractData->state == 0 ) {
						if ( $oLogger ) {
							$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::beforeCheckout', ['sMessage' => 'Weiterleitung zur Zahlungsseite (bestehende CP Nummer '.$oOrder->getTransactionId().')', 'sSessionId' => $sSessionId, 'sTransactionMode' => $sTransactionMode]);
						}
						$this->redirectToPaymentPage($oOrder, $oController);
					} else {
						if ( $oLogger ) {
							$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::beforeCheckout', ['sMessage' => 'Keine Weiterleitung (bestehende CP Nummer '.$oOrder->getTransactionId().' mit Status '.$oContractData->state.')', 'sSessionId' => $sSessionId, 'sTransactionMode' => $sTransactionMode]);
						}
					}
				}
			} else {
				if ( $oLogger ) {
					$oLogger->writeRequestInformation(date('Y-m-d H:i:s'), false, 'Bootstrap::beforeCheckout', ['sMessage' => 'Zahlungsart ist nicht CreditPlus Finanzierung', 'sSessionId' => $sSessionId, 'sTransactionMode' => $sTransactionMode]);
				}
			}
		}
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 * @param \Enlight_Controller_Action|\Shopware_Controllers_Frontend_Checkout $oController
	 * @throws Exception
	 */
	private function redirectToPaymentPage( $oOrder, $oController ) {
		$sPaymentAction = 'showPopup';
		$sPaymentActionConfig = Shopware()->Config()->getByNamespace('SicoCreditPlus', 'sCPShownAs');
		if ( $sPaymentActionConfig == 'iframe' ) {
			$sPaymentAction = 'showIframe';
		} elseif ( $sPaymentActionConfig == 'new_window' ) {
			$sPaymentAction = 'showNewWindow';
		}
		Shopware()->Session()->offsetSet('SicoLastOrder', $oOrder);
		$oController->redirect(array(
			'controller' => 'sicotrigger',
			'action' => $sPaymentAction,
			'cporder' => $oOrder->getId(),
			'don' => $oOrder->getTransactionId()
		));
	}

	/**
	 * Fügt dem Template des Warenkorbs und der Bezahlung die fallbasierte Ratentabelle hinzu
	 * Wenn die action "finish" ist, wird der Link auf dem Button "Jetzt Finanzieren" zusammengebaut.
	 * Der Text für die Abschlussseite befindet sich in der Konstante "FinishTextToPayment"
	 *
	 * @param Enlight_Controller_ActionEventArgs $args
	 *
	 * @throws Enlight_Exception
	 */
	public function afterBasketCart(\Enlight_Controller_ActionEventArgs $args) {
		$this->registerCustomModels();
		/** @var $oController Shopware_Controllers_Frontend_Checkout */
		$oController = $args->getSubject();
		$oRequest = $oController->Request();
		$oView = $oController->View();
		$this->registerTemplates($oView);
		$aBasket = $oView->sBasket;
		$bHasTemplate = false;
		if ( $oView ) {
			$bHasTemplate = $oView->hasTemplate();
		}
		if ( $aBasket && $aBasket['content'] ) {
			$aArticleNumbers = array();
			foreach ( $aBasket['content'] as $aArticle ) {
				$aArticleNumbers[] = $aArticle['ordernumber'];
			}

			$oQueryBuilder = Shopware()->Models()->createQueryBuilder();
			/** @var \Doctrine\ORM\Query\Expr\Base|\Doctrine\ORM\Query\Expr\Func $oWhere */
			$oWhere = $oQueryBuilder->expr()->in('detail.number','?1');
			$oQueryBuilder->select('detail')
					->from('\Shopware\Models\Article\Detail','detail')
					->add('where', $oWhere)
					->setParameters(array('1' => $aArticleNumbers));
			$oQuery = $oQueryBuilder->getQuery();

			$aArticles = $oQuery->execute(null, \Doctrine\ORM\AbstractQuery::HYDRATE_OBJECT);
			$oHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();

			$oReference = $oHelper->getFinancingArticleReference(null, $aArticles, $aBasket['sAmount']);
			$aMonths = $oHelper->getFinancingMonths($aArticles[$oReference->id], null, $aBasket['sAmount']);
			$oConfig = Shopware()->Config();

			$oTranslation = Shopware()->Snippets()->getNamespace('frontend/plugins/payment/creditplus');

			$bShowPAngVText = ($oTranslation->get('sccppangvtext_text') != '');
			$bShowBasket = $oConfig->getByNamespace('SicoCreditPlus', 'bShowBasket');
			$sTransactionMode = $oConfig->getByNamespace('SicoCreditPlus', 'sTransactionMode');
			$bShowPayment = $oConfig->getByNamespace('SicoCreditPlus', 'bShowPayment');
			$bPaymentAllowed = !$oHelper->isPaymentExcluded();
			$bShowBasket = $bShowBasket && $bPaymentAllowed;
			$bShowPayment = $bShowPayment && $bPaymentAllowed;

			if ( $bHasTemplate ) {
				if ( $bShowBasket || $bShowPayment ) {
					$oView->Template()->assign('bShowBasket', $bShowBasket);
					$oView->Template()->assign('bShowPayment', $bShowPayment);
					$oView->Template()->assign('aSicoRateTable', $aMonths);
					$oView->Template()->assign('bShowPAngVText', $bShowPAngVText);
				} else {
					$oView->Template()->assign('aSicoRateTable', array());
					$oView->Template()->assign('bShowPAngVText', false);
				}
				if ( $oRequest->getActionName() == 'shippingPayment' ) {
					$aErrorMessages = $oView->Template()->getVariable('sErrorMessages')->value;
					if ( !$aErrorMessages ) { $aErrorMessages = array(); }
					if ( $oHelper->isPaymentExcludedBySession() ) {
						$aErrorMessages[] = $oTranslation->get('ErrorSicoPaymentExcludedBySession');
					}
					$oView->Template()->assign('sErrorMessages', $aErrorMessages);
				}
			}

			if ( ($oRequest->getActionName() == 'finish') && ($sTransactionMode == 'postorder') ) {
				$sTransactionID = null;
				$sOrderNumber = $oView->sOrderNumber;

				// Hole die Bestellung aus der Datenbank um die TransactionID zu erhalten...
				/** @var \Shopware\Models\Order\Repository $oOrderRepository */
				$oOrderRepository = Shopware()->Models()->getRepository('\Shopware\Models\Order\Order');
				/** @var \Shopware\Models\Order\Order $oDBOrder */
				$oDBOrder = $oOrderRepository->findOneBy(array('number' => $sOrderNumber));
				if ( $oDBOrder ) {
					$sTransactionID = $oDBOrder->getTransactionId();
				}
				// Wenn Transaktions ID existiert und unser Format hat
				if ( $sOrderNumber && $sTransactionID && preg_match('/^CP[1-9]([0-9]+)$/', $sTransactionID) ) {
					// Ziel für URL = dieser Controller mit anderer Action
					$aTarget = array(
						'controller' => 'sicotrigger',
						'action' => 'showPopup',
						'cporder' => $sOrderNumber,
						'don' => $sTransactionID,
					);
					$sShownAs = Shopware()->Config()->getByNamespace('SicoCreditPlus','sCPShownAs');
					if ( $sShownAs == 'iframe' ) {
						$aTarget['action'] = 'showIframe';
					} elseif ( $sShownAs == 'new_window' ) {
						$aTarget['action'] = 'showNewWindow';
					}
					$oRouter = Shopware()->Front()->Router();
					$sPayNowURL = $oRouter->assemble($aTarget);

					$aTarget['action'] = 'cancelAndReorder';
					$sCancelURL = $oRouter->assemble($aTarget);

					$oTranslation = Shopware()->Snippets()->getNamespace('frontend/plugins/payment/creditplus');

					$sSicoFinishText = $oTranslation->get('FinishTextToPayment');
					$sSicoFinishText = str_replace('http://retry.me/', $sPayNowURL, $sSicoFinishText);
					$sSicoFinishText = str_replace('http://replace.me/', $sCancelURL, $sSicoFinishText);

					$oView->assign('sSicoFinishText', $sSicoFinishText);
				}
			} elseif ( $oRequest->getActionName() == 'finish' ) {
				if ( $sTransactionMode === 'inorder' ) {
					$sTransactionID = null;
					$sOrderNumber = $oView->sOrderNumber;

					// Hole die Bestellung aus der Datenbank um die TransactionID zu erhalten...
					/** @var \Shopware\Models\Order\Repository $oOrderRepository */
					$oOrderRepository = Shopware()->Models()->getRepository('\Shopware\Models\Order\Order');
					/** @var \Shopware\Models\Order\Order $oDBOrder */
					$oDBOrder = $oOrderRepository->findOneBy(array('number' => $sOrderNumber));
					if ( $oDBOrder ) {
						$sTransactionID = $oDBOrder->getTransactionId();
					}
					if ( !$sTransactionID || !preg_match('/^CP[1-9]([0-9]+)$/', $sTransactionID) ) {
						return;
					}
					if ( isset($this->aLastOrderData['order_id']) && isset($this->aLastOrderData['transaction_id']) ) {
						$this->moveOldOrderDataToNewOrder($this->aLastOrderData['order_id'], $this->aLastOrderData['transaction_id'], $oDBOrder->getId(), $oDBOrder->getTransactionId());
					}
					// Ziel für URL = dieser Controller mit anderer Action
					$aTarget = array(
						'controller' => 'sicotrigger',
						'action' => 'showPopup',
						'cporder' => $sOrderNumber,
						'don' => $sTransactionID,
					);
					$sShownAs = Shopware()->Config()->getByNamespace('SicoCreditPlus','sCPShownAs');
					if ( $sShownAs == 'iframe' ) {
						$aTarget['action'] = 'showIframe';
					} elseif ( $sShownAs == 'new_window' ) {
						$aTarget['action'] = 'showNewWindow';
					}
					$oRouter = Shopware()->Front()->Router();
					$sPayNowURL = $oRouter->assemble($aTarget);
					$oTranslation = Shopware()->Snippets()->getNamespace('frontend/plugins/payment/creditplus');
					$sSicoFinishText = $oTranslation->get('FinishTextToPaymentInorder');
					$sSicoFinishText = str_replace('http://retry.me/', $sPayNowURL, $sSicoFinishText);
					$oCPHelper = new SicoCreditPlus\Components\SicoCreditPlusHelper();
					if ( $oContract = $oCPHelper->getContractData($oDBOrder) ) {
						if ( in_array(((int)$oContract->state), array(20,92,93)) ) {
							$sSicoFinishText = $oTranslation->get('FinishTextWaitInorder');
						}
					}
					$oView->assign('sSicoFinishText', $sSicoFinishText);
				}
			}
		}
	}

	/**
	 * Ekelhafter Hack um in 4.3.x die Transaktions ID in der Datenbank speichern zu können
	 *
	 * @param Enlight_Event_EventArgs $oEventArgs
	 */
	public function filterSaveOrderOrderFilterSQL( Enlight_Event_EventArgs $oEventArgs ) {
		$sStatement = trim($oEventArgs->getReturn());
		/** @var sOrder $oOrderHandler */
		$oOrderHandler = $oEventArgs->get('subject');
		if ( !$oOrderHandler->bookingId ) {
			// Find fields, expecting them to be between the first round brackets
			$iStartOfFields = strpos($sStatement,'(')+1;
			$iEndOfFields = strpos($sStatement, ')', $iStartOfFields)-$iStartOfFields-1;
			$sFields = trim(substr($sStatement, $iStartOfFields, $iEndOfFields));
			$aFields = explode(',', $sFields);

			// Send data to memory, and read it as CSV
			$oMemoryCSV = fopen('php://memory', 'rw');
			$iStartOfData = strpos($sStatement, '(', $iEndOfFields)+1;
			$iEndOfData = strlen($sStatement)-$iStartOfData-1;
			$sData = trim(substr($sStatement, $iStartOfData, $iEndOfData));
			$aData = explode("\n",$sData);
			$sData = implode('',$aData);
			fputs($oMemoryCSV, $sData);
			fseek($oMemoryCSV, 0);
			$aValues = fgetcsv($oMemoryCSV, 8192, ',', "'", '\\' );
			fclose($oMemoryCSV);

			// Call already working function further down
			$oEventArgsForFilter = new Enlight_Event_EventArgs();
			$aOrder = array();
			foreach ( $aFields as $iKey => $sField ) {
				$aOrder[trim($sField)] = trim($aValues[$iKey]);
			}
			$oEventArgsForFilter->setReturn($aOrder);
			$this->saveOrderFilterParams($oEventArgsForFilter);
			$aOrder = $oEventArgsForFilter->getReturn();

			$oDB = Shopware()->Db();
			$sFields = implode(', ', array_keys($aOrder));
			$sValues = '';
			foreach ( $aOrder as $sValue ) {
				if ( $sValues ) {
					$sValues .= ', ';
				}
				if ( $sValue == 'now()' ) {
					$sValues .= 'now()';
				} elseif ( $sValue === '' ) {
					$sValues .= "''";
				}  elseif ( is_numeric($sValue) ) {
					$sValues .= $sValue;
				} else {
					$sValues .= $oDB->quote($sValue);
				}
			}
			/** @noinspection SqlInsertValues */
			$sStatement = "INSERT INTO s_order ($sFields) VALUES ($sValues)";

			$oEventArgs->setReturn($sStatement);
		}
	}

	/**
	 * Fügt der Bestellung die TransaktionsID hinzu,
	 * wenn die Zahlungsart sccp_financing ist
	 * und noch keine TransaktionsID gesetzt wurde.
	 *
	 * @param \Enlight_Event_EventArgs $oEventArgs
	 */
	public function saveOrderFilterParams( \Enlight_Event_EventArgs $oEventArgs ) {
		/** @var array $aOrder */
		$aOrder = $oEventArgs->getReturn();
		if ( $aOrder['paymentID'] ) {
			$iPaymentID = $aOrder['paymentID'];
			/** @var \Shopware\Models\Payment\Repository $oPaymentRepository */
			$oPaymentRepository = Shopware()->Models()->getRepository('\Shopware\Models\Payment\Payment');
			/** @var \Shopware\Models\Payment\Payment $oPayment */
			$oPayment = $oPaymentRepository->find($iPaymentID);
			if ( $oPayment->getName() == 'sccp_financing' ) {
				if ( Shopware()->Config()->getByNamespace('SicoCreditPlus', 'sTransactionMode') == 'inorder' ) {
					// "Vor Bestellabschluss" setzt Order in Session mit Daten
					/** @var \Shopware\Models\Order\Order $oOrder */
					$oOrder = Shopware()->Session()->offsetGet('SicoLastOrder');
					if ( $oOrder ) {
						// Wenn Bestellung in Session vorhanden, diese nehmen und Daten von dieser übetragen
						$this->registerCustomModels();
						$this->aLastOrderData['order_id'] = $oOrder->getId();
						$this->aLastOrderData['transaction_id'] = $oOrder->getTransactionId();
						$aOrder['transactionID'] = $oOrder->getTransactionId();
						$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
						$oContractData = $oCPHelper->getContractData($oOrder);
						if ( $oContractData->state != '0' ) {
							$sState = $oCPHelper->getCreditPlusPaymentState($oContractData->state, $oContractData->finallyApproved, $oContractData->deliveryDone, $oContractData->informationRequest);
							$oState = $oCPHelper->getPaymentStateByName($sState);
							$aOrder['cleared'] = $oState->getId();
						}
					} else {
						// Wenn Bestellung nicht mehr in Session vorhanden, Status auf "zu Prüfen" setzen
						$aOrder['cleared'] = 21;
						if ( defined('Shopware\Models\Order\Status::PAYMENT_STATE_REVIEW_NECESSARY') ) {
							$aOrder['cleared'] = \Shopware\Models\Order\Status::PAYMENT_STATE_REVIEW_NECESSARY;
						}
					}
				} else {
					// "Nach Bestellabschluss" setzt Transaktionsnummer erst beim Speichern
					$aOrder['transactionID'] = 'CP'.date( 'YmdHis' );
				}
				$oEventArgs->setReturn($aOrder);
			}
		}
	}

	/**
	 * Verschiebt den verwaisten Eintrag aus der sico_credit_plus_payment Tabelle
	 * auf die neue Bestellung.
	 *
	 * @param int $iOldOrderID
	 * @param string $sOldTransactionID
	 * @param int $iNewOrderID
	 */
	private function moveOldOrderDataToNewOrder( $iOldOrderID, $sOldTransactionID, $iNewOrderID, $sNewTransactionID ) {
		if ( $sOldTransactionID === $sNewTransactionID ) {
			if ( $iOldOrderID != $iNewOrderID ) {
				$this->Application()->Db()->executeUpdate('UPDATE sico_credit_plus_payment SET sico_order_id = :sicoNewOrderId WHERE sico_order_id = :sicoOldOrderID', array('sicoNewOrderId' => $iNewOrderID, 'sicoOldOrderID' => $iOldOrderID));
			}
		}
	}

	/**
	 * Fügt die all.less aus diesem Plugin hinzu
	 *
	 * @return \Shopware\Components\Theme\LessDefinition
	 */
	public function onCollectLessFiles() {
		return new \Shopware\Components\Theme\LessDefinition(
			array(),
			array($this->Path().'Views/frontend/_public/src/less/all.less')
		);
	}

	/**
	 * Für Zahlungsart "sccp_financing" wird anderes Mailtemplate verwendet.
	 * Für alle anderen Zahlungsarten wird keine Zend_Mail Instanz zurückgegeben.
	 *
	 * @param Enlight_Event_EventArgs $oEventArgs
	 *
	 * @return \Enlight_Components_Mail|null
	 * @throws Enlight_Exception
	 */
	public function onOrderSendMailCreate( \Enlight_Event_EventArgs $oEventArgs ) {
		$aContext = $oEventArgs->get('context');
		$aVariables = $oEventArgs->get('variables');
		/** @var \Shopware\Models\Order\Repository $oOrderRepository */
		$oOrderRepository = Shopware()->Models()->getRepository('\Shopware\Models\Order\Order');
		/** @var \Shopware\Models\Order\Order $oOrder */
		$oOrder = $oOrderRepository->findOneBy(array('number' => $aVariables['ordernumber']));
		if ( $oOrder && ($oOrder->getPayment()->getName() == 'sccp_financing') ) {
			$oConfig = Shopware()->Config();
			$sTransactionMode = $oConfig->getByNamespace('SicoCreditPlus', 'sTransactionMode');
			$aContext['transactionID'] = $oOrder->getTransactionId();

			if ( $aVariables['ordernumber'] && $aContext['transactionID'] && preg_match('/^CP[1-9]([0-9]+)$/', $aContext['transactionID']) ) {
				// Ziel für URL = dieser Controller mit anderer Action
				$aTarget = array(
					'controller' => 'sicotrigger',
					'action' => 'showPopup',
					'cporder' => $aVariables['ordernumber'],
					'don' => $aContext['transactionID'],
				);
				$sShownAs = Shopware()->Config()->getByNamespace('SicoCreditPlus','sCPShownAs');
				if ( $sShownAs == 'iframe' ) {
					$aTarget['action'] = 'showIframe';
				} elseif ( $sShownAs == 'new_window' ) {
					$aTarget['action'] = 'showNewWindow';
				}
				$oRouter = Shopware()->Front()->Router();
				$sPayNowURL = $oRouter->assemble($aTarget);

				$aTarget['action'] = 'cancelAndReorder';
				$sCancelURL = $oRouter->assemble($aTarget);

				$aContext['sTargetURL'] = $sPayNowURL;
				$aContext['sCancelURL'] = $sCancelURL;
			}
			// If going a deprecated way still used in Shopware does not yield results...
			if ( !$aContext['sDispatch'] && ($iDispatchId = $oOrder->getDispatch()->getId()) ) {
				$aContext['sDispatch'] = Shopware()->Modules()->Admin()->sGetPremiumDispatch($iDispatchId);
			}
			$sMailTemplate = 'sORDERForCreditPlus';
			if(!class_exists('SicoCreditPlus\Components\SicoCreditPlusHelper')){
				spl_autoload_register(function($className){
					if(strpos($className,'SicoCreditPlus\Components\SicoCreditPlusHelper') !== false){
						include $this->Path().'/Components/SicoCreditPlusHelper.php';
					} else if(strpos($className,'SicoCreditPlus') !== false){
						$sClassName = str_replace("\\",'/',$className);
						$sClassName = str_replace('SicoCreditPlus/','',$sClassName);
						$sClassName = trim($sClassName,'/');
						include $this->Path().$sClassName.'.php';
					}
				});
			}
			$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
			$oContract = $oCPHelper->getContractData($oOrder);
			$aContext['oContract'] = array(
				'state' => "{$oContract->state}",
				'completeOrderUrl' => $oContract->completeOrderUrl,
				'finallyApproved' => $oContract->finallyApproved,
				'name' => $oContract->name,
				'id' => $oContract->id,
				'addressChanged' => $oContract->addressChanged,
				'areaCode' => $oContract->areaCode,
				'cancelationDate' => $oContract->cancelationDate,
				'cancelationFrom' => $oContract->cancelationFrom,
				'conditionName' => $oContract->conditionName,
				'cpReferenceNumber' => $oContract->cpReferenceNumber,
				'creditDuration' => $oContract->creditDuration,
				'dealerNumber' => $oContract->dealerNumber,
				'dealerOrderNumber' => $oContract->dealerOrderNumber,
				'decisionDate' => $oContract->decisionDate,
				'deliveryDone' => $oContract->deliveryDone,
				'firstDecisionDate' => $oContract->firstDecisionDate,
				'forename' => $oContract->forename,
				'informationRequest' => $oContract->informationRequest,
				'krebesReference' => $oContract->krebesReference,
				'loginWebshop' => $oContract->loginWebshop,
				'orderDate' => $oContract->orderDate,
				'price' => $oContract->price,
				'stateText' => $oContract->stateText,
				'street' => $oContract->street,
				'town' => $oContract->town,
				'voucher' => $oContract->voucher,
			);
			if ( $sSelectedMailTemplate = $oConfig->getByNamespace('SicoCreditPlus', 'sMailTemplate') ) {
				$sMailTemplate = $sSelectedMailTemplate;
			}
			if ( ($sTransactionMode === 'inorder') && ($sMailTemplate === 'sORDERForCreditPlus') ) {
				$sMailTemplate = 'sORDERForCreditPlusInOrder';
			}
			$oMail = Shopware()->TemplateMail()->createMail($sMailTemplate, $aContext);
			return $oMail;
		}
		return null;
	}

	/**
	 * Die Funktion füllt die Order Daten für das Frontend
	 * (Bestellübersicht) mit Links zur Bezahlung über CreditPlus auf.
	 *
	 * @param Enlight_Event_EventArgs $oEventArgs
	 *
	 * @throws \Doctrine\ORM\ORMException
	 * @throws \Doctrine\ORM\OptimisticLockException
	 * @throws \Doctrine\ORM\TransactionRequiredException
	 */
	public function onModulesAdminGetOpenOrderDataFilterResult( \Enlight_Event_EventArgs $oEventArgs ) {
		$this->registerCustomModels();
		$aOrders = $oEventArgs->getReturn();
		$oShopwareAdmin = Shopware()->Modules()->Admin();
		$oModelManager = Shopware()->Models();
		$oRouter = Shopware()->Front()->Router();
		$oHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$aTarget = array(
			'controller' => 'sicotrigger',
			'action' => 'showPopup'
		);
		$iOrderStateCancelled = 4;
		if ( defined('Shopware\Models\Order\Status::ORDER_STATE_CANCELLED_REJECTED') ) {
			$iOrderStateCancelled = \Shopware\Models\Order\Status::ORDER_STATE_CANCELLED_REJECTED;
		}
		foreach ( $aOrders as $iKey => $aOrder ) {
			// Default Params for other payments and cases
			$aOrders[$iKey]['sicoCreditPlusNeedToPay'] = false;
			$aOrders[$iKey]['sicoPayNowURL'] = '';
			$aPayment = $oShopwareAdmin->sGetPaymentMeanById($aOrder['paymentID']);
			if ( $aPayment['name'] == 'sccp_financing' ) {
				/** @var \Shopware\Models\Order\Order $oOrder */
				$oOrder = $oModelManager->find('\Shopware\Models\Order\Order', $aOrder['id']);
				// Only non-cancelled may be paid.
				if ( $oOrder->getOrderStatus()->getId() != $iOrderStateCancelled ) {
					$oContractData = $oHelper->getContractData($oOrder);
					if ( !$oContractData || !$oContractData->state ) {
						$aOrders[$iKey]['sicoCreditPlusNeedToPay'] = true;
						$aTarget['cporder'] = $oOrder->getNumber();
						$aTarget['don'] = $oOrder->getTransactionId();
						$aOrders[$iKey]['sicoPayNowURL'] = $oRouter->assemble($aTarget);
					}
				}
			}
		}
		$oEventArgs->setReturn($aOrders);
	}

	/**
	 * Fügt der Account Seite weitere Templates und Erweiterungen hinzu.
	 *
	 * @param \Enlight_Controller_ActionEventArgs $oEventArgs
	 */
	public function afterFrontendAccount( \Enlight_Controller_ActionEventArgs $oEventArgs ) {
		$oSubject = $oEventArgs->getSubject();
		$oRequest = $oEventArgs->getRequest();
		$oResponse = $oEventArgs->getResponse();
		$oView = $oSubject->View();

		if ( !$oRequest->isDispatched() || $oResponse->isException() || !$oView->hasTemplate() ) {
			return;
		}

		$this->registerTemplates($oView);
	}

	/**
	 * Entfernt die Zahlungsart sccp_financing,
	 * wenn die Zahlungsart ausgeschlossen ist.
	 *
	 * @param \Enlight_Event_EventArgs $oEventArgs
	 */
	public function onModulesAdminGetPaymentMeansDataFilter( \Enlight_Event_EventArgs $oEventArgs ) {
		/** @var array[] $aPaymentMeans */
		$aPaymentMeans = $oEventArgs->getReturn();

		$this->registerCustomModels();

		$oHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();

		$bSccpExcluded = $oHelper->isPaymentExcluded();


		// If excluded, exclude :)
		if ( $bSccpExcluded ) {
			foreach ( $aPaymentMeans as $iKey => $aPaymentMean ) {
				if ( $aPaymentMean['name'] == 'sccp_financing' ) {
					unset($aPaymentMeans[$iKey]);
				}
			}
		}
		$oEventArgs->setReturn($aPaymentMeans);
	}

	/**
	 * Entfernt den Ausschluss der Zahlungsart,
	 * wenn dem Warenkorb Artikel hinzugefügt werden.
	 *
	 * Ausgenommen davon ist das cancelAndReorder aus diesem Plugin.
	 *
	 * @param Enlight_Event_EventArgs $oEventArgs
	 */
	public function afterBasketAddArticle( \Enlight_Event_EventArgs $oEventArgs ) {
		/** @var int|bool $iReturn */
		$iReturn = $oEventArgs->getReturn();
		$oRequest = Shopware()->Front()->Request();
		if ( ($oRequest->getControllerName() == 'sicotrigger') && ($oRequest->getActionName() == 'cancelAndReorder') ) {
			// Our "cancel and reorder" may not unset the blocker
			return;
		}
		if ( isset($iReturn) && $iReturn ) {
			Shopware()->Session()->offsetSet('bSicoFinancingExcluded', false);
		}
	}

	public function beforeBackendOrder( \Enlight_Controller_ActionEventArgs $oEventArgs ) {
		$this->registerCustomModels();
		$oRequest = $oEventArgs->getRequest();
		if ( $oRequest->getActionName() == 'save' ) {
			$iOrderID = $oRequest->getParam('id');
			if ( !isset($iOrderID) || !$iOrderID ) {
				return;
			}
			$oRep = Shopware()->Models()->getRepository('Shopware\Models\Order\Order');
			$oOrder = $oRep->find($iOrderID);
			if ( !$oOrder ) {
				return;
			}
			$oPayment = $oOrder->getPayment();
			if ( $oPayment->getName() == 'sccp_financing' ) {
				$iPaymentOpen = 17;
				if ( defined('Shopware\Models\Order\Status::PAYMENT_STATE_OPEN') ) {
					$iPaymentOpen = \Shopware\Models\Order\Status::PAYMENT_STATE_OPEN;
				}
				$iOrderOpen = 0;
				if ( defined('Shopware\Models\Order\Status::ORDER_STATE_OPEN') ) {
					$iOrderOpen = \Shopware\Models\Order\Status::ORDER_STATE_OPEN;
				}
				$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
				$oReviewNecessaryState = $oCPHelper->getPaymentStateByName('review_necessary');
				$iPaymentStateId = $oOrder->getPaymentStatus()->getId();
				$oRequest->setParam('clearedBeforeUpdate', $oRequest->getParam('cleared'));
				$oRequest->setParam('cleared', $iPaymentStateId);
				if ( ($iPaymentStateId == $iPaymentOpen) || ($iPaymentStateId == $iOrderOpen) || ($iPaymentStateId == $oReviewNecessaryState->getId()) ) {
					// Prevent change, if state does not allow it
					// Save previous value for later
					$oRequest->setParam('statusBeforeUpdate', $oRequest->getParam('status'));
					$oRequest->setParam('status', $oOrder->getOrderStatus()->getId());
				}
			}
		}
	}

	/**
	 * Nach dem Order Controller des Backends werden je nach Funktion
	 * weitere Daten oder Templates dazu geladen. Diese füllen die Daten
	 * mit CreditPlus Informationen oder Tabs auf. In anderen Fällen
	 * geben diese Informationen an die CreditPlus weiter.
	 *
	 * @param Enlight_Controller_ActionEventArgs $oEventArgs
	 *
	 * @throws \Doctrine\ORM\ORMException
	 * @throws \Doctrine\ORM\OptimisticLockException
	 * @throws \Doctrine\ORM\TransactionRequiredException
	 * @throws \Enlight_Event_Exception If an event exception occurs
	 */
	public function afterBackendOrder(\Enlight_Controller_ActionEventArgs $oEventArgs)
	{
		$oController = $oEventArgs->getSubject();
		$oView = $oController->View();
		$oRequest = $oController->Request();

		$this->registerTemplates($oView);
		$this->registerCustomModels();

		if ( $oRequest->getActionName() == 'index' ) {
			$oView->extendsTemplate('backend/order/extends.app.js');
		}

		if ( $oRequest->getActionName() == 'load' ) {
			$oView->extendsTemplate('backend/order/model/extends.order.js');
			$oView->extendsTemplate('backend/order/view/detail/extends.overview.js');
			$oView->extendsTemplate('backend/order/view/detail/extends.position.js');
			$oView->extendsTemplate('backend/order/view/detail/extends.window.js');
			$oView->extendsTemplate('backend/order/view/batch/extends.form.js');
			$oView->extendsTemplate('backend/order/view/list/extends.list.js');
		}

		if ( strtolower($oRequest->getActionName()) == 'getlist' ) {
			$this->afterBackendOrderGetList($oEventArgs);
		}

		if ( strtolower($oRequest->getActionName()) == 'saveposition' ) {
			$this->afterBackendOrderSavePosition($oEventArgs);
		}

		if ( strtolower($oRequest->getActionName()) == 'deleteposition' ) {
			$this->afterBackendOrderDeletePosition($oEventArgs);
		}

		if ( strtolower($oRequest->getActionName()) == 'save' ) {
			$this->afterBackendOrderSave($oEventArgs);
		}
	}

	/**
	 * Nach dem Durchlauf der Shopware Funktionen werden zusätzlich die Vertragsdaten der CreditPlus hinzugefügt.
	 * Dies wird über ein Einsammeln der TransactionIDs und eine darauf folgende Sammelanfrage gelöst.
	 *
	 * @param Enlight_Controller_ActionEventArgs $oEventArgs
	 *
	 * @throws Enlight_Event_Exception
	 */
	public function afterBackendOrderGetList( \Enlight_Controller_ActionEventArgs $oEventArgs ) {
		$oController = $oEventArgs->getSubject();
		$oView = $oController->View();

		if ( $aData = $oView->getAssign('data') ) {
			$oCreditPlusHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
			// Collect TransactionIDs
			$aTransactionIDs = array();
			foreach ( $aData as $aOrder ) {
				if ( $aOrder['payment'] && ($aOrder['payment']['name'] == 'sccp_financing') ) {
					$aTransactionIDs[] = $aOrder['transactionId'];
					$aTransactionIDs[] = $aOrder['number'];
				}
			}
			// Collect WebshopContract data
			$aContractDatas = $oCreditPlusHelper->getContractDataMultiple($aTransactionIDs);
			// Add data to order array
			foreach ( $aData as $iKey => $aOrder ) {
				if ( $aOrder['payment'] && ($aOrder['payment']['name'] == 'sccp_financing') ) {
					if ( isset($aContractDatas[$aOrder['transactionId']]) ) {
						$oContractData = $aContractDatas[$aOrder['transactionId']];
					} else {
						$oContractData = $aContractDatas[$aOrder['number']];
					}
					$aContractData = $oCreditPlusHelper->convertContractDataToArray($oContractData);
					$aContractData = $oCreditPlusHelper->convertContractDataArrayForDisplay($aContractData);
					$aCustomerData = array(
						'id' => $oContractData->id,
						'firstName' => $oContractData->forename,
						'lastName' => $oContractData->name,
						'street' => $oContractData->street,
						'zip' => $oContractData->areaCode,
						'city' => $oContractData->town
					);
					$this->changeOrderState($aOrder, $oContractData);
					$aOrder['sicoCreditPlusData'] = array(
						'customer' => $aCustomerData,
						'contract' => $aContractData
					);
				} else {
					$aOrder['sicoCreditPlusData'] = array(
						'customer' => null,
						'contract' => null
					);
				}
				$aData[$iKey] = $aOrder;
			}
			$oView->assign('data', $aData);
		}
	}

	/**
	 * Nach dem Durchlauf der Shopware Funktionen wird zusätzlich die Änderung an CreditPlus gemeldet.
	 * Sollte ein Fehler dabei auftreten, wird er über den Standardweg von Shopware ausgegeben.
	 *
	 * @param Enlight_Controller_ActionEventArgs $oEventArgs
	 *
	 * @throws \Doctrine\ORM\ORMException
	 * @throws \Doctrine\ORM\OptimisticLockException
	 * @throws \Doctrine\ORM\TransactionRequiredException
	 */
	private function afterBackendOrderSavePosition( \Enlight_Controller_ActionEventArgs $oEventArgs ) {
		$oController = $oEventArgs->getSubject();
		$oView = $oController->View();
		$oRequest = $oController->Request();

		$oCreditPlusHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$iOrderId = $oRequest->getParam('orderId', null);
		if ( $iOrderId ) {
			/** @var Shopware\Models\Order\Order $oOrder */
			$oOrder = Shopware()->Models()->find('Shopware\Models\Order\Order', $iOrderId);
			if ( $oOrder ) {
				$aError = $oCreditPlusHelper->partiallyReturnOrder( $oOrder );
				if ( $aError ) {
					$aViewError = array(
						'success' => false,
						'message' => $aError['sErrorMessage'] . ' (' . $aError['sError'] . ')'
					);
					$oView->assign( $aViewError );
				}
			} else {
				$oView->assign(
					array(
						'success' => false,
						'message' => 'Bestellung nicht gefunden.'
					)
				);
			}
		}
	}

	/**
	 * Nach dem Durchlauf der Shopware Funktionen wird zusätzlich die Änderung an CreditPlus gemeldet.
	 * Sollte ein Fehler dabei auftreten, wird er über den Standardweg von Shopware ausgegeben.
	 *
	 * @param Enlight_Controller_ActionEventArgs $oEventArgs
	 *
	 * @throws \Doctrine\ORM\ORMException
	 * @throws \Doctrine\ORM\OptimisticLockException
	 * @throws \Doctrine\ORM\TransactionRequiredException
	 */
	private function afterBackendOrderDeletePosition( \Enlight_Controller_ActionEventArgs $oEventArgs ) {
		$oController = $oEventArgs->getSubject();
		$oView = $oController->View();
		$oRequest = $oController->Request();

		$iOrderId = null;
		$aArtNum = $aQuantity = $aPrice = array();

		/** @var array $aPositions */
		$aPositions = $oRequest->getParam('positions', null);
		if ( $aPositions ) {
			foreach ( $aPositions as $aPosition ) {
				$iOrderId = $aPosition['orderId'];
				$aArtNum[] = $aPosition['articleNumber'];
				$aPrice[] = $aPosition['price'];
				$aQuantity[] = $aPosition['quantity'];
			}
		} else {
			/** @var int $iOrderId */
			$iOrderId = $oRequest->getParam('orderId', null)?:$oRequest->getParam('orderID', null);
			/** @var string $sArtNum */
			$sArtNum = $oRequest->getParam('articleNumber', null);
			/** @var int $iQuantity */
			$iQuantity = $oRequest->getParam('quantity', null);
			/** @var float $dPrice */
			$dPrice = $oRequest->getParam('price', null);

			$aArtNum[] = $sArtNum;
			$aQuantity[] = $iQuantity;
			$aPrice[] = $dPrice;
		}
		if ( $iOrderId && $aArtNum && $aQuantity && $aPrice ) {
			/** @var Shopware\Models\Order\Order $oOrder */
			$oOrder = Shopware()->Models()->find('Shopware\Models\Order\Order', $iOrderId);
			if ( $oOrder ) {
				$oCreditPlusHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
				$aError = $oCreditPlusHelper->returnOrderArticles( $oOrder, $aArtNum, $aQuantity, $aPrice );
				if ( $aError ) {
					$aViewError = array(
						'success' => false,
						'message' => $aError['sErrorMessage'] . ' (' . $aError['sError'] . ')'
					);
					$oView->assign( $aViewError );
				}
			} else {
				$oView->assign(
					array(
						'success' => false,
						'message' => 'Bestellung nicht gefunden.'
					)
				);
			}
		}
	}

	/**
	 * @param array $aOrder
	 * @param \SicoCreditPlus\Lib\CreditPlusObjects\WebshopContract $oContractData
	 * @throws \Enlight_Event_Exception If notify fails
	 */
	private function changeOrderState( &$aOrder, $oContractData ) {
		$iOrderNotPaid = 17;
		if ( defined('Shopware\Models\Order\Status::PAYMENT_STATE_OPEN') ) {
			$iOrderNotPaid = \Shopware\Models\Order\Status::PAYMENT_STATE_OPEN;
		}
		$oConfig = Shopware()->Config();
		$sTransactionMode = $oConfig->getByNamespace('SicoCreditPlus', 'sTransactionMode');
		if ( ($aOrder['cleared'] == $iOrderNotPaid) && ($sTransactionMode == 'inorder') ) {
			// Payment pending, do not touch!
			return;
		}

		$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		if ( isset($oContractData->completeOrderUrl) && $oContractData->completeOrderUrl ) {
			/** @var SicoCreditPlus\Models\CreditPlusPayment\Repository $oCPPaymentRepository */
			$oCPPaymentRepository = Shopware()->Models()->getRepository('\SicoCreditPlus\Models\CreditPlusPayment\CreditPlusPayment');
			/** @var SicoCreditPlus\Models\CreditPlusPayment\CreditPlusPayment $oCPPayment */
			$oCPPayment = $oCPPaymentRepository->findOneBy(array('sicoOrderId' => $aOrder['id']));
			$oCPPayment->setSicoOrderFinish($oContractData->completeOrderUrl);
			try {
				Shopware()->Models()->flush();
			} catch ( \Doctrine\ORM\OptimisticLockException $oEx ) {
				// Do nothing
			}
		}
		$sNewState = $oCPHelper->getCreditPlusPaymentState($oContractData->state, $oContractData->finallyApproved, $oContractData->deliveryDone, $oContractData->informationRequest);
		$oState = $oCPHelper->getPaymentStateByName($sNewState);
		if ( $oState && ($aOrder['paymentStatus']['id'] != $oState->getId()) ) {
			// Update needed

			// This value is the ONLY value changing the look on the backend list
			$aOrder['cleared'] = $oState->getId();

			// This is just for fun, none of these values are actually read!
			$aOrder['paymentStatus']['id'] = $oState->getId();
			$aOrder['paymentStatus']['name'] = $sNewState;
			if ( method_exists($oState,'getDescription') ) {
				$aOrder['paymentStatus']['description'] = $oState->getDescription();
			} else {
				$oTranslation = Shopware()->Snippets()->getNamespace('backend/static/payment_status');
				$aOrder['paymentStatus']['description'] = $oTranslation->get($sNewState);
			}
			$aOrder['paymentStatus']['position'] = $oState->getPosition();
			$aOrder['paymentStatus']['group'] = $oState->getGroup();
			$aOrder['paymentStatus']['mail'] = $oState->getSendMail();

			if ( $sNewState != 'review_necessary' ) {
				// Update in the database
				// only if it's not "review_necessary"
				$oOrderHandler = Shopware()->Modules()->Order();
				$oOrderHandler->setPaymentStatus($aOrder['id'], $oState->getId());
				$oEventArgs = new \Enlight_Controller_EventArgs(array(
					'aOrder'  => $aOrder,
					'oOrder'  => $aOrder['id']
				));
				Shopware()->Events()->notify(
					'SicoCreditPlus_Financing_Status_Changed',
					$oEventArgs
				);
			}
		}
	}

	private function afterBackendOrderSave( \Enlight_Controller_ActionEventArgs $oEventArgs ) {
		// Only on secure
		$oRequest = $oEventArgs->getRequest();
		$oView = $oEventArgs->getSubject()->View();
		if ($oRequest->isDispatched()
			&& !$oEventArgs->getResponse()->isException()
			&& $oView->hasTemplate()
		) {
			$aOrder = $oView->getAssign('data');
			if ( $aOrder['payment']['name'] !== 'sccp_financing' ) {
				// Needs to be our payment type for handling
				return;
			}
			if ( ($iStateBeforeUpdate = $oRequest->getParam('statusBeforeUpdate')) !== null ) {
				if ( $aOrder && ($aOrder['status'] !== $iStateBeforeUpdate) ) {
					$oView->assign('success', false);
					$oView->assign('message', 'Status ändern nicht möglich');
				}
			}
			if ( ($iPaymentStateBeforeUpdate = $oRequest->getParam('clearedBeforeUpdate')) !== null ) {
				if ( $aOrder && ($oRequest->getParam('statusBeforeUpdate') == null) && ( $aOrder['cleared'] != $iPaymentStateBeforeUpdate ) ) {
					$oView->assign('success', false);
					$oView->assign('message', 'Zahlungsstatus darf nicht geändert werden.');
				}
			}
		}
	}

	/**
	 * Fügt die fehlenden Infos zum Plugin hinzu
	 *
	 * @param \Enlight_Controller_ActionEventArgs $oEventArgs
	 */
	public function afterPluginManager( \Enlight_Controller_ActionEventArgs $oEventArgs ) {
		$oSubject = $oEventArgs->getSubject();
		$oRequest = $oEventArgs->getRequest();
		$oResponse = $oEventArgs->getResponse();
		$oView = $oSubject->View();

		if ( !$oRequest->isDispatched() || $oResponse->isException() || !$oView->hasTemplate() ) {
			return;
		}

		if ( $oRequest->getActionName() == 'localListing' ) {
			$aData = $oView->getAssign('data');
			/**
			 * @var int $iKey
			 * @var Shopware\Bundle\PluginInstallerBundle\Struct\PluginStruct $oPlugin
			 */
			foreach ( $aData as $iKey => $oPlugin ) {
				if ( $oPlugin->getTechnicalName() == 'SicoCreditPlus' ) {
					$aInfo = $this->getInfo();
					if ( $oPlugin->getLocalDescription() != $aInfo['description'] ) {
						$oPlugin->setLocalDescription($aInfo['description']);
					}
					if ( $oPlugin->getChangelog() == '' ) {
						$oPlugin->setChangelog($aInfo['changelog']);
					}
					if ( $oPlugin->getInstallationManual() == '' ) {
						$oPlugin->setInstallationManual($aInfo['installationManual']);
					}
					$aData[$iKey] = $oPlugin;
				}
			}
			$oView->assign('data', $aData);
		}
	}

	/**
	 * Speichert neues Mail Template, welches in onOrderSendMailCreate genutzt wird.
	 * Fasst existierendes Template jedoch nicht mehr an.
	 */
	private function installMailTemplates() {

		/** @var \Shopware\Models\Mail\Repository $oMailRepository */
		$oMailRepository = Shopware()->Models()->getRepository('\Shopware\Models\Mail\Mail');
		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sORDERForCreditPlus'));

		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			/** @noinspection HtmlUnknownTarget */
			$oMail->fromArray(array(
				'name' => 'sORDERForCreditPlus',
				'fromMail' => '{config name=mail}',
				'fromName' => '{config name=shopName}',
				'subject' => 'Ihre Bestellung im {config name=shopName}',
				'content' => '{include file="string:{config name=emailheaderplain}"}

Hallo {$billingaddress.firstname} {$billingaddress.lastname},

vielen Dank für Ihre Bestellung bei {config name=shopName} (Nummer: {$sOrderNumber}) am {$sOrderDay|date:"DATE_MEDIUM"} um {$sOrderTime|date:"TIME_SHORT"}.
Informationen zu Ihrer Bestellung:

Pos. Art.Nr.              Menge         Preis        Summe
{foreach item=details key=position from=$sOrderDetails}
{$position+1|fill:4} {$details.ordernumber|fill:20} {$details.quantity|fill:6} {$details.price|padding:8} EUR {$details.amount|padding:8} EUR
{$details.articlename|wordwrap:49|indent:5}
{/foreach}

Versandkosten: {$sShippingCosts}
Gesamtkosten Netto: {$sAmountNet}
{if !$sNet}
Gesamtkosten Brutto: {$sAmount}
{/if}

Gewählte Zahlungsart: {$additional.payment.description}
{$additional.payment.additionaldescription}
{if $additional.payment.name == "sccp_financing"}
Wenn Sie die Finanzierung im Shop abgebrochen haben, können Sie über folgenden Link wieder zur Finanzierung kommen:

{$sTargetURL}

Wenn Sie die Finanzierung bereits nach dem Kauf durchgeführt haben, müssen Sie diesen Link nicht anklicken.
Als registrierter Kunde erreichen Sie dieselbe Zahlungsseite über Ihren Kundenaccount.
{/if}

{if $sComment}
Ihr Kommentar:
{$sComment}
{/if}

Rechnungsadresse:
{$billingaddress.company}
{$billingaddress.firstname} {$billingaddress.lastname}
{$billingaddress.street}{if $billingaddress.streetnumber} {$billingaddress.streetnumber}{/if}
{if {config name=showZipBeforeCity}}{$billingaddress.zipcode} {$billingaddress.city}{else}{$billingaddress.city} {$billingaddress.zipcode}{/if}
{$billingaddress.phone}
{$additional.country.countryname}

Lieferadresse:
{$shippingaddress.company}
{$shippingaddress.firstname} {$shippingaddress.lastname}
{$shippingaddress.street}{if $shippingaddress.streetnumber} {$shippingaddress.streetnumber}{/if}
{if {config name=showZipBeforeCity}}{$shippingaddress.zipcode} {$shippingaddress.city}{else}{$shippingaddress.city} {$shippingaddress.zipcode}{/if}
{$additional.countryShipping.countryname}

{if $billingaddress.ustid}
Ihre Umsatzsteuer-ID: {$billingaddress.ustid}
Bei erfolgreicher Prüfung und sofern Sie aus dem EU-Ausland
bestellen, erhalten Sie Ihre Ware umsatzsteuerbefreit.
{/if}


Für Rückfragen stehen wir Ihnen jederzeit gerne zur Verfügung.

Wir wünschen Ihnen noch einen schönen Tag.

{include file="string:{config name=emailfooterplain}"}',
				'contentHtml' => '{include file="string:{config name=emailheaderhtml}"}
<br/><br/>
<p>
Hallo {$billingaddress.firstname} {$billingaddress.lastname},<br/><br/>

vielen Dank für Ihre Bestellung bei {config name=shopName} (Nummer: {$sOrderNumber}) am {$sOrderDay|date:"DATE_MEDIUM"} um {$sOrderTime|date:"TIME_SHORT"}.
<br/>
<br/>
<strong>Informationen zu Ihrer Bestellung:</strong></p>
  <table width="80%" border="0" style="font-family:Arial, Helvetica, sans-serif; font-size:10px;">
    <tr>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Artikel</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Pos.</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Art-Nr.</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Menge</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Preis</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Summe</strong></td>
    </tr>

    {foreach item=details key=position from=$sOrderDetails}
    <tr>
      <td rowspan="2" style="border-bottom:1px solid #cccccc;">{if $details.image.src.0 && $details.modus != 2}<img style="height: 57px;" height="57" src="{$details.image.src.0}" alt="{$details.articlename}" />{else} {/if}</td>
      <td>{$position+1|fill:4} </td>
      <td>{$details.ordernumber|fill:20}</td>
      <td>{$details.quantity|fill:6}</td>
      <td>{$details.price|padding:8}{$sCurrency}</td>
      <td>{$details.amount|padding:8} {$sCurrency}</td>
    </tr>
    <tr>
      <td colspan="5" style="border-bottom:1px solid #cccccc;">{$details.articlename|wordwrap:80|indent:4}</td>
    </tr>
    {/foreach}

  </table>

<p>
  <br/>
  <br/>
    Versandkosten: {$sShippingCosts}<br/>
    Gesamtkosten Netto: {$sAmountNet}<br/>
    {if !$sNet}
    Gesamtkosten Brutto: {$sAmount}<br/>
    {/if}
  <br/>
  <br/>
    <strong>Gewählte Zahlungsart:</strong> {$additional.payment.description}<br/>
    {include file="string:{$additional.payment.additionaldescription}"}
    {if $additional.payment.name == "sccp_financing"}
    Wenn Sie die Finanzierung im Shop abgebrochen haben, können Sie über folgenden Link wieder zur Finanzierung kommen: <br />
    <a style="border: 1px solid #0A9EA7; border-radius: 3px; background-color: #1AAEB7; display: block; width: 200px; text-align: center; font-size: 22px; margin: 0 auto; text-decoration: none; text-shadow: 0 1px 1px #660100; color: #FFFFFF;" href="{$sTargetURL}">Jetzt&nbsp;finanzieren</a><br />
    Wenn Sie die Finanzierung bereits nach dem Kauf durchgeführt haben, müssen Sie diesen Link nicht anklicken.<br />
    <br />
    Als registrierter Kunde erreichen Sie dieselbe Zahlungsseite über Ihren Kundenaccount.
    {/if}
  <br/>
  <br/>
    <strong>Gewählte Versandart:</strong> {$sDispatch.name}<br/>{$sDispatch.description}
</p>
<p>
  {if $sComment}
    <strong>Ihr Kommentar:</strong><br/>
    {$sComment}<br/>
  {/if}
  <br/>
  <br/>
    <strong>Rechnungsadresse:</strong><br/>
    {$billingaddress.company}<br/>
    {$billingaddress.firstname} {$billingaddress.lastname}<br/>
    {$billingaddress.street}{if $billingaddress.streetnumber} {$billingaddress.streetnumber}{/if}<br/>
    {$billingaddress.zipcode} {$billingaddress.city}<br/>
    {$billingaddress.phone}<br/>
    {$additional.country.countryname}<br/>
  <br/>
  <br/>
    <strong>Lieferadresse:</strong><br/>
    {$shippingaddress.company}<br/>
    {$shippingaddress.firstname} {$shippingaddress.lastname}<br/>
    {$shippingaddress.street}{if $shippingaddress.streetnumber} {$shippingaddress.streetnumber}{/if}<br/>
    {$shippingaddress.zipcode} {$shippingaddress.city}<br/>
    {$additional.countryShipping.countryname}<br/>
  <br/>
    {if $billingaddress.ustid}
    Ihre Umsatzsteuer-ID: {$billingaddress.ustid}<br/>
    Bei erfolgreicher Prüfung und sofern Sie aus dem EU-Ausland<br/>
    bestellen, erhalten Sie Ihre Ware umsatzsteuerbefreit.<br/>
    {/if}
  <br/>
  <br/>
    Für Rückfragen stehen wir Ihnen jederzeit gerne zur Verfügung. Sie erreichen uns wie folgt: <br/>{config name=address}
</p>
<br/><br/>
{include file="string:{config name=emailfooterhtml}"}',
				'isHtml' => true,
				'mailtype' => 2,
			));
			Shopware()->Models()->persist($oMail);
		}


		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sORDERForCreditPlusInOrder'));

		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			/** @noinspection HtmlUnknownTarget */
			$oMail->fromArray(array(
				'name' => 'sORDERForCreditPlusInOrder',
				'fromMail' => '{config name=mail}',
				'fromName' => '{config name=shopName}',
				'subject' => 'Ihre Bestellung im {config name=shopName}',
				'content' => '{include file="string:{config name=emailheaderplain}"}

Hallo {$billingaddress.firstname} {$billingaddress.lastname},

vielen Dank für Ihre Bestellung bei {config name=shopName} (Nummer: {$sOrderNumber}) am {$sOrderDay|date:"DATE_MEDIUM"} um {$sOrderTime|date:"TIME_SHORT"}.
Informationen zu Ihrer Bestellung:

Pos. Art.Nr.              Menge         Preis        Summe
{foreach item=details key=position from=$sOrderDetails}
{$position+1|fill:4} {$details.ordernumber|fill:20} {$details.quantity|fill:6} {$details.price|padding:8} EUR {$details.amount|padding:8} EUR
{$details.articlename|wordwrap:49|indent:5}
{/foreach}

Versandkosten: {$sShippingCosts}
Gesamtkosten Netto: {$sAmountNet}
{if !$sNet}
Gesamtkosten Brutto: {$sAmount}
{/if}

Gewählte Zahlungsart: {$additional.payment.description}
{$additional.payment.additionaldescription}
{if $additional.payment.name == "sccp_financing"}
{if $oContract.state|in_array:array("20", "92", "93")}
Da sich die Bestellung noch in der Prüfung befindet, werden Sie zur weiteren Abarbeitung gesondert per E-Mail durch die CreditPlus informiert.
{else}
Wenn Sie die Finanzierung im Shop nicht bestätigt haben, können Sie über folgenden Link zur Bestätigung kommen:

{$sTargetURL}

Wenn Sie die Finanzierung bereits nach dem Kauf bestätigt haben, müssen Sie diesen Link nicht anklicken.
Als registrierter Kunde erreichen Sie dieselbe Bestätigungsseite über Ihren Kundenaccount.
{/if}
{/if}

{if $sComment}
Ihr Kommentar:
{$sComment}
{/if}

Rechnungsadresse:
{$billingaddress.company}
{$billingaddress.firstname} {$billingaddress.lastname}
{$billingaddress.street}{if $billingaddress.streetnumber} {$billingaddress.streetnumber}{/if}
{if {config name=showZipBeforeCity}}{$billingaddress.zipcode} {$billingaddress.city}{else}{$billingaddress.city} {$billingaddress.zipcode}{/if}
{$billingaddress.phone}
{$additional.country.countryname}

Lieferadresse:
{$shippingaddress.company}
{$shippingaddress.firstname} {$shippingaddress.lastname}
{$shippingaddress.street}{if $shippingaddress.streetnumber} {$shippingaddress.streetnumber}{/if}
{if {config name=showZipBeforeCity}}{$shippingaddress.zipcode} {$shippingaddress.city}{else}{$shippingaddress.city} {$shippingaddress.zipcode}{/if}
{$additional.countryShipping.countryname}

{if $billingaddress.ustid}
Ihre Umsatzsteuer-ID: {$billingaddress.ustid}
Bei erfolgreicher Prüfung und sofern Sie aus dem EU-Ausland
bestellen, erhalten Sie Ihre Ware umsatzsteuerbefreit.
{/if}


Für Rückfragen stehen wir Ihnen jederzeit gerne zur Verfügung.

Wir wünschen Ihnen noch einen schönen Tag.

{include file="string:{config name=emailfooterplain}"}',
				'contentHtml' => '{include file="string:{config name=emailheaderhtml}"}
<br/><br/>
<p>
Hallo {$billingaddress.firstname} {$billingaddress.lastname},<br/><br/>

vielen Dank für Ihre Bestellung bei {config name=shopName} (Nummer: {$sOrderNumber}) am {$sOrderDay|date:"DATE_MEDIUM"} um {$sOrderTime|date:"TIME_SHORT"}.
<br/>
<br/>
<strong>Informationen zu Ihrer Bestellung:</strong></p>
  <table width="80%" border="0" style="font-family:Arial, Helvetica, sans-serif; font-size:10px;">
    <tr>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Artikel</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Pos.</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Art-Nr.</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Menge</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Preis</strong></td>
      <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Summe</strong></td>
    </tr>

    {foreach item=details key=position from=$sOrderDetails}
    <tr>
      <td rowspan="2" style="border-bottom:1px solid #cccccc;">{if $details.image.src.0 && $details.modus != 2}<img style="height: 57px;" height="57" src="{$details.image.src.0}" alt="{$details.articlename}" />{else} {/if}</td>
      <td>{$position+1|fill:4} </td>
      <td>{$details.ordernumber|fill:20}</td>
      <td>{$details.quantity|fill:6}</td>
      <td>{$details.price|padding:8}{$sCurrency}</td>
      <td>{$details.amount|padding:8} {$sCurrency}</td>
    </tr>
    <tr>
      <td colspan="5" style="border-bottom:1px solid #cccccc;">{$details.articlename|wordwrap:80|indent:4}</td>
    </tr>
    {/foreach}

  </table>

<p>
  <br/>
  <br/>
    Versandkosten: {$sShippingCosts}<br/>
    Gesamtkosten Netto: {$sAmountNet}<br/>
    {if !$sNet}
    Gesamtkosten Brutto: {$sAmount}<br/>
    {/if}
  <br/>
  <br/>
    <strong>Gewählte Zahlungsart:</strong> {$additional.payment.description}<br/>
    {include file="string:{$additional.payment.additionaldescription}"}
    {if $additional.payment.name == "sccp_financing"}
        {if $oContract.state|in_array:array("20", "92", "93")}
            Da sich die Bestellung noch in der Prüfung befindet, werden Sie zur weiteren Abarbeitung gesondert per E-Mail durch die CreditPlus informiert.
        {else}
            Wenn Sie die Finanzierung im Shop nicht bestätigt haben, können Sie über folgenden Link zur Bestätigung kommen: <br />
            <a style="border: 1px solid #0A9EA7; border-radius: 3px; background-color: #1AAEB7; display: block; width: 200px; text-align: center; font-size: 22px; margin: 0 auto; text-decoration: none; text-shadow: 0 1px 1px #660100; color: #FFFFFF;" href="{$sTargetURL}">Jetzt&nbsp;bestätigen</a><br />
            Wenn Sie die Finanzierung bereits nach dem Kauf bestätigt haben, müssen Sie diesen Link nicht anklicken.<br />
            <br />
            Als registrierter Kunde erreichen Sie dieselbe Bestätigungsseite über Ihren Kundenaccount.
        {/if}
    {/if}
  <br/>
  <br/>
    <strong>Gewählte Versandart:</strong> {$sDispatch.name}<br/>{$sDispatch.description}
</p>
<p>
  {if $sComment}
    <strong>Ihr Kommentar:</strong><br/>
    {$sComment}<br/>
  {/if}
  <br/>
  <br/>
    <strong>Rechnungsadresse:</strong><br/>
    {$billingaddress.company}<br/>
    {$billingaddress.firstname} {$billingaddress.lastname}<br/>
    {$billingaddress.street}{if $billingaddress.streetnumber} {$billingaddress.streetnumber}{/if}<br/>
    {$billingaddress.zipcode} {$billingaddress.city}<br/>
    {$billingaddress.phone}<br/>
    {$additional.country.countryname}<br/>
  <br/>
  <br/>
    <strong>Lieferadresse:</strong><br/>
    {$shippingaddress.company}<br/>
    {$shippingaddress.firstname} {$shippingaddress.lastname}<br/>
    {$shippingaddress.street}{if $shippingaddress.streetnumber} {$shippingaddress.streetnumber}{/if}<br/>
    {$shippingaddress.zipcode} {$shippingaddress.city}<br/>
    {$additional.countryShipping.countryname}<br/>
  <br/>
    {if $billingaddress.ustid}
    Ihre Umsatzsteuer-ID: {$billingaddress.ustid}<br/>
    Bei erfolgreicher Prüfung und sofern Sie aus dem EU-Ausland<br/>
    bestellen, erhalten Sie Ihre Ware umsatzsteuerbefreit.<br/>
    {/if}
  <br/>
  <br/>
    Für Rückfragen stehen wir Ihnen jederzeit gerne zur Verfügung. Sie erreichen uns wie folgt: <br/>{config name=address}
</p>
<br/><br/>
{include file="string:{config name=emailfooterhtml}"}',
				'isHtml' => true,
				'mailtype' => 2,
			));
			Shopware()->Models()->persist($oMail);
		}

		$aStateMailDefault = array(
			'name' => 'sCreditPlusState20',
			'fromMail' => '{config name=mail}',
			'fromName' => '{config name=shopName}',
			'subject' => 'Status der Bestellung im {config name=shopName}',
			'content' => '{include file="string:{config name=emailheaderplain}"}

Hallo {$aOrder.billing.firstName} {$aOrder.billing.lastName},

die Bestellung bei {config name=shopName} (Nummer: {$sOrderNumber}, Transaktion: {$aOrder.transactionId}) am {$sOrderDay|date:"DATE_MEDIUM"} um {$sOrderTime|date:"TIME_SHORT"} wurde soeben geändert.

Neuer Status: {$sccp_newstate}

Wir wünschen Ihnen noch einen schönen Tag.

{include file="string:{config name=emailfooterplain}"}',
			'contentHtml' => '{include file="string:{config name=emailheaderhtml}"}
<br/><br/>
<p>
Hallo {$aOrder.billing.firstName} {$aOrder.billing.lastName},<br/><br/>

die Bestellung bei {config name=shopName} (Nummer: {$sOrderNumber}, Transaktion: {$aOrder.transactionId}) am {$sOrderDay|date:"DATE_MEDIUM"} um {$sOrderTime|date:"TIME_SHORT"} wurde soeben geändert.
<br/>
<br/>
<strong>Informationen zu Ihrer Bestellung:</strong></p>
<table width="80%" border="0" style="font-family:Arial, Helvetica, sans-serif; font-size:10px;">
	<tr>
		<td>Neuer Status</td>
		<td>{$sccp_newstate}</td>
	</tr>
</table>
<p>
Für Rückfragen stehen wir Ihnen jederzeit gerne zur Verfügung. Sie erreichen uns wie folgt: <br/>{config name=address}
</p>
<br/><br/>
{include file="string:{config name=emailfooterhtml}"}',
			'isHtml' => true,
			'mailtype' => 3,
		);
		$this->registerCustomModels();
		$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sCreditPlusState20'));
		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			$oMail->fromArray(array_merge($aStateMailDefault,array('name' => 'sCreditPlusState20')));
			$sStateForMail = $oCPHelper->getCreditPlusPaymentState(20);
			$oStateForMail = $oCPHelper->getPaymentStateByName($sStateForMail);
			$oMail->setStatus($oStateForMail);
			Shopware()->Models()->persist($oMail);
		}
		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sCreditPlusState24n'));
		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			$oMail->fromArray(array_merge($aStateMailDefault,array('name' => 'sCreditPlusState24n')));
			$sStateForMail = $oCPHelper->getCreditPlusPaymentState(24);
			$oStateForMail = $oCPHelper->getPaymentStateByName($sStateForMail);
			$oMail->setStatus($oStateForMail);
			Shopware()->Models()->persist($oMail);
		}
		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sCreditPlusState24a'));
		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			$oMail->fromArray(array_merge($aStateMailDefault,array('name' => 'sCreditPlusState24a')));
			$sStateForMail = $oCPHelper->getCreditPlusPaymentState(24, true);
			$oStateForMail = $oCPHelper->getPaymentStateByName($sStateForMail);
			$oMail->setStatus($oStateForMail);
			Shopware()->Models()->persist($oMail);
		}
		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sCreditPlusState25'));
		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			$oMail->fromArray(array_merge($aStateMailDefault,array('name' => 'sCreditPlusState25')));
			$sStateForMail = $oCPHelper->getCreditPlusPaymentState(25);
			$oStateForMail = $oCPHelper->getPaymentStateByName($sStateForMail);
			$oMail->setStatus($oStateForMail);
			Shopware()->Models()->persist($oMail);
		}
		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sCreditPlusState92'));
		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			$oMail->fromArray(array_merge($aStateMailDefault,array('name' => 'sCreditPlusState92')));
			$sStateForMail = $oCPHelper->getCreditPlusPaymentState(92);
			$oStateForMail = $oCPHelper->getPaymentStateByName($sStateForMail);
			$oMail->setStatus($oStateForMail);
			Shopware()->Models()->persist($oMail);
		}
		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sCreditPlusState95'));
		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			$oMail->fromArray(array_merge($aStateMailDefault,array('name' => 'sCreditPlusState95')));
			$sStateForMail = $oCPHelper->getCreditPlusPaymentState(95);
			$oStateForMail = $oCPHelper->getPaymentStateByName($sStateForMail);
			$oMail->setStatus($oStateForMail);
			Shopware()->Models()->persist($oMail);
		}
		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sCreditPlusState99'));
		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			$oMail->fromArray(array_merge($aStateMailDefault,array('name' => 'sCreditPlusState99')));
			$sStateForMail = $oCPHelper->getCreditPlusPaymentState(99);
			$oStateForMail = $oCPHelper->getPaymentStateByName($sStateForMail);
			$oMail->setStatus($oStateForMail);
			Shopware()->Models()->persist($oMail);
		}

		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sCreditPlusRetryErrors'));

		if ( !$oOldMail ) {
			$oMail = new \Shopware\Models\Mail\Mail();
			$oMail->fromArray(array(
				'name' => 'sCreditPlusRetryErrors',
				'fromMail' => '{config name=mail}',
				'fromName' => '{config name=shopName}',
				'subject' => 'Fehlerhafte CreditPlus Status Updates in {config name=shopName}',
				'content' => '{include file="string:{config name=emailheaderplain}"}

Hallo Bearbeiter,

leider scheint es bei Updates im Shop zu Fehlern gekommen zu sein, die nach 3 Neuversuchen heute nicht aktualisiert werden konnten.

{foreach item="oRQ" key="position" from=$aRQs}{assign var="oOrder" value=$oRQ->getOrder()}
{$position+1|fill:4} Nummer: {$oOrder->getNumber()} Transaktionsnummer: {$oOrder->getTransactionId()}
{/foreach}

Bitte rufen Sie im Backend die Bestellungen einmal auf. Sollte es dann auch nicht funktionieren (Zahlungsstatus: "Überprüfung notwendig"), wenden Sie sich bitte zur Klärung an die CreditPlus.

{include file="string:{config name=emailfooterplain}"}',
				'contentHtml' => '{include file="string:{config name=emailheaderhtml}"}
<br/><br/>
<p>
Hallo Bearbeiter,<br/><br/>

leider scheint es bei Updates im Shop zu Fehlern gekommen zu sein, die nach 3 Neuversuchen heute nicht aktualisiert werden konnten.

<table width="80%" border="0" style="font-family:Arial, Helvetica, sans-serif; font-size:10px;">
	<tr>
		<td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Pos.</strong></td>
		<td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Bestellnummer</strong></td>
		<td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Transaktions ID</strong></td>
	</tr>

	{foreach item="oRQ" key="position" from=$aRQs}
	{assign var="oOrder" value=$oRQ->getOrder()}
	<tr>
		<td>{$position+1|fill:4} </td>
		<td>{$oOrder->getNumber()}</td>
		<td>{$oOrder->getTransactionId()}</td>
	</tr>
	{/foreach}

</table>

<p>
	Bitte rufen Sie im Backend die Bestellungen einmal auf. Sollte es dann auch nicht funktionieren (Zahlungsstatus: "Überprüfung notwendig"), wenden Sie sich bitte zur Klärung an die CreditPlus.
</p>
<br/><br/>
{include file="string:{config name=emailfooterhtml}"}',
				'isHtml' => true,
				'mailtype' => 2,
			));
			Shopware()->Models()->persist($oMail);
		}
	}

	protected function updateMailTemplates() {
		/** @var \Shopware\Models\Mail\Repository $oMailRepository */
		$oMailRepository = Shopware()->Models()->getRepository('\Shopware\Models\Mail\Mail');
		/** @var \Shopware\Models\Mail\Mail $oOldMail */
		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sORDERForCreditPlus'));
		if ( strpos($oOldMail->getContentHtml(),'00723F') !== false ) {
			// Replace old green with new blue
			$sMailHtml = str_replace(
				'style="border: 1px solid #00723F; border-radius: 3px; background-color: #008C56; display: block; width: 200px; text-align: center; font-size: 22px; margin: 0 auto; text-decoration: none; text-shadow: 0 1px 1px #660100; color: #FFFFFF;"',
				'style="border: 1px solid #0A9EA7; border-radius: 3px; background-color: #1AAEB7; display: block; width: 200px; text-align: center; font-size: 22px; margin: 0 auto; text-decoration: none; text-shadow: 0 1px 1px #660100; color: #FFFFFF;"',
				$oOldMail->getContentHtml().''
			);
			$oOldMail->setContentHtml($sMailHtml);
		}
		Shopware()->Models()->persist($oOldMail);

		$oOldMail = $oMailRepository->findOneBy(array('name' => 'sORDERForCreditPlusInOrder'));
		if ( $oOldMail ) {
			$bChangedEmail = false;
			if  ( !strpos($oOldMail->getContent(), 'state === "20"') ) {
				$sContent = $oOldMail->getContent();
				$sContent = str_replace(
					'{if $additional.payment.name == "sccp_financing"}
Wenn Sie die Finanzierung im Shop nicht bestätigt haben, können Sie über folgenden Link zur Bestätigung kommen:

{$sTargetURL}

Wenn Sie die Finanzierung bereits nach dem Kauf bestätigt haben, müssen Sie diesen Link nicht anklicken.
Als registrierter Kunde erreichen Sie dieselbe Bestätigungsseite über Ihren Kundenaccount.
{/if}',
					'{if $additional.payment.name == "sccp_financing"}
{if $oContract.state === "20"}
Da sich die Bestellung noch in der Prüfung befindet, werden Sie zur weiteren Abarbeitung gesondert per E-Mail durch die CreditPlus informiert.
{else}
Wenn Sie die Finanzierung im Shop nicht bestätigt haben, können Sie über folgenden Link zur Bestätigung kommen:

{$sTargetURL}

Wenn Sie die Finanzierung bereits nach dem Kauf bestätigt haben, müssen Sie diesen Link nicht anklicken.
Als registrierter Kunde erreichen Sie dieselbe Bestätigungsseite über Ihren Kundenaccount.
{/if}
{/if}',
					$sContent
				);
				$oOldMail->setContent($sContent);

				$sContent = $oOldMail->getContentHtml();
				$sContent = str_replace(
					'{if $additional.payment.name == "sccp_financing"}
    Wenn Sie die Finanzierung im Shop nicht bestätigt haben, können Sie über folgenden Link zur Bestätigung kommen: <br />
    <a style="border: 1px solid #0A9EA7; border-radius: 3px; background-color: #1AAEB7; display: block; width: 200px; text-align: center; font-size: 22px; margin: 0 auto; text-decoration: none; text-shadow: 0 1px 1px #660100; color: #FFFFFF;" href="{$sTargetURL}">Jetzt&nbsp;bestätigen</a><br />
    Wenn Sie die Finanzierung bereits nach dem Kauf bestätigt haben, müssen Sie diesen Link nicht anklicken.<br />
    <br />
    Als registrierter Kunde erreichen Sie dieselbe Bestätigungsseite über Ihren Kundenaccount.
    {/if}',
					'{if $additional.payment.name == "sccp_financing"}
        {if $oContract.state === "20"}
            Da sich die Bestellung noch in der Prüfung befindet, werden Sie zur weiteren Abarbeitung gesondert per E-Mail durch die CreditPlus informiert.
        {else}
            Wenn Sie die Finanzierung im Shop nicht bestätigt haben, können Sie über folgenden Link zur Bestätigung kommen: <br />
            <a style="border: 1px solid #0A9EA7; border-radius: 3px; background-color: #1AAEB7; display: block; width: 200px; text-align: center; font-size: 22px; margin: 0 auto; text-decoration: none; text-shadow: 0 1px 1px #660100; color: #FFFFFF;" href="{$sTargetURL}">Jetzt&nbsp;bestätigen</a><br />
            Wenn Sie die Finanzierung bereits nach dem Kauf bestätigt haben, müssen Sie diesen Link nicht anklicken.<br />
            <br />
            Als registrierter Kunde erreichen Sie dieselbe Bestätigungsseite über Ihren Kundenaccount.
        {/if}
    {/if}',
					$sContent
				);
				$oOldMail->setContentHtml($sContent);
				$bChangedEmail = true;
			}
			if ( strpos($oOldMail->getContent(), '{if $oContract.state === "20"}') !== false ) {
				$sContent = $oOldMail->getContent();
				$sContent = str_replace(
					'{if $oContract.state === "20"}',
					'{if $oContract.state|in_array:array("20", "92", "93")}',
					$sContent
				);
				$oOldMail->setContent($sContent);

				$sContent = $oOldMail->getContentHtml();
				$sContent = str_replace(
					'{if $oContract.state === "20"}',
					'{if $oContract.state|in_array:array("20", "92", "93")}',
					$sContent
				);
				$oOldMail->setContentHtml($sContent);

				$bChangedEmail = true;
			}
			if ( $bChangedEmail ) {
				Shopware()->Models()->persist($oOldMail);
			}
		}
	}

	/**
	 * @param Shopware_Components_Cron_CronJob $oCJ
	 *
	 * @return string
	 */
	public function onCronJobRequestQueue( \Shopware_Components_Cron_CronJob $oCJ ) {
		$this->registerCustomModels();
		$oCJC = new \SicoCreditPlus\Components\SicoCreditPlusCronJob();
		return $oCJC->runCron($oCJ);
	}

	/**
	 * @param Shopware_Components_Cron_CronJob $oCJ
	 *
	 * @return string
	 */
	public function onCronJobDailyError( \Shopware_Components_Cron_CronJob $oCJ ) {
		$this->registerCustomModels();
		$oCJC = new \SicoCreditPlus\Components\SicoCreditPlusCronJob();
		return $oCJC->runDailyInformation($oCJ);
	}
	/**
	 * @throws Zend_Db_Adapter_Exception
	 */
	protected function updateSnippets() {
		$oShopware = Shopware();
		$oDB = $oShopware->Db();
		$oTranslation = $oShopware->Snippets()->getNamespace('frontend/plugins/payment/creditplus');
		$sDefaultTextCheapestRate = '<img class="financing-mini-info--logo" src="###LOGOLINK###"/><span class="financing-mini-info--text">Schon ab ###PRICE### für ###MONTHS### Monate <a href="javascript:void(0);" onclick="jQuery(\'a[data-tabname=\\\'sico-creditplus\\\']\').click(); jQuery(\'html, body\').animate({scrollTop: jQuery(\'[data-tabname=\\\'sico-creditplus\\\']\').offset().top}, 500);">finanzieren</a>.</span>';
		$sDetailsTextCheapestRate = $oTranslation->get('DetailsFinancingCheapestRate', $sDefaultTextCheapestRate);
		$sPAngVText = $oTranslation->get('sccppangvtext_text','Kaufpreis entspricht Nettodarlehensbetrag. Diese Angaben stellen zugleich das 2/3-Beispiel gemäß § 6a Abs. 4 PAngV dar. Kreditvermittlung erfolgt alleine für die CreditPlus Bank AG, Augustenstraße 7, 70178 Stuttgart. Bonität vorausgesetzt.<br /><br />Gilt nur für ausgewählte Produkte.');
		$sPopupBlockerDefaultText = 'Ihre Browser Einstellungen haben das Laden der CreditPlus Antragsstrecke verhindert.<br /><br/>Bitte lassen Sie das Popup zu oder klicken auf';
		$sPopupBlockerText = $oTranslation->get('AutoOpenOrClick', $sPopupBlockerDefaultText);
		$sFinanceNowDefaultText = 'Jetzt Finanzieren!';
		$sFinanceNowText = $oTranslation->get('OpenFinancingInNewWindow', $sFinanceNowDefaultText);
		$sCancelNowDefaultText = 'Finanzierung abbrechen';
		$sCancelNowText = $oTranslation->get('CancelLink', $sCancelNowDefaultText);
		$sCancelIntroDefaultText = 'Sollten Sie von der Finanzierung absehen, klicken Sie bitte auf "Finanzierung abbrechen":';
		$sCancelIntroText = $oTranslation->get('StartCancelProcess', $sCancelIntroDefaultText);
		$aDirty = $oTranslation->getDirtyFields();
		$aWasDirty = array();
		$aPossibleDirties = array(
			'DetailsFinancingCheapestRate',
			'sccppangvtext_text',
			'AutoOpenOrClick',
			'OpenFinancingInNewWindow',
			'CancelLink',
			'StartCancelProcess',
		);
		foreach ( $aPossibleDirties as $sPossibleDirty ) {
			if ( $aKeys = array_keys($aDirty,$sPossibleDirty) ) {
				foreach ( $aKeys as $iKey ) {
					array_splice($aDirty, $iKey, 1, '');
				}
				$oTranslation->setDirtyFields($aDirty);
				$aWasDirty[] = $sPossibleDirty;
			}
		}
		if ( $sPAngVText == 'Kaufpreis entspricht Nettodarlehensbetrag. Diese Angaben stellen zugleich das 2/3-Beispiel gemäß § 6a Abs. 4 PAngV dar. Kreditvermittlung erfolgt alleine für die CreditPlus Bank AG, Augustenstraße 7, 70178 Stuttgart. Bonität vorausgesetzt.<br /><br />Gilt nur für ausgewählte Produkte.' ) {
			$oDB->exec("DELETE FROM s_core_snippets WHERE name = 'sccp_pangvtext' AND namespace = 'frontend/plugins/payment/creditplus'");
			$oTranslation->set('sccppangvtext_text','Der Kaufpreis entspricht dem Nettokreditbetrag. Diese Angaben stellen zugleich das 2/3-Beispiel gemäß § 6a Abs. 4 PAngV dar. Kreditvermittlung erfolgt alleine für die CreditPlus Bank AG, Augustenstraße 7, 70178 Stuttgart. Bonität vorausgesetzt.<br /><br />Gilt nur für ausgewählte Produkte.');
			$oTranslation->write();
		}
		if ( $sDetailsTextCheapestRate === 'Schon ab ###PRICE### für ###MONTHS### Monate <a href="javascript:void(0);" onclick="jQuery(\'a[data-tabname=\\\'sico-creditplus\\\']\').click();">finanzieren</a>.' ) {
			$oDB->exec("DELETE FROM s_core_snippets WHERE name = 'DetailsFinancingCheapestRate' AND namespace = 'frontend/plugins/payment/creditplus'");
			$oTranslation->set('DetailsFinancingCheapestRate', $sDefaultTextCheapestRate);
			$oTranslation->write();
		}
		if ( $sDetailsTextCheapestRate === 'Schon ab ###PRICE### für ###MONTHS### Monate <a href="javascript:void();" onclick="jQuery(\'a[data-tabname=\\\'sico-creditplus\\\']\').click();">finanzieren</a>.' ) {
			$oDB->exec("DELETE FROM s_core_snippets WHERE name = 'DetailsFinancingCheapestRate' AND namespace = 'frontend/plugins/payment/creditplus'");
			$oTranslation->set('DetailsFinancingCheapestRate', $sDefaultTextCheapestRate);
			$oTranslation->write();
		}
		if ( $sDetailsTextCheapestRate === '<img class="financing-mini-info--logo" src="###LOGOLINK###"/><span class="financing-mini-info--text">Schon ab ###PRICE### für ###MONTHS### Monate <a href="javascript:void(0);" onclick="jQuery(\'a[data-tabname=\\\'sico-creditplus\\\']\').click();">finanzieren</a>.</span>' ) {
			$oDB->exec("DELETE FROM s_core_snippets WHERE name = 'DetailsFinancingCheapestRate' AND namespace = 'frontend/plugins/payment/creditplus'");
			$oTranslation->set('DetailsFinancingCheapestRate', $sDefaultTextCheapestRate);
			$oTranslation->write();
		}
		if ( $sPopupBlockerText === 'Wenn die Seite sich nicht von selbst in einem neuen Tab oder Popup öffnet, lassen Sie dieses Popup bitte zu oder klicken Sie bitte auf folgenden Link:' ) {
			$oDB->exec("DELETE FROM s_core_snippets WHERE name = 'AutoOpenOrClick' AND namespace = 'frontend/plugins/payment/creditplus'");
			$oTranslation->set('AutoOpenOrClick', $sPopupBlockerDefaultText);
			$oTranslation->write();
		}
		if ( $sFinanceNowText === 'Antragsstrecke öffnen' ) {
			$oDB->exec("DELETE FROM s_core_snippets WHERE name = 'OpenFinancingInNewWindow' AND namespace = 'frontend/plugins/payment/creditplus'");
			$oTranslation->set('OpenFinancingInNewWindow', $sFinanceNowDefaultText);
			$oTranslation->write();
		}
		if ( $sCancelNowText === 'Stornierung starten' ) {
			$oDB->exec("DELETE FROM s_core_snippets WHERE name = 'CancelLink' AND namespace = 'frontend/plugins/payment/creditplus'");
			$oTranslation->set('CancelLink', $sCancelNowDefaultText);
			$oTranslation->write();
		}
		if ( $sCancelIntroText === 'Sollten Sie von der Finanzierung absehen, klicken Sie bitte auf "Stornierung starten":' ) {
			$oDB->exec("DELETE FROM s_core_snippets WHERE name = 'StartCancelProcess' AND namespace = 'frontend/plugins/payment/creditplus'");
			$oTranslation->set('StartCancelProcess', $sCancelIntroDefaultText);
			$oTranslation->write();
		}
		if ( $aWasDirty ) {
			foreach ( $aWasDirty as $sWasDirty ) {
				$aDirty[] = $sWasDirty;
			}
			$oTranslation->setDirtyFields($aDirty);
		}
	}

	/**
	 * Installiert Standardwerte für die Models "Productgroup" und "OfferedOption"
	 * @throws Zend_Db_Statement_Exception
	 * @throws \Doctrine\ORM\OptimisticLockException
	 */
	private function installDefaultFinancingOptions() {
		$oDB = Shopware()->Db();
		$oModelManager = Shopware()->Models();
		// Product group
		$oRes = $oDB->executeQuery('SELECT COUNT(*) amount FROM sico_productgroup');
		if ( $oRes && $oRes->rowCount() ) {
			$oResult = $oRes->fetchObject();
			if ( intval($oResult->amount) == 0 ) {
				$oProdGroup = new \SicoCreditPlus\Models\Productgroup\Productgroup();
				$oProdGroup->fromArray(array(
					'sicoName' => 'Finanzierbare Produkte',
					'sicoProductTypeId' => '56',
					'sicoProductClassId' => '11',
				));
				$oModelManager->persist($oProdGroup);
				// Offered option
				$oResOptions = $oDB->executeQuery('SELECT COUNT(*) amount FROM sico_offered_option');
				if ( $oResOptions && $oResOptions->rowCount() ) {
					$oResultOptions = $oResOptions->fetchObject();
					if ( intval($oResultOptions->amount) == 0 ) {
						$aDefaultMonths = array(
							6 => 4.99,
							8 => 4.99,
							10 => 4.99,
							12 => 4.99,
							18 => 4.99,
							20 => 5.99,
							24 => 5.99,
							30 => 5.99,
							36 => 5.99,
							42 => 5.99,
							48 => 5.99,
							54 => 5.99,
							60 => 5.99,
							66 => 5.99,
							72 => 5.99
						);
						$oOfferedOptions = $oProdGroup->getOfferedOptions();
						foreach ( $aDefaultMonths as $iMonths => $dInterest ) {
							$oOfferedOption = new \SicoCreditPlus\Models\OfferedOption\OfferedOption();
							$oOfferedOption->fromArray(array(
								'sicoMonths' => $iMonths,
								'sicoInterest' => $dInterest,
								'sicoProdcode' => 'Standard',
								'sicoActive' => 1,
								'sicoRatefactor' => -1
							));
							$oOfferedOptions->add($oOfferedOption);
						}
					}
				}
				$oModelManager->persist($oProdGroup);
				foreach ( $oProdGroup->getOfferedOptions() as $oOfferedOption ) {
					$oOfferedOption->setProductgroup($oProdGroup);
					$oModelManager->persist($oOfferedOption);
				}
				$oModelManager->flush();
			}
		}
	}

	/**
	 * Wechselt die bereits installierten Werte auf neue Standardwerte
	 * Wenn der Benutzer bereits nicht mehr im Testmodus ist, werden die Werte nicht mehr angefasst.
	 *
	 * @throws \Doctrine\ORM\OptimisticLockException If flush does not work
	 */
	private function updateDefaultFinancings() {
		$this->registerCustomModels();
		$oConfig = Shopware()->Config();
		$sDealerNumber = $oConfig->getByNamespace('SicoCreditPlus', 'sCPDealer');
		$sSoapUser = $oConfig->getByNamespace('SicoCreditPlus', 'sSoapUser');
		if ( ( $sDealerNumber !== '500276' ) || ( $sSoapUser !== 'testwebshop-500276' ) ) {
			// Daten sind nicht mehr Testdaten
			return;
		}
		$aDefaultMonths = array(
			6 => 0.00,
			8 => 0.00,
			10 => 0.00,
			12 => 3.99,
			18 => 3.99,
			20 => 3.99,
			24 => 7.99,
			30 => 7.99,
			36 => 7.99,
			42 => 7.99,
			48 => 7.99,
			54 => 7.99,
			60 => 7.99,
			66 => 7.99,
			72 => 7.99
		);
		$aNewDefaultMonths = array(
			6 => 4.99,
			8 => 4.99,
			10 => 4.99,
			12 => 4.99,
			18 => 4.99,
			20 => 5.99,
			24 => 5.99,
			30 => 5.99,
			36 => 5.99,
			42 => 5.99,
			48 => 5.99,
			54 => 5.99,
			60 => 5.99,
			66 => 5.99,
			72 => 5.99
		);
		/** @var \SicoCreditPlus\Models\OfferedOption\Repository $oOptionRepository */
		$oOptionRepository = Shopware()->Models()->getRepository('SicoCreditPlus\Models\OfferedOption\OfferedOption');
		/** @var \SicoCreditPlus\Models\OfferedOption\OfferedOption[] $aOptions */
		$aOptions = $oOptionRepository->findAll();
		$oModelManager = Shopware()->Models();
		foreach ( $aOptions as &$oOption ) {
			if ( $oOption->getSicoInterest() == $aDefaultMonths[$oOption->getSicoMonths()] ) {
				$oProdGroup = $oOption->getProductgroup();
				// Nur für 56 / 11 ändern
				if ( ($oProdGroup->getSicoProductClassId() == 11) && ($oProdGroup->getSicoProductTypeId() == 56) ) {
					$oOption->setSicoInterest($aNewDefaultMonths[$oOption->getSicoMonths()]);
				}
			}
		}
		$oModelManager->flush($aOptions);
	}
}
