<?php
/**
 * Created by PhpStorm.
 * @author mihovil.bubnjar
 * @date 23.10.2018
 * @time 14:46
 */

namespace SicoCreditPlus\Components;

use SicoCreditPlus\Models\RequestQueue\Repository;
use SicoCreditPlus\Models\RequestQueue\RequestQueue;

class SicoCreditPlusCronJob {
	/**
	 * 
	 * @param \Shopware_Components_Cron_CronJob $oCJ CronJob object
	 * @return \Shopware_Components_Cron_CronJob CronJob object with a return value
	 */
	public function runCron( \Shopware_Components_Cron_CronJob $oCJ ) {
		/** @var Repository $oRep */
		$oRep = Shopware()->Models()->getRepository('SicoCreditPlus\Models\RequestQueue\RequestQueue');
		$aRQs = $oRep->findUnexecutedEntriesBeforeDate(date('Y-m-d H:i:s'));
		$aSuccess = [
			'total' => 0,
			'successInRun' => [
				1 => 0,
				2 => 0,
				3 => 0,
				4 => 0,
			],
			'requeued' => 0,
			'givenUp' => 0,
		];
		if ( $aRQs && count($aRQs) ) {
			$oCPHelper = new SicoCreditPlusHelper();
			if ( $oWSAPI = $oCPHelper->getWebshopAPI() ) {
				if ( $oWSAPI->isApiCallable() ) {
					$aTransactionIDs = [];
					// Collect all Transaction IDs
					foreach ( $aRQs as $oRQ ) {
						$oOrder = $oRQ->getOrder();
						$aTransactionIDs[] = $oOrder->getNumber();
						$aTransactionIDs[] = $oOrder->getTransactionId();
						$aSuccess['total']++;
					}
					// Request only once
					$oCPHelper->getContractDataMultiple($aTransactionIDs);
					// Go through check method for every order - takes less time if data was already requested in bulk
					foreach ( $aRQs as $oRQ ) {
						$oOrder = $oRQ->getOrder();
						try {
							$oCPHelper->checkForUpdateMail($oOrder);
							$oRQ->setAlreadyExecuted(1);
							$oRep->update($oRQ);
							$aSuccess['successInRun'][$oRQ->getRetryRound()]++;
						} catch ( \Exception $oE ) {
							// Any error returns this to the queue
							// unless it has too many rounds
							if ( $oRQ->getRetryRound() < 4 ) {
								$this->cloneRQ($oRep, $oRQ);
								$aSuccess['requeued']++;
							} else {
								$aSuccess['givenUp']++;
							}
						}
					}
				} else {
					foreach ( $aRQs as $oRQ ) {
						$oRQ->setAlreadyExecuted(1);
						try {
							$oRep->update($oRQ);
						} catch ( \Exception $oE ) {
							// Doesn't happen
						}
						if ( $oRQ->getRetryRound() > 3 ) {
							// More than three retrys? This is dead.
							$aSuccess['givenUp']++;
							continue;
						}
						$aSuccess['requeued']++;
						$this->cloneRQ($oRep, $oRQ);
					}
				}
			}
		}
		$sReturn  = '';
		if ( $aSuccess['total'] > 0 ) {
			if ( $sReturn ) { $sReturn .= "\r\n"; }
			$sReturn .= "Ausgeführt gesamt: ".$aSuccess['total'];
		}
		for ( $iRun = 1 ; $iRun <= count($aSuccess['successInRun']) ; $iRun++ ) {
			if ( $aSuccess['successInRun'][$iRun] > 0 ) {
				if ( $sReturn ) { $sReturn .= "\r\n"; }
				$sReturn .= "Erfolgreich im Lauf ".$iRun.": ".$aSuccess['successInRun'][$iRun];
			}
		}
		
		if ( $aSuccess['requeued'] > 0 ) { 
			if ( $sReturn ) { $sReturn .= "\r\n"; }
			$sReturn .= "Wieder eingereiht: ".$aSuccess['requeued'];
		}
		
		if ( $aSuccess['givenUp'] > 0 ) { 
			if ( $sReturn ) { $sReturn .= "\r\n"; }
			$sReturn .= "Aufgegeben: ".$aSuccess['givenUp'];
		}
		if ( !$sReturn ) {
			$sReturn = 'Keine Einträge zum Bearbeiten.';
		}
		$oCJ->setReturn($sReturn);
		return $oCJ->getReturn();
	}

	/**
	 * @param Repository $oRep
	 * @param RequestQueue $oRQ
	 */
	protected function cloneRQ( Repository $oRep, RequestQueue $oRQ ) {
		$oNewRQ = new RequestQueue();
		$oNewRQ->setOrder($oRQ->getOrder());
		// 40-80 minutes
		$iSeconds = rand(2400, 4800);
		try {
			$oDI = new \DateInterval( 'PT' . $iSeconds . 'S' );
			$oNewRQ->setExecutionDate($oRQ->getExecutionDate()->add($oDI));
		} catch ( \Exception $oE ) {
			// The interval should always succeed
		}
		$oNewRQ->setAlreadyExecuted(0);
		$oNewRQ->setRetryRound($oRQ->getRetryRound()+1);
		$oRep->append($oNewRQ);
	}

	/**
	 * @param \Shopware_Components_Cron_CronJob $oCJ CronJob object
	 *
	 * @return string Return value containing CronJob result
	 */
	public function runDailyInformation( \Shopware_Components_Cron_CronJob $oCJ ) {
		$oJob = $oCJ->getJob();
		$oLastRun = $oJob->getNext();
		try { 
			$oLastRun->add(-1 * $oJob->getInterval());
		} catch (\Zend_Date_Exception $oZDE ) {
			// Nothing happened
		}
		$sLastRun = $oLastRun->toString(\Zend_Date::YEAR.'-'.\Zend_Date::MONTH.'-'.\Zend_Date::DAY.' '.\Zend_Date::HOUR.':'.\Zend_Date::MINUTE.':'.\Zend_Date::SECOND);
		if ( ($sData = $oJob->getData()) && (strpos($sData, "\r\n") !== false) ) {
			$aData = explode("\r\n", $sData);
			if ( substr($aData[0],0, 14) == 'Letzter Lauf: ' ) {
				$aLastRun = explode(':', $aData[0], 2);
				$sLastRun = trim($aLastRun[1]);
			}
		}
		$sNextLastRun = $oJob->getStart()->toString(\Zend_Date::YEAR.'-'.\Zend_Date::MONTH.'-'.\Zend_Date::DAY.' '.\Zend_Date::HOUR.':'.\Zend_Date::MINUTE.':'.\Zend_Date::SECOND);
		$sReturn = 'Letzter Lauf: '.$sNextLastRun;
		$oRep = Shopware()->Models()->getRepository('SicoCreditPlus\Models\RequestQueue\RequestQueue');
		$aRQs = $oRep->findGivenUpEntriesAfterDate($sLastRun);
		if ( $aRQs && count($aRQs) ) {
			$aContext = [
				'aRQs' => $aRQs,
				'oJob' => $oJob,
			];

			try {
				$mail = Shopware()->TemplateMail()->createMail(
					$oJob->get('inform_template'),
					$aContext
				);
	
				$sRecipient = $oJob->get('inform_mail');
				if ($sRecipient === trim('{$sConfig.sMAIL}')) {
					$sRecipient = Shopware()->Config()->get('mail');
				}
	
				$mail->addTo($sRecipient);
				$mail->send();
				$sReturn .= "\r\nMail an $sRecipient gesendet, ".count($aRQs).' Elemente waren betroffen.';
			} catch ( \Enlight_Exception $oE ) {
				// At least we tried
				$sReturn .= "\r\nMail Objekt konnte nicht angelegt werden: ".$oE->getMessage();
			}
		} else {
			$sReturn .= "\r\nKeine Fehler gefunden.";
		}
		$oCJ->setReturn($sReturn);
		return $oCJ->getReturn();
	}
}