<?php
/**
 * Created by PhpStorm.
 * @author mihovil.bubnjar
 * @date 18.08.2016
 * @time 13:42
 */

#namespace SicoCreditPlus\Controllers\Backend;

class Shopware_Controllers_Backend_SicoCreditPlusOrder extends Shopware_Controllers_Backend_Application {
	protected $model = 'Shopware\\Models\\Order\\Order';
	protected $alias = 'order';
	/** @var \Shopware\Models\Order\Order $_oOrder */
	protected $_oOrder = null;

	public function getAdditionalDataAction() {
		$oRequest = $this->Request();
		$iId = $oRequest->getParam('id', null);
		/** @var string|array $aReturn */
		$aReturn = array( 'success' => false );
		if ( $iId !== null ) {
			$oOrder = $this->getOrder($iId);
			$aContractData = $this->getContractData($oOrder);
			$aCustomerData = $this->getCustomerData($oOrder);
			$aData = array( 'id' => $iId, 'contract' => ($aContractData), 'customer' => ($aCustomerData) );
			$aReturn['data'] = array($aData);
			$aReturn['success'] = true;
		} else {
			$aReturn['error'] = 'ID fehlt.';
		}
		$this->View()->assign($aReturn);
	}

	/**
	 * This action is called on save, update and delete.
	 * This data will never be saved, updated or deleted.
	 */
	public function blankAction() {
		/** @var string|array $aReturn */
		$aReturn = array(
			'success' => 'true',
			'mika' => 'happy'
		); 
		$this->View()->assign(
			$aReturn
		);
	}

	/**
	 * This action is called on the CreditPlus tab on the order details page when click the "Cancel Now" Button.
	 * It should return success or an error message given by the CreditPlus API.
	 */
	public function cancelOrderAction() {
		$this->View()->assign(
			$this->cancelOrder()
		);
	}

	/**
	 * This action is called on the CreditPlus tab on the order details page when click the "Send Now" Button.
	 * It should return success or an error message given by the CreditPlus API.
	 */
	public function sendOrderAction() {
		$this->View()->assign(
			$this->sendOrder()
		);
	}

	/**
	 * @return \Shopware\Models\Order\Repository
	 */
	protected function getRepository() {
		/** @var \Shopware\Models\Order\Repository $oRepository */
		$oRepository = parent::getRepository(); 
		return $oRepository;
	}

	protected function getContractData( $oOrder ) {
		$oCreditPlusHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$oContractData = $oCreditPlusHelper->getContractData($oOrder);
		$aContractData = $oCreditPlusHelper->convertContractDataToArray($oContractData);
		$aContractData = $oCreditPlusHelper->convertContractDataArrayForDisplay($aContractData);
		return $aContractData;
	}

	protected function getCustomerData( $oOrder ) {
		$oCreditPlusHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$oContractData = $oCreditPlusHelper->getContractData($oOrder);

		$aCustomerData = array(
			'id' => $oContractData->id,
			'firstName' => $oContractData->forename,
			'lastName' => $oContractData->name,
			'street' => $oContractData->street,
			'zip' => $oContractData->areaCode,
			'city' => $oContractData->town
		);
		return $aCustomerData;
	}

	/**
	 * @param $iId
	 *
	 * @return \Shopware\Models\Order\Order
	 */
	protected function getOrder($iId) {
		if ( $this->_oOrder === null ) {
			$this->_oOrder = $this->getRepository()->find($iId);
		}
		return $this->_oOrder;
	}

	/**
	 * @return string|array
	 */
	protected function cancelOrder() {
		$iId = $this->Request()->getParam('id', null);
		if ( $iId ) {
			$aReturn = array( 'success' => true );
			$oOrder = $this->getOrder($iId);
			$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
			$aError = $oCPHelper->cancelOrder($oOrder);
			//$aError = array( 'sErrorMessage' => 'Die Wurst ist nicht genug.' );
			//$aError = array();
			if ( $aError ) {
				$aReturn['success'] = false;
				$aReturn['error'] = $aError['sErrorMessage'];
			}
			return $aReturn;
		} else {
			return array( 'success' => 'false', 'error' => 'No ID given.' );
		}
	}

	/**
	 * @return string|array
	 */
	protected function sendOrder() {
		$iId = $this->Request()->getParam('id', null);
		if ( $iId ) {
			$aReturn = array( 'success' => true );
			$oOrder = $this->getOrder($iId);
			$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
			$aError = $oCPHelper->sendOrder($oOrder);
			//$aError = array( 'sErrorMessage' => 'Die Wurst ist nicht genug.' );
			//$aError = array();
			if ( $aError ) {
				$aReturn['success'] = false;
				$aReturn['error'] = $aError['sErrorMessage'];
			}
			return $aReturn;
		} else {
			return array( 'success' => 'false', 'error' => 'No ID given.' );
		}
	}

	/**
	 * Gets Order Details and Mail informations for opening order confirmation mail window
	 * @copyright Update by Leica technicians
	 */
	public function getOrderDetailsAction()
	{
		$orderId = $this->Request()->getParam('orderId');
		$statusId = \Shopware\Models\Order\Status::ORDER_STATE_COMPLETELY_DELIVERED;
		$query = $this->getRepository()->getOrdersQuery([['property' => 'orders.id', 'value' => $orderId]], []);
		$dataEntry = $query->getArrayResult();
		$data = $dataEntry[0];
		$oConfig = Shopware()->Config();
		if ( $oConfig->getByNamespace('SicoCreditPlus', 'bSendMailOnBackendStatechange') ) {
			// If there is no state change, we send a generic "here are some documents regarding your order"-mail
			if ($statusId === null) {
				/** @var $mail Enlight_Components_Mail */
				$mail = Shopware()->Modules()->Order()->createStatusMail($orderId, 0, 'sORDERDOCUMENTS');
			} else {
				/** @var $mail Enlight_Components_Mail */
				$mail = Shopware()->Modules()->Order()->createStatusMail($orderId, $statusId);
			}
			if ($mail instanceof Enlight_Components_Mail) {
				$mail = [
					'mail' => $mail,
					'data' => [
						'error' => false,
						'content' => $mail->getPlainBodyText(),
						'contentHtml' => $mail->getPlainBody(),
						'subject' => $mail->getPlainSubject(),
						'to' => implode(', ', $mail->getTo()),
						'fromMail' => $mail->getFrom(),
						'fromName' => $mail->getFromName(),
						'sent' => false,
						'isHtml' => !empty($mail->getPlainBody()),
						'orderId' => $orderId,
					],
				];
			}
			if (!empty($mail)) {
				$data['mail'] = $mail['data'];
			} else {
				$data['mail'] = null;
			}
		}
		$this->View()->assign(['success' => true, 'data' => $data]);
	}
}