<?php
/**
 * Created by PhpStorm.
 * @author mihovil.bubnjar
 * @date 18.08.2016
 * @time 13:42
 */

#namespace SicoCreditPlus\Controllers\Backend;

class Shopware_Controllers_Backend_SicoCreditPlusProductgroups extends Shopware_Controllers_Backend_Application {
	protected $model = 'SicoCreditPlus\\Models\\Productgroup\\Productgroup';
	protected $alias = 'sicoproductgroup';

	protected function getListQuery()
	{
		$builder = parent::getListQuery();

		//$builder->leftJoin('scvariant.scArticle', 'scArticle');
		//$builder->addSelect(array('scArticle'));

		return $builder;
	}

	protected function getDetailQuery($id)
	{
		$builder = parent::getDetailQuery($id);

		$builder->leftJoin('sicoproductgroup.offeredOptions', 'sicoofferedoption');
		$builder->leftJoin('sicoproductgroup.articles', 'articles');
		$builder->addSelect(array('sicoofferedoption', 'articles'));

		return $builder;
	}

	public function searchAssociation($sSearch, $sAssociation, $iOffset, $iLimit, $iId = null, $filter = [], $sort = []) {
		$aReturn = parent::searchAssociation($sSearch, $sAssociation, $iOffset, $iLimit, $iId,$filter,$sort);
		if ( $sAssociation == 'offeredOptions' ) {
			if ( $aReturn['data'] ) {
				foreach ( $aReturn['data'] as $iKey => $aData ) {
					$aData['name'] = $aData['sicoMonths'].' Monate zu '.number_format($aData['sicoInterest'],2,',','.');
					$aReturn['data'][$iKey] = $aData;
				}
			}
		}
		return $aReturn;
	}

	/**
	 * The loadStoresAction function is an ExtJs event listener method of the article backend module.
	 * The function is used to load all required stores for the article detail page in one request.
	 * @return array
	 */
	public function loadStoresAction()
	{
		$id = $this->Request()->getParam('articleId');
		/** @var \SicoCreditPlus\Models\OfferedOption\Repository $oOfferedOptionsRepository */
		$oOfferedOptionsRepository = $this->getModelManager()->getRepository('\\SicoCreditPlus\\Models\\OfferedOption\\OfferedOption');

		if (!empty($id)) {
			$aProductGroup = $this->getRepository()->findBy(array('id' => $id));
		} else {
			$aProductGroup = array();
		}

		$this->View()->assign(array(
			'success' => true,
			'data' => array(
				'offeredOptions' => $oOfferedOptionsRepository->queryBy(array())->getArrayResult(),
				'productgroup' => $aProductGroup,
				'settings' => array()
			)
		));
	}

	/**
	 * The getAssignedArticlesAction function is an ExtJs event listener method of the backend stores.
	 * This function returns all assigned articles.
	 *
	 */
	public function getAssignedArticlesAction() {
		$iID = $this->Request()->getParam('id', null);
		$sSearch = $this->Request()->getParam('search', null);
		$iPage = intval($this->Request()->getParam('page', 1));
		$iStart = intval($this->Request()->getParam('start', 0));
		$iLimit = intval($this->Request()->getParam('limit', 20));
		$aData = array();
		$iTotal = 0;
		$bSuccess = false;
		if ( $iID ) {
			list( $iTotal, $aData ) = $this->getArticles($iID, $sSearch, $iPage, $iStart, $iLimit, true);
			$bSuccess = true;
		}
		$this->View()->assign(array(
			'success' => $bSuccess,
			'total' => $iTotal,
			'data' => $aData
		));
	}

	/**
	 * The getUnassignedArticlesAction function is an ExtJs event listener method of the backend stores.
	 * This function returns all unassigned articles.
	 *
	 */
	public function getUnassignedArticlesAction() {
		$iID = $this->Request()->getParam('id', null);
		$sSearch = $this->Request()->getParam('search', null);
		$iPage = intval($this->Request()->getParam('page', 1));
		$iStart = intval($this->Request()->getParam('start', 0));
		$iLimit = intval($this->Request()->getParam('limit', 20));
		$aData = array();
		$iTotal = 0;
		$bSuccess = false;
		if ( $iID ) {
			list ( $iTotal, $aData ) = $this->getArticles($iID, $sSearch, $iPage, $iStart, $iLimit, false);
			$bSuccess = true;
		}
		$this->View()->assign(array(
			'success' => $bSuccess,
			'total' => $iTotal,
			'data' => $aData
		));
	}

	/**
	 * @param int $iID Integer of the current Productgroup
	 * @param string $sSearch Searchword for Article Number or Title
	 * @param int $iPage Seitennummer
	 * @param int $iStart Start
	 * @param int $iLimit Limit
	 * @param bool $bAssigned true for assigend, false for unassigned
	 *
	 * @return array
	 */
	public function getArticles( $iID, $sSearch, $iPage, $iStart, $iLimit, $bAssigned ) {
		$oDB = Shopware()->Db();
		$oSelect = $oDB->select();
		$oSelect->from(array('a' => 's_articles'), array('articleId' => 'id', 'name'))
			->joinLeft(array('md' => 's_articles_details'), 'a.main_detail_id = md.id', array('number' => 'ordernumber'))
			->joinLeft(array('s' => 's_articles_supplier'), 'a.supplierID = s.id', array('supplierName' => 'name'))
			->joinLeft(array('pg2a' => 'sico_productgroup_s_articles'), 'pg2a.article_id = a.id', '')
			->joinLeft(array('pg' => 'sico_productgroup'), 'pg2a.productgroup_id = pg.id', '')
		;
		$aBind = array( 'prodgroup' => $iID );
		if ( $bAssigned ) {
			$oSelect->where('pg.id = :prodgroup')
				->group('a.id');
		} else {
			// It's a real no-"where" script, sitting in its no-"where" land :)
			$oSelect->group('a.id')
				->having('( (SUM(pg.id = :prodgroup) = 0) || (SUM(pg.id = :prodgroup) IS NULL) )');
		}
		if ( $sSearch ) {
			$oSelect->where('( ( md.ordernumber LIKE :search ) OR ( a.name LIKE :search ) OR ( s.name LIKE :search ) )');
			$aBind['search'] = '%'.$sSearch.'%';
		}

		$oCount = clone $oSelect;

		if ( $iPage || $iStart || $iLimit ) {
			$oSelect->limit($iLimit, $iStart);
		}
		try {
			$oCount->columns(array('total' => 'COUNT(*)'));
			$oStatement = $oSelect->query(Zend_Db::FETCH_ASSOC, $aBind);
			$aData = $oStatement->fetchAll();
			$oStatementCount = $oCount->query(Zend_Db::FETCH_ASSOC, $aBind);
			$aTotal = $oStatementCount->fetchAll();
			// Because of the group by the amount of results is the total count
			$iTotal = count($aTotal);
		} catch (\Exception $e) {
			// Not good...
			$aData = array('error' => $e->getMessage());
			$iTotal = 0;
		}
		return array( $iTotal, $aData );
	}

	/**
	 * Corrects the assigned articles and sends it to the parent function for handling.
	 *
	 * @param array $aData
	 *
	 * @return array
	 */
	public function save( $aData ) {
		if ( $aData['id'] ) {
			$iId = intval($aData['id']);
			$oArticleQuery = Shopware()->Models()->createQuery();
			$sSelect = 'a, pg';
			$sFrom = '\SicoCreditPlus\Models\Productgroup\Productgroup pg INNER JOIN pg.articles a';
			$sWhere = 'pg.id = :id';
			$sDQL = "SELECT $sSelect FROM $sFrom WHERE $sWhere";
			$oArticleQuery->setDQL($sDQL);
			$oArticleQuery->setParameter('id', $iId);
			$aRealArticles = $oArticleQuery->getArrayResult();
			if ( $aRealArticles ) {
				$aData['articles'] = $aRealArticles[0]['articles'];
			}
		}
		return parent::save($aData);
	}

	/**
	 * Controller action which can be accessed over a request.
	 * This function adds the passed article ids which have to be in the "ids" parameter
	 * to the passed Productgroup.
	 */
	public function addAssignedArticlesAction()
	{
		$this->View()->assign(
			$this->addAssignedArticles(
				$this->Request()->getParam('id'),
				json_decode($this->Request()->getParam('ids'))
			)
		);
	}

	/**
	 * Controller action which can be accessed over a request.
	 * This function adds the passed article ids which have to be in the "ids" parameter
	 * to the passed Productgroup.
	 */
	public function removeAssignedArticlesAction()
	{
		$this->View()->assign(
			$this->removeAssignedArticles(
				$this->Request()->getParam('id'),
				json_decode($this->Request()->getParam('ids'))
			)
		);
	}

	/**
	 * Controller action which can be accessed over a request.
	 * This function adds all articles to the passed Productgroup.
	 */
	public function assignAllArticlesAction() {
		$this->View()->assign(
			$this->assignAllArticles(
				$this->Request()->getParam('id')
			)
		);
	}

	/**
	 * Internal function which is used to remove the passed article ids
	 * from the assigned Productgroup.
	 *
	 * @param int $iId
	 * @param int[] $aArticleIDs
	 * @return array
	 */
	protected function removeAssignedArticles($iId, $aArticleIDs)
	{
		if (empty($aArticleIDs)) {
			return array('success' => false, 'error' => 'No articles selected');
		}

		if (empty($iId)) {
			return array('success' => false, 'error' => 'No productgroup id passed.');
		}

		/** @var \SicoCreditPlus\Models\Productgroup\Productgroup $oProductgroup */
		$oProductgroup = Shopware()->Models()->getReference('SicoCreditPlus\Models\Productgroup\Productgroup', $iId);
		$oArticles = $oProductgroup->getArticles();

		$counter = 0;
		foreach ($aArticleIDs as $iArticleID) {
			if (empty($iArticleID)) {
				continue;
			}

			/** @var \Shopware\Models\Article\Article $oArticle */
			$oArticle = Shopware()->Models()->getReference('Shopware\Models\Article\Article', (int)$iArticleID);
			$oArticles->removeElement($oArticle);

			$counter++;
		}

		Shopware()->Models()->flush();

		return array('success' => true, 'counter' => $counter);
	}

	/**
	 * Helper function to add multiple articles to an category.
	 *
	 * @param int $iId
	 * @param int[] $aArticleIDs
	 * @return array
	 */
	protected function addAssignedArticles($iId, $aArticleIDs)
	{
		if (empty($aArticleIDs)) {
			return array('success' => false, 'error' => 'No articles selected');
		}

		if (empty($iId)) {
			return array('success' => false, 'error' => 'No productgroup id passed.');
		}

		/** @var \SicoCreditPlus\Models\Productgroup\Productgroup $oProductGroup */
		$oProductGroup = Shopware()->Models()->getReference('SicoCreditPlus\Models\Productgroup\Productgroup', $iId);

		$counter = 0;
		$oArticles = $oProductGroup->getArticles();
		foreach ($aArticleIDs as $iArticleID) {
			if (empty($iArticleID)) {
				continue;
			}

			/** @var \Shopware\Models\Article\Article $article */
			$oArticle = Shopware()->Models()->getReference('Shopware\Models\Article\Article', (int)$iArticleID);
			$oArticles->add($oArticle);

			$counter++;
		}

		Shopware()->Models()->flush();

		return array('success' => true, 'counter' => $counter);
	}

	/**
	 * Helper function to add multiple articles to an category.
	 *
	 * @param int $iId
	 * @return array
	 */
	protected function assignAllArticles($iId)
	{
		/** @var \SicoCreditPlus\Models\Productgroup\Productgroup $oProductGroup */
		$oProductGroup = Shopware()->Models()->getReference('SicoCreditPlus\Models\Productgroup\Productgroup', $iId);

		$oArticles = $oProductGroup->getArticles();

		$iPage = 1;
		list ( $iTotal, $aData) = $this->getArticles($iId, '', 1, 0, 20, false);
		if ( isset($aData['error']) ) { return array('success' => false); }
		while ( ($iPage-1)*20 <= $iTotal ) {
			list ( $iTotal, $aData) = $this->getArticles($iId, '', $iPage, ($iPage-1)*20, 20, false);
			if ( isset($aData['error']) ) { return array('success' => false); }
			foreach ( $aData as $aArticle ) {
				$iArticleID = $aArticle['articleId'];
				/** @var \Shopware\Models\Article\Article $article */
				$oArticle = Shopware()->Models()->getReference('Shopware\Models\Article\Article', (int)$iArticleID);
				$oArticles->add($oArticle);
			}
			$iPage++;
		}

		Shopware()->Models()->flush();

		return array('success' => true);
	}
}
