<?php
/**
 * Created by PhpStorm.
 * @author mihovil.bubnjar
 * @date 10.08.2016
 * @time 12:11
 */

#namespace SicoCreditPlus\Controllers\Frontend;


class Shopware_Controllers_Frontend_Sicotrigger extends \Enlight_Controller_Action {

	/** @var \SicoCreditPlus\Components\SicoCreditPlusHelper $_oCreditPlusHelper */
	protected $_oCreditPlusHelper = null;

	public function preDispatch() {
		parent::preDispatch();
		if ( version_compare(Shopware()->Config()->get('version'),'5.0.0','<') ) {
			$this->View()->extendsTemplate('frontend_emotion/index/header.tpl');
			$this->View()->extendsTemplate('frontend_emotion/sicotrigger/dispatch.tpl');
			$this->View()->extendsTemplate('frontend_emotion/sicotrigger/show_popup.tpl');
		}
	}

	public function indexAction() {
		$oTemplate = $this->View()->Template();
		$oTemplate->assign('sicoCalledAction', 'index');
		$iStatusText = intval($this->Request()->get('statusText'));
		$oTemplate->assign('iSuccess', $iStatusText);
		if ( $oOrder = $this->getOrder() ) {
			// Ziel für URL = dieser Controller mit anderer Action
			$aTarget = array(
				'controller' => 'sicotrigger',
				'action' => 'cancelAndReorder',
				'cporder' => $oOrder->getNumber(),
				'don' => $oOrder->getTransactionId(),
			);
			$oRouter = Shopware()->Front()->Router();
			$sReplaceMeURL = $oRouter->assemble($aTarget);
			$aTarget['action'] = 'showPopup';
			$sShownAs = Shopware()->Config()->getByNamespace('SicoCreditPlus','sCPShownAs');
			if ( $sShownAs == 'iframe' ) {
				$aTarget['action'] = 'showIframe';
			} elseif ( $sShownAs == 'new_window' ) {
				$aTarget['action'] = 'showNewWindow';
			}
			$sRetryMeURL = $oRouter->assemble($aTarget);
			// Variablen an Template übergeben
			$oTemplate->assign('sReplaceMeURL', $sReplaceMeURL);
			$oTemplate->assign('sRetryMeURL', $sRetryMeURL);

			// Übersetzung aus Textbausteinen ziehen
			$oTranslation = Shopware()->Snippets()->getNamespace('frontend/plugins/payment/creditplus');
			$sSccpAbortInfoText = $oTranslation->get('sccpabortinfo_text');
			$sSccpAbortInfoText = str_replace(
				array(
					'http://retry.me/',
					'http://replace.me/'
				),
				array(
					$sRetryMeURL,
					$sReplaceMeURL
				),
				$sSccpAbortInfoText
			);
			$oTemplate->assign('sSccpAbortInfoText', $sSccpAbortInfoText);
		} else {
			// Für 30303 wird eine Bestellung benötigt, sonst "Bestellung nicht gefunden" anzeigen
			if ( $iStatusText == 30303 ) {
				$oTemplate->assign('iSuccess', 30305);
			}
		}
	}

	public function cancelAndReorderAction() {
		if ( $oOrder = $this->getOrder() ) {
			$this->fillBasket($oOrder);

			// Dies könnte eine Sicherheitslücke sein:
			// Wenn man die Bestellbestätigung eines Kunden hat,
			// kommt man mit dieser in seinen Account rein.
			//
			// Alternative, wenn dies weggelassen wird:
			// Umbesteller muss sich neu einloggen.
			$this->setActiveUser($oOrder);

			$this->cancelOrder($oOrder);
			$this->redirectToPaymentPage();
			return;
		} else {
			// Bestellung nicht gefunden
			$this->forward('index', null, null, array('statusText' => 30305));
			return;
		}
	}

	public function showPopupAction() {
		$this->showCommon();
		$oTemplate = $this->View()->Template();
		$this->splitGetParamsForTemplate($oTemplate);
		$oTemplate->assign('sicoCalledAction', 'showPopup');
	}

	public function showIframeAction() {
		$this->showCommon();
		$oTemplate = $this->View()->Template();
		$oTemplate->assign('sicoCalledAction', 'showIframe');
	}

	public function showNewWindowAction() {
		$this->showCommon();
		$oTemplate = $this->View()->Template();
		$this->splitGetParamsForTemplate($oTemplate);
		$oTemplate->assign('sicoCalledAction', 'showNewWindow');
	}

	protected function showCommon() {
		if ( $oOrder = $this->getOrder() ) {
			if ( !$this->needsToBePaid($oOrder) ) {
				$this->redirect($this->getForwardURL(array('index', null, null, array('statusText' => 30309))));
				return;
			}
			if ( $this->isCancelledOrder($oOrder) ) {
				$this->redirect($this->getForwardURL(array('index', null, null, array('statusText' => 30310))));
			}
			$sTargetURL = $this->getTargetURL($oOrder);
			$sBackURL = $this->getBackURL($oOrder);
			$oTemplate = $this->View()->Template();
			$oTemplate->assign('order', $oOrder);
			$oTemplate->assign('sTargetURL', $sTargetURL);
			$oTemplate->assign('sBackURL', $sBackURL);
		} else {
			if ( !$this->View()->Template()->getVariable('iSuccess') ) {
				// Bestellung nicht gefunden
				$this->redirect($this->getForwardURL(array('index', null, null, array('statusText' => 30305))));
				return;
			}
		}
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 *
	 * @return bool
	 */
	protected function needsToBePaid( $oOrder ) {
		$iOrderOpen = 0;
		if ( defined('Shopware\Models\Order\Status::ORDER_STATE_OPEN') ) {
			$iOrderOpen = \Shopware\Models\Order\Status::ORDER_STATE_OPEN;
		}
		$iOrderNotPaid = 17;
		if ( defined('Shopware\Models\Order\Status::PAYMENT_STATE_OPEN') ) {
			$iOrderNotPaid = \Shopware\Models\Order\Status::PAYMENT_STATE_OPEN;
		}
		$iPaymentStatus = $oOrder->getPaymentStatus()->getId();
		if ( ( $iPaymentStatus == $iOrderOpen ) || ( $iPaymentStatus == $iOrderNotPaid ) ) {
			return true;
		}
		// Fallback für inorder Bestellung, wenn die Status oben nicht gelten
		if ( $this->isInorderMode() ) {
			if ( $oOrder->getNumber() == '0' ) { return true; }
			$oCPHelper = $this->getCreditPlusHelper();
			$oContractData = $oCPHelper->getContractData($oOrder);
			if ( in_array($oContractData->state, array('20', '24')) ) {
				if ( $oContractData->finallyApproved != 'true' ) {
					return true;
				}
			}
		}
		return false;
	}

	/**
     * Checks whether the order is in the state cancelled to prevent advancing the payment
     * after aborting it and choosing another payment method.
     *
	 * @param \Shopware\Models\Order\Order $oOrder The customer order
     * @return bool True if $oOrder is in a cancelled state
	 */
	protected function isCancelledOrder($oOrder) {
        $oOrderStatus = $oOrder->getOrderStatus();
        $oPaymentStatus = $oOrder->getPaymentStatus();
        $iOrderStatus = $oOrderStatus->getId();
        $iPaymentStatus = $oPaymentStatus->getId();
        $iCancelledStatus = 4;
        if ( defined('Shopware\Models\Order\Status::ORDER_STATE_CANCELLED_REJECTED') ) {
            $iCancelledStatus = Shopware\Models\Order\Status::ORDER_STATE_CANCELLED_REJECTED;
        }
        if ( $iOrderStatus === $iCancelledStatus ) {
            return true;
        }
        $iCancelledPayment = 35;
        if ( defined('Shopware\Models\Order\Status::PAYMENT_STATE_THE_PROCESS_HAS_BEEN_CANCELLED') ) {
            $iCancelledPayment = Shopware\Models\Order\Status::PAYMENT_STATE_THE_PROCESS_HAS_BEEN_CANCELLED;
        }
        if ( $iPaymentStatus === $iCancelledPayment ) {
            return true;
        }
        return false;
	}

	/**
	 * @return \Shopware\Models\Order\Order Order, if cporder (or fallback oxid) and don are correct, null otherwise
	 */
	protected function getOrder() {
		/** @var \Shopware\Models\Order\Order $oOrder */
		$oOrder = null;
		if ( $this->Request()->has('cporder') ) {
			$iOrderNumber = intval(trim($this->Request()->get('cporder')));
		} else {
			$iOrderNumber = intval(trim($this->Request()->get('oxid')));
		}
		$sDON = trim($this->Request()->get('don'));
		if ( $iOrderNumber && $sDON ) {
			/** @var \Shopware\Models\Order\Repository $oOrderRepository */
			$oOrderRepository = Shopware()->Models()->getRepository('\Shopware\Models\Order\Order');
			/** @var \Shopware\Models\Order\Order $oDBOrder */
			$oDBOrder = null;
			if ( !$this->isInorderMode() ) {
				// Postorder = Bestellnummer
				$oDBOrder = $oOrderRepository->findOneBy(array('number' => $iOrderNumber));
			} else {
				// Inorder = ID ...
				$oDBOrder = $oOrderRepository->find($iOrderNumber);
				if ( !$oDBOrder || ($oDBOrder->getTransactionId() != $sDON) ) {
					// ... oder Nummer, in späteren Ansichten
					$oDBOrder = $oOrderRepository->findOneBy(array('number' => $iOrderNumber));
				}
			}
			if ( $oDBOrder && ($oDBOrder->getTransactionId() == $sDON) ) {
				$oOrder = $oDBOrder;
			}
		}
		return $oOrder;
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 *
	 * @return string
	 */
	protected function getTargetURL( $oOrder ) {
		$sTargetURL = '';
		/** @var SicoCreditPlus\Models\CreditPlusPayment\Repository $oCPPaymentRepository */
		$oCPPaymentRepository = Shopware()->Models()->getRepository('\SicoCreditPlus\Models\CreditPlusPayment\CreditPlusPayment');
		/** @var SicoCreditPlus\Models\CreditPlusPayment\CreditPlusPayment $oCPPayment */
		$oCPPayment = $oCPPaymentRepository->findOneBy(array('sicoOrderId' => $oOrder->getId()));
		if ( $oCPPayment && $oCPPayment->getSicoOrderFinish() ) {
			$sTargetURL = $oCPPayment->getSicoOrderFinish();
		} elseif ( $oCPPayment && $oCPPayment->getSicoOrderLink() ) {
			$sTargetURL = $oCPPayment->getSicoOrderLink();
		} else {
			try {
				$oWSApi = $this->getWebshopApi();

				$aInput = $this->getCreditPlusAmount($oOrder);
				$oCreditPlusMainData = $this->getCreditPlusMainData($oOrder);
				$aCustomerAddress = $this->getCustomerAddress($oOrder);
				$aShippingAddress = $this->getShippingAddress($oOrder, $aCustomerAddress);
				$sTriggerURL = $this->getTriggerUrl($oOrder);
				$sGoodFinishURL = $this->getGoodFinishURL($oOrder);
				$sStatusChangeURL = $this->getStatusChangeURL($oOrder);
				$bInOrder = $this->isInorderMode();
				$aShopIntegrationData = $this->getShopIntegrationData($oOrder, $sTriggerURL, $sGoodFinishURL, $bInOrder, $sStatusChangeURL);

				$oCreditOfferResponse = $oWSApi->createCreditOfferCPWebshop($aInput, $oCreditPlusMainData, $aCustomerAddress, $aShippingAddress, $aShopIntegrationData);

				// Response && Format of Response == OK && Value is not empty string
				if ( isset($oCreditOfferResponse) && isset($oCreditOfferResponse->createCreditOffer) && isset($oCreditOfferResponse->createCreditOffer->customerUrl) && $oCreditOfferResponse->createCreditOffer->customerUrl ) {
					if ( !isset($oCPPayment) || !$oCPPayment ) {
						$oCPPayment = new \SicoCreditPlus\Models\CreditPlusPayment\CreditPlusPayment();
						$oCPPayment->setOrder($oOrder);
						Shopware()->Models()->persist($oCPPayment);
					}
					if ( !$oCPPayment->getSicoOrderLink() ) {
						$oCPPayment->setSicoLinkgenTimestamp(time());
						$oCPPayment->setSicoOrderLink($oCreditOfferResponse->createCreditOffer->customerUrl);
					}
					Shopware()->Models()->flush();
					return $oCPPayment->getSicoOrderLink();
				}
			} catch ( \Exception $e ) {
				// Do something about it...
			}
			// Payment page with error flag
			$aTarget = array(
				'controller' => 'sicotrigger',
				'action' => 'index',
				'payerror' => 4,
				'statusText' => 30308,
				// 'cporder' => $oOrder->getId(),
				// 'don' => $oOrder->getTransactionId(),
			);
			if ( !$this->isInorderMode() ) {
				unset($aTarget['payerror']);
			}
			$oRouter = Shopware()->Front()->Router();
			return $oRouter->assemble($aTarget);
		}
		return $sTargetURL;
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 *
	 * @return string
	 */
	protected function getBackURL( $oOrder ) {
		$aTarget = array();
		if ( $this->isInorderMode() && ((int)$oOrder->getNumber() === 0) ) {
			$aTarget['controller'] = 'checkout';
			$aTarget['action'] = $this->getShippingPaymentActionTarget();
		} else {
			$aTarget['controller'] = 'sicotrigger';
			$aTarget['action'] = 'index';
			$aTarget['statusText'] = 30303;
			$aTarget['cporder'] = $oOrder->getNumber();
			$aTarget['don'] = $oOrder->getTransactionId();
		}
		$oRouter = Shopware()->Front()->Router();
		$sBackURL = $oRouter->assemble($aTarget);
		return $sBackURL;
	}

	/**
	 * @return \SicoCreditPlus\Lib\CreditPlusObjects\WebshopCurrency
	 */
	protected function getWebshopCurrency() {
		return $this->getCreditPlusHelper()->getWebshopCurrency();
	}

	/**
	 * @return \SicoCreditPlus\Lib\Controller\CreditPlusWebshopAPI
	 */
	protected function getWebshopApi() {
		return $this->getCreditPlusHelper()->getWebshopAPI();
	}

	/**
	 * @return \SicoCreditPlus\Components\SicoCreditPlusHelper
	 */
	protected function getCreditPlusHelper() {
		if ( !$this->_oCreditPlusHelper ) {
			$this->_oCreditPlusHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		}
		return $this->_oCreditPlusHelper;
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 *
	 * @return \SicoCreditPlus\Lib\CreditPlusHelper\CreditPlusMainData
	 */
	protected function getCreditPlusMainData( $oOrder ) {
		$oCreditPlusMainData = new \SicoCreditPlus\Lib\CreditPlusHelper\CreditPlusMainData();
		$oConfig = Shopware()->Config();
		$oFinancingArticle = $this->getCreditPlusHelper()->getFinancingArticleReference($oOrder);
		$oCreditPlusMainData
			->setDealerNumber($oConfig->getByNamespace('SicoCreditPlus','sCPDealer'))
			->setDealerName($oConfig->getByNamespace('SicoCreditPlus','sCPDealerName'))
			->setSoapUser($oConfig->getByNamespace('SicoCreditPlus','sSoapUser'))
			->setSoapPass($oConfig->getByNamespace('SicoCreditPlus','sSoapPass'))
			->setSoapType($oConfig->getByNamespace('SicoCreditPlus','sSoapType'))
			->setPartnerName($oConfig->getByNamespace('SicoCreditPlus','sPartnerName'))
			->setProductTypeID($oFinancingArticle->productTypeID)
			->setProductClassID($oFinancingArticle->productClassID)
			->setWSDL($oConfig->getByNamespace('SicoCreditPlus','sWSDL'));
		return $oCreditPlusMainData;
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 * @return array
	 */
	protected function getCreditPlusAmount( $oOrder ) {
		$aInput = array(
			'agentId' => 'Webshop',
			'loanAmount' => $oOrder->getInvoiceAmount(),
			'dealerOrderNumber' => $oOrder->getNumber()?:$oOrder->getTransactionId()
		);
		$oConfig = Shopware()->Config();
		if ( $oConfig->getByNamespace('SicoCreditPlus', 'sTransactionNumber') === 'cpnumber' ) {
			$aInput['dealerOrderNumber'] = $oOrder->getTransactionId();
		}

		return $aInput;
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 * @param string $sTriggerURL URL used for status changes
	 * @param string $sGoodFinishURL URL used only on inorder finish to carry you to the order finished page
	 * @param bool $bInOrder True if order is created after payment ("Vor Bestellabschluss")
	 * @param string $sStatusChangeURL Trigger URL with status 3
	 *
	 * @return array
	 * @throws Exception
	 */
	protected function getShopIntegrationData( $oOrder, $sTriggerURL, $sGoodFinishURL = '', $bInOrder = false, $sStatusChangeURL = '' ) {
		$aCart = array();
		$oArticleRep = Shopware()->Models()->getRepository('\Shopware\Models\Article\Article');
		if ( $aDetails = $oOrder->getDetails() ) {
			/** @var \Shopware\Models\Order\Detail $oDetail */
			foreach ( $aDetails as $oDetail ) {
				$dUnitPrice = $dRetailPrice = $oDetail->getPrice();
				// Skip basket contents which are not normal products
				if ( $oDetail->getMode() != 0 ) {
					$sEan = '';
					if ( method_exists($oDetail, 'getEan') ) {
						$sEan = $oDetail->getEan();
					}
					if ( $sEan == '' ) {
						$sEan = $oDetail->getArticleNumber();
					}
					$oShoppingCartItem = new \SicoCreditPlus\Lib\CreditPlusObjects\ShoppingCartItem(array(
						'amount' => $oDetail->getQuantity(),
						'description' => mb_substr($oDetail->getArticleName(), 0, 50),
						'ean' => substr($sEan,0,13),
						'listPrice' => $dRetailPrice,
						'promotion' => false,
						'sellingPrice' => $dUnitPrice
					));
					$aCart[] = $oShoppingCartItem;
					continue;
				}
				/** @var \Shopware\Models\Article\Article $oArticle */
				$oArticle = $oArticleRep->find($oDetail->getArticleId());
				/** @var \Shopware\Models\Article\Detail[] $oArticleDetails */
				$oArticleDetails = $oArticle->getDetails();
				foreach ( $oArticleDetails as $oArticleDetail ) {
					// If article number equals ordered items article number
					if ( $oArticleDetail->getNumber() == $oDetail->getArticleNumber() ) {
						/** @var \Shopware\Models\Article\Price[] $oDetailPrices */
						$oDetailPrices = $oArticleDetail->getPrices();
						foreach ( $oDetailPrices as $oDetailPrice ) {
							// If from <= quantity <= to
							if ( ($oDetailPrice->getFrom() <= $oDetail->getQuantity()) && ( ($oDetailPrice->getTo() === 'beliebig') || ($oDetailPrice->getTo() >= $oDetail->getQuantity()) ) ) {
								$dRetailPrice = $oDetailPrice->getPseudoPrice();
								if ( $dRetailPrice == 0.00 ) {
									// If not set, use selling price
									$dRetailPrice = $oDetailPrice->getPrice();
								}
								// Jump 2 foreaches out as this is the only hit we will find
								break(2);
							}
						}
					}
				}
				$sEan = '';
				if ( method_exists($oDetail, 'getEan') ) {
					$sEan = $oDetail->getEan();
				}
				if ( $sEan == '' ) {
					$sEan = $oDetail->getArticleNumber();
				}
				$oShoppingCartItem = new \SicoCreditPlus\Lib\CreditPlusObjects\ShoppingCartItem(array(
					'amount' => $oDetail->getQuantity(),
					'description' => mb_substr($oDetail->getArticleName(), 0, 50),
					'ean' => substr($sEan,0,13),
					'listPrice' => $dRetailPrice,
					'promotion' => false,
					'sellingPrice' => $dUnitPrice
				));
				if ( $oShoppingCartItem->getSellingPrice() < $oShoppingCartItem->getListPrice() ) {
					$oShoppingCartItem->setPromotion(true);
				}
				$aCart[] = $oShoppingCartItem;
			}
		}
		$aShopIntegrationData = array(
			'triggerUrl' => $sTriggerURL,
			'finishedUrl' => $sGoodFinishURL,
			'inorder' => $bInOrder,
			'statusChangeUrl' => $sStatusChangeURL,
			'shoppingCart' => $aCart
		);

		return $aShopIntegrationData;
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 *
	 * @return string
	 */
	protected function getTriggerUrl( $oOrder ) {
		$aData = array(
			'sRequestTime' => date('YmdHis'),
			'sAddressMD5' => $this->getAddressesMD5($oOrder)
		);
		$sData = urlencode(base64_encode(gzdeflate(serialize($aData))));

		return $this->getTriggerUrlWithDataAndDon($sData, $oOrder->getTransactionId());
	}

	protected function getGoodFinishURL( $oOrder ) {
		$oConfig = Shopware()->Config();
		$sSaltValue = $oConfig->getByNamespace('SicoCreditPlus', 'sSignatureSalt');
		$sURL = $this->getTriggerUrl($oOrder);
		$sCheckURL = $sURL.'&'.$sSaltValue.'7';
		$sSignature = md5($sCheckURL);
		$sURL .= 'callback=7&signature='.$sSignature;
		return $sURL;
	}

	protected function getStatusChangeURL( $oOrder ) {
		$oConfig = Shopware()->Config();
		$sSaltValue = $oConfig->getByNamespace('SicoCreditPlus', 'sSignatureSalt');
		$sURL = $this->getTriggerUrl($oOrder);
		$sCheckURL = $sURL.'&'.$sSaltValue.'3';
		$sSignature = md5($sCheckURL);
		$sURL .= 'callback=3&signature='.$sSignature;
		return $sURL;
	}

	/**
	 * @param string $sData Data - usually urlencoded, base64_encoded, gzdeflated, serialized array
	 * @param string $sDON Dealer Order Number aka CP Number
	 *
	 * @return string Some or other format of https://my.shop.mpsc.intern/sicotrigger/dispatch/data/S0m3w3irdText/don/CP20140511041322?
	 */
	protected function getTriggerUrlWithDataAndDon($sData, $sDON) {
		$oRouter = Shopware()->Front()->Router();
		$aTarget = array(
			'controller' => 'sicotrigger',
			'action' => 'dispatch',
			'data' => $sData,
			'don' => $sDON
		);
		$sTriggerURL = $oRouter->assemble($aTarget);
		if ( strpos($sTriggerURL,'?') === false ) {
			$sTriggerURL .= '?';
		} else {
			if ( substr($sTriggerURL,-1) === '?' ) {
				// Ends with ? - no need to change
			} elseif ( substr($sTriggerURL, -1) === '&' ) {
				// Ends with & - no need to change
			} else {
				$sTriggerURL .= '&';
			}
		}
		return $sTriggerURL;
	}

	/**
	 * Returns md5 string for addresses
	 *
	 * @param \Shopware\Models\Order\Order $oOrder
	 * @return string md5(serialize(array('billing' => $aBillingAddress, 'shipping' => $aShippingAddress)))
	 */
	protected function getAddressesMD5( $oOrder ) {
		$aCustomerAddress = $this->getCustomerAddress($oOrder);
		$aShippingAddress = $this->getShippingAddress($oOrder, $aCustomerAddress);
		$aMD5Array = array(
			'billing' => $aCustomerAddress,
			'shipping' => $aShippingAddress
		);
		return md5(serialize($aMD5Array));
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 *
	 * @return array Billing address
	 */
	protected function getCustomerAddress( $oOrder ) {
		$oBilling = $oOrder->getBilling();
		if ( ($oBilling == null) && ( $this->isInorderMode() ) ) {
			$oCustomer = $oOrder->getCustomer();
			if ( method_exists($oCustomer, 'getBilling') ) {
				$oBilling = $oCustomer->getBilling();
			} else {
				$oBilling = $oCustomer->getDefaultBillingAddress();
			}
		}
		return $this->getAddress($oBilling);
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 * @param array $aCustomerAddress
	 *
	 * @return array Shipping address
	 */
	protected function getShippingAddress( $oOrder, $aCustomerAddress ) {
		$oShipping = $oOrder->getShipping();
		if ( ($oShipping == null) && ( $this->isInorderMode() ) ) {
			$oCustomer = $oOrder->getCustomer();
			if ( method_exists($oCustomer,'getShipping') ) {
				$oShipping = $oCustomer->getShipping();
			} else {
				$oShipping = $oCustomer->getDefaultShippingAddress();
			}

		}
		if ( $oShipping == null ) {
			return $aCustomerAddress;
		}
		$aAddress = $this->getAddress($oShipping);
		// If empty, return billing address
		if ( $aAddress['firstName'].$aAddress['lastName'].$aAddress['city'].$aAddress['postalCode'].$aAddress['street'] == '' ) {
			return $aCustomerAddress;
		}
		return $aAddress;
	}

	/**
	 * @param \Shopware\Models\Order\Shipping|\Shopware\Models\Order\Billing $oAddress
	 * @return array Address as required for this plugin
	 */
	protected function getAddress( $oAddress ) {
		$iDefaultFinancingTime = 6;

		$sStreet = $oAddress->getStreet();
		$aStreet = explode(' ',$sStreet);
		$sStreetNo = array_pop($aStreet);

		$aAddress = array(
			'salutation' => ($oAddress->getSalutation() == 'mr')?1:2,
			'salutation_string' => ($oAddress->getSalutation() == 'mr')?'Herr':'Frau',
			'city' => $oAddress->getCity(),
			'postalCode' => $oAddress->getZipCode(),
			'street' => $sStreet,
			'Hausnummer' => $sStreetNo,
			'firstName' => $oAddress->getFirstName(),
			'Laufzeit' => $iDefaultFinancingTime,
			'lastName' => $oAddress->getLastName(),
			'email' => $oAddress->getCustomer()->getEmail()
		);
		// Method exists in 4.3.x Shop
		if ( method_exists($oAddress, 'getStreetNumber') ) {
			$sStreetNo = $oAddress->getStreetNumber();
			if ( $sStreetNo ) {
				$aAddress['street'] .= ' '.$sStreetNo;
			}
		}
		return $aAddress;
	}

	public function dispatchAction() {
		// This is how an URL looks like:
		// http://cpshopware.intern.dev.sinka.de/sicotrigger/dispatch/
		// data/HcuxDYQwDEDRXTKBbZyQmArpWhrEApAYiYLizlyF2J1A%252B%252FT%252FLCSnCZI4G%252FX7VzumbVfXVWNxBBggYkAi9uFVrGVfyk%252FNho9%252FqKkz%252BxQQuGGKc1pWLUkZ2qyQC645Lq67bg%253D%253D/
		// don/CP20160816121957
		// ?callback=24
		// &signature=dd454cff93abae877b7d458241085d00
		$oTemplate = $this->View()->Template();
		$oTemplate->assign('sicoCalledAction', 'dispatch');
		try {
			$sData = $this->Request()->get('data');
			$bDoubleEncoded = (strpos($sData, '%25') !== false);
			if ( $bDoubleEncoded ) {
				$sData = urldecode($sData);
			}
			$aData = unserialize(
				gzinflate(
					base64_decode(
						$sData
					)
				)
			);
			$sDON = $this->Request()->get('don');
			$iStatus = ((int)$this->Request()->get('callback'));
			/** @var \Shopware\Models\Order\Repository $oOrderRepository */
			$oOrderRepository = Shopware()->Models()->getRepository('\Shopware\Models\Order\Order');
			/** @var \Shopware\Models\Order\Order $oOrder */
			$oOrder = $oOrderRepository->findOneBy(array('transactionId' => $sDON));
			if ( !$oOrder ) {
				// No Order found
				throw new \Exception('Bestellung nicht in Datenbank');
			}
			if ( $aData['sAddressMD5'] !== $this->getAddressesMD5($oOrder) ) {
				// Order or data has been manipulated!
				// But who cares :D
			}
			if ( !$this->checkSignature('', $oOrder) ) {
				// Error in Signature
				throw new \Exception('Signatur der Bestellung passt nicht');
			}
			$sSicoTargetURL = $this->updateContract($oOrder, $iStatus);
			$this->View()->assign('sSicoTargetURL', $sSicoTargetURL);
		} catch ( \Exception $e ) {
			// Error with decription should end on "Order not found" page...
			$oCPHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
			if ( isset($oOrder) && $oOrder && ( (!$oCPHelper->getWebshopAPI()) || (!$oCPHelper->getWebshopAPI()->isApiCallable()) ) ) {
				$oRQ = new \SicoCreditPlus\Models\RequestQueue\RequestQueue();
				$oRQ->setRetryRound(1)
					->setAlreadyExecuted(0)
					->setExecutionDate(new \DateTime())
					->setOrder($oOrder);
				/** @var \SicoCreditPlus\Models\RequestQueue\Repository $oRep */
				$oRep = Shopware()->Models()->getRepository('SicoCreditPlus\Models\RequestQueue\RequestQueue');
				$oRep->append($oRQ);
			}
			$this->redirect($this->getForwardURL(array('index', null, null, array('statusText' => 30305))));
			return;
		}
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 * @param int $iStatus
	 *
	 * @return null|string String if redirect url exists
	 * @throws Exception
	 */
	protected function updateContract( $oOrder, $iStatus ) {
		$oCPHelper = new SicoCreditPlus\Components\SicoCreditPlusHelper();
		$oContractData = $oCPHelper->getContractData($oOrder);
		if ( $oContractData->completeOrderUrl !== '' ) {
			$oCPPaymentRepository = Shopware()->Models()->getRepository('SicoCreditPlus\Models\CreditPlusPayment\CreditPlusPayment');
			$oCPPayment = $oCPPaymentRepository->findOneBy(array('sicoOrderId' => $oOrder));
			if ( $oCPPayment ) {
				$oCPPayment->setSicoOrderFinish($oContractData->completeOrderUrl);
				try {
					Shopware()->Models()->flush();
				} catch ( \Doctrine\ORM\OptimisticLockException $oEx ) {
					// Do nothing
				}
			}
		}
		$sURL = null;
		$iStateOpen = 17;
		if ( defined('Shopware\Models\Order\Status::PAYMENT_STATE_OPEN') ) {
			$iStateOpen = \Shopware\Models\Order\Status::PAYMENT_STATE_OPEN;
		}
		if ( $this->isInorderMode() ) {
			if ( $oLogger = $oCPHelper->getLogger() ) {
				$sToday = date('Y-m-d H:i:s');
				$oLogger->writeRequestInformation($sToday, false, 'updateContract', ['status' => $iStatus, 'dealerOrderNumber' => $oOrder->getTransactionId()]);
			}
			$iStateOpen = 0;
			if ( defined('Shopware\Models\Order\Status::ORDER_STATE_OPEN') ) {
				$iStateOpen = \Shopware\Models\Order\Status::ORDER_STATE_OPEN;
			}
			if ( $iStatus === 7 ) {
				$iStatus = (int)$oContractData->state;
			}
		}
		// Todo: || PaymentStatus == $oHelper->getStatus($iStatus)
		$iPaymentStateID = $oOrder->getPaymentStatus()->getId();
		$sCPPaymentState = $oCPHelper->getCreditPlusPaymentState($iStatus);
		$oCPPaymentState = $oCPHelper->getPaymentStateByName($sCPPaymentState);
		if ( ($iPaymentStateID == $iStateOpen) || (($oCPPaymentState !== null) && ($iPaymentStateID == $oCPPaymentState->getId())) ) {
			// First time finisher
			// or First time after backend already updated finisher
			$oCPHelper->checkForUpdateMail($oOrder);
			$sURL = $this->finishContract($oOrder, $iStatus);
		} else {
			// Already finished
			$oCPHelper->checkForUpdateMail($oOrder);
			$sURL = $this->getForwardURL(array('index', null, null, array('statusText' => 30309)));
		}
		return $sURL;
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 * @param int $iStatus
	 * @return null|string String if redirect URL exists
	 */
	protected function finishContract( $oOrder, $iStatus ) {
		$aGoodStates = array(
			// Tentative OK or Hard OK, customer needs to sign or has signed contract accepted by the bank
			24,
			// Payment in progress (Bank to Shop)
			32,
			// Contract is already paid (Bank to Shop)
			99
		);
		$aGreyStates = array(
			// Open - Manual decision
			20,
			// Open - Additional info required from customer
			25,
		);
		$aBadStates = array(
			// Unknown Error
			2,
			// Declined - soft
			92,
			// Declined - hard
			93
		);
		$oOrderHandler = Shopware()->Modules()->Order();
		$iStateOfPayment = 0;
		if ( in_array($iStatus, $aGoodStates) ) {
			if ( defined('Shopware\Models\Order\Status::PAYMENT_STATE_THE_CREDIT_HAS_BEEN_PRELIMINARILY_ACCEPTED') ) {
				$iStateOfPayment = \Shopware\Models\Order\Status::PAYMENT_STATE_THE_CREDIT_HAS_BEEN_PRELIMINARILY_ACCEPTED;
			} else {
				$iStateOfPayment += 31;
			}
			$oOrderHandler->setPaymentStatus($oOrder->getId(), $iStateOfPayment);
			return $this->getForwardURL(array('index', null, null, array('statusText' => 30307)));
		} elseif ( in_array($iStatus, $aBadStates) ) {
			if ( !$this->isInorderMode() ) {
				if ( defined('Shopware\Models\Order\Status::PAYMENT_STATE_NO_CREDIT_APPROVED') ) {
					$iStateOfPayment = \Shopware\Models\Order\Status::PAYMENT_STATE_NO_CREDIT_APPROVED;
				} else {
					$iStateOfPayment += 30;
				}
				$oOrderHandler->setPaymentStatus($oOrder->getId(), $iStateOfPayment);
			}
			Shopware()->Session()->offsetSet('bSicoFinancingExcluded', true);
			return $this->getForwardURL(array('cancelAndReorder', null, null, array('cporder' => $oOrder->getNumber(), 'don' => $oOrder->getTransactionId())));
		} elseif ( in_array($iStatus, $aGreyStates) ) {
			if ( defined('Shopware\Models\Order\Status::PAYMENT_STATE_THE_CREDIT_HAS_BEEN_PRELIMINARILY_ACCEPTED') ) {
				$iStateOfPayment = \Shopware\Models\Order\Status::PAYMENT_STATE_THE_CREDIT_HAS_BEEN_PRELIMINARILY_ACCEPTED;
			} else {
				$iStateOfPayment += 31;
			}
			$oOrderHandler->setPaymentStatus($oOrder->getId(), $iStateOfPayment);
			return $this->getForwardURL(array('index', null, null, array('statusText' => 30307)));
		}

		// Unknown yet signed Status Code from CreditPlus!
		return $this->getForwardURL(array('index', null, null, array('statusText' => 30308)));
	}

	public function testSignatureAction() {
		$oConfig = Shopware()->Config();
		$bTestMode = $oConfig->getByNamespace('SicoCreditPlus', 'bTestMode');
		if ( !$bTestMode ) {
			$sTarget = $this->getForwardURL(array('index', null, null, array('statusText' => 30308)));
			$this->redirect($sTarget);
			return;
		}
		$oOrder = $this->getOrder();
		$_SERVER['REQUEST_URI'] = str_replace('/testSignature/', '/dispatch/', $_SERVER['REQUEST_URI']);
		$this->checkSignature('', $oOrder);
		exit();
	}

	/**
	 * Checks if the signature is correct. If bTestMode is active, any signature is accepted.
	 * If bTestMode is not set, only the two possible calculations are accepted.
	 * @param string $sSignature Signature given by request - Signing Parameter callback and the Base URL before that
	 * @param \Shopware\Models\Order\Order $oOrder Order object, if given for check
	 * @return bool True if $sSignature is OK, false if not
	 */
	protected function checkSignature( $sSignature = '', $oOrder = null ) {
		$oConfig = Shopware()->Config();
		$oRequest = $this->Request();
		$bTestMode = $oConfig->getByNamespace('SicoCreditPlus', 'bTestMode');
		if ( $bTestMode && ($oRequest->getActionName() !== 'testSignature') ) {
			return true;
		} else {
			// Do a real check in Live Mode and even Test Mode (if 'testSignatureAction' was called)
			$sSaltValue = $oConfig->getByNamespace('SicoCreditPlus', 'sSignatureSalt');
			$sSignature = $sSignature?$sSignature:$oRequest->get('signature');
			$sDON = $oRequest->get('don');
			$sCallback = $oRequest->get('callback');
			$sData = '';
			if ( $oRequest->has('data') ) {
				$sData = $oRequest->get('data');
			}
			$bDoubleEncoded = (strpos($sData, '%25') !== false);
			if ( $bDoubleEncoded ) {
				$sData = urldecode($sData);
			}
			$aGet = $_GET;
			foreach ( $aGet as $sKey => $sValue ) {
				if ( in_array($sKey, array(
					'callback',
					'signature',
					'module',
					'controller',
					'action',
					'data',
					'don',
				)) && $sValue ) {
					unset($aGet[$sKey]);
				}
			}
			if ( count($aGet) > 0 ) {
				// Only Callback, Signature and some others are allowed
				return false;
			}
			$sSignedURL = $this->getTriggerUrlWithDataAndDon($sData, $sDON).'&';
			// URL+Salt+Callback
			$bValid = ($sSignature == md5($sSignedURL.$sSaltValue.$sCallback));
			// or DON+Salt+Callback
			$bValid = $bValid || ($sSignature == md5($sDON.$sSaltValue.$sCallback));
			// or OrderNumber+Salt+Callback
			if ( ( $oOrder !== null ) && ( $sOrderNumber = $oOrder->getNumber() ) ) {
				$bValid = $bValid || ($sSignature == md5($sOrderNumber.$sSaltValue.$sCallback));
			}
			if ( $oRequest->getActionName() == 'testSignature' ) {
				var_dump(md5($sSignedURL.$sSaltValue.$sCallback));
				var_dump(md5($sDON.$sSaltValue.$sCallback));
				if (isset($sOrderNumber) ) {
					var_dump(md5($sOrderNumber.$sSaltValue.$sCallback));
				}
				var_dump($bValid);
			}
			return $bValid;
		}
	}

	/**
	 * Takes params as given to $this->forward() and converts them to an URL
	 * @param array $aParams
	 *
	 * @return string URL Target
	 */
	protected function getForwardURL( $aParams ) {
		$aRouterParams = array();
		if ( $aParams[0] ) {
			$aRouterParams['action'] = $aParams[0];
		} else {
			$aRouterParams['action'] = $this->Request()->getActionName();
		}
		if ( $aParams[1] ) {
			$aRouterParams['controller'] = $aParams[1];
		} else {
			$aRouterParams['controller'] = $this->Request()->getControllerName();
		}
		if ( $aParams[2] ) {
			$aRouterParams['module'] = $aParams[2];
		} else {
			$aRouterParams['module'] = $this->Request()->getModuleName();
		}
		if ( $aParams[3] ) {
			foreach ( $aParams[3] as $sKey => $sValue ) {
				$aRouterParams[$sKey] = $sValue;
			}
		}

		if ( $this->isInorderMode() ) {
			if ( ($aRouterParams['controller'] == 'sicotrigger') && ($aRouterParams['action'] == 'cancelAndReorder') ) {
				// cancelAndReorder muss nur auf Payment Seite zurück
				$aRouterParams = array(
					'controller' => 'checkout',
					'action' => $this->getShippingPaymentActionTarget()
				);
			} elseif ( ($aRouterParams['controller'] == 'sicotrigger') && ($aRouterParams['statusText'] == 30307) ) {
				// Erfolgsfall auf Bestellabschluss schicken
				$aRouterParams = array(
					'controller' => 'checkout',
					'action' => 'finish',
					'sAGB' => 1
				);
			} else {
				// Fehlerfälle auf Fehlerseite schicken lassen
			}
		}


		$oRouter = Shopware()->Front()->Router();
		return $oRouter->assemble($aRouterParams);
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 */
	private function fillBasket( $oOrder ) {
		/** @var \Doctrine\Common\Collections\ArrayCollection|\Shopware\Models\Order\Detail[] $aOrderDetails */
		$aOrderDetails = $oOrder->getDetails();
		if ( $aOrderDetails->count() ) {
			foreach ( $aOrderDetails as $oOrderDetail ) {
				$sArtNum = $oOrderDetail->getArticleNumber();
				$iQuantity = $oOrderDetail->getQuantity();
				$oBasket = Shopware()->Modules()->Basket();
				$oBasket->sAddArticle($sArtNum, $iQuantity);
			}
		}
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 */
	private function setActiveUser( $oOrder ) {
		$iUserID = Shopware()->Session()->get('sUserId', false);
		if ( !$iUserID ) {
			// Set three needed parameters for simulated login...
			/** @var \Shopware\Models\Customer\Customer $oCustomer */
			$oCustomer = $oOrder->getCustomer();

			$iUserID = $oCustomer->getId();
			$sPassword = $oCustomer->getPassword();
			$sUserEmail = $oCustomer->getEmail();

			Shopware()->Session()->offsetSet('sUserId', $iUserID);
			Shopware()->Session()->offsetSet('sUserPassword', $sPassword);
			Shopware()->Session()->offsetSet('sUserMail', $sUserEmail);

		}
	}

	/**
	 * @param \Shopware\Models\Order\Order $oOrder
	 * @return array|null Array with error or null if all is ok
	 */
	private function cancelOrder( $oOrder ) {
		$oOrderHandler = Shopware()->Modules()->Order();
		$oHelper = new \SicoCreditPlus\Components\SicoCreditPlusHelper();
		$aErrors = $oHelper->cancelOrder($oOrder);
		$iOrderState = 4;
		if ( defined('Shopware\Models\Order\Status::ORDER_STATE_CANCELLED_REJECTED') ) {
			$iOrderState = \Shopware\Models\Order\Status::ORDER_STATE_CANCELLED_REJECTED;
		}
		$oOrderHandler->setOrderStatus($oOrder->getId(), $iOrderState);
		return $aErrors;
	}

	private function redirectToPaymentPage() {
		$oRouter = Shopware()->Front()->Router();
		$aTarget = array(
			'controller' => 'checkout',
			'action' => $this->getShippingPaymentActionTarget()
		);
		$this->redirect($oRouter->assemble($aTarget));
	}

	/**
	 * @var bool $_bIsInorderMode
	 */
	private $_bIsInorderMode = null;

	/**
	 * Prüft, ob die aktuelle Einstellung "inorder" ist und speichert dies zwischen.
	 * @return bool
	 */
	private function isInorderMode() {
		if ( $this->_bIsInorderMode === null ) {
			$this->_bIsInorderMode = (Shopware()->Config()->getByNamespace('SicoCreditPlus', 'sTransactionMode') == 'inorder');
		}
		return $this->_bIsInorderMode;
	}

	/**
	 * Returns the page for shipping and payment choice in checkout - based on shop version
	 * @return string
	 */
	private function getShippingPaymentActionTarget() {
		$sShopVersion = Shopware()->Config()->get('version');
		$sShippingPayment = 'shippingPayment';
		if ( version_compare($sShopVersion,'5.0.0','<') ) {
			$sShippingPayment = 'confirm';
		}
		return $sShippingPayment;
	}

	/**
	 * @param Enlight_Template_Default $oTemplate
	 */
	protected function splitGetParamsForTemplate( $oTemplate ) {
		$sTargetURL = $oTemplate->getTemplateVars('sTargetURL');
		$aGetParams = array();
		if ( strpos($sTargetURL,'?') !== false ) {
			$aTargetURL = explode('?', $sTargetURL);
			if ( $aTargetURL[1] ) {
				$aParamBlocks = explode('&', $aTargetURL[1]);
				foreach ( $aParamBlocks as $sParamBlock ) {
					$aParam = explode('=', $sParamBlock);
					$aGetParams[urldecode($aParam[0])] = urldecode($aParam[1]);
				}
			}
		}
		$oTemplate->assign('aGetParams', $aGetParams);
	}
}
