<?php
namespace SicoCreditPlus\Models\PaymentMethod;

use Shopware\Models\Customer\Repository as CustomerRepository;
use Shopware\Models\Customer\Customer;
use ShopwarePlugin\PaymentMethods\Components\GenericPaymentMethod;
use Doctrine\ORM\AbstractQuery;


/**
 * Created by PhpStorm.
 * @author mihovil.bubnjar
 * @date 09.08.2016
 * @time 10:17
 */

class SicoFinancingPaymentMethod extends GenericPaymentMethod {
	/**
	 * @inheritdoc
	 */
	public function validate($paymentData) {
		$aErrorFlag     = array();
		$aErrorMessages = array();

		$oTranslation = Shopware()->Snippets()->getNamespace('frontend/plugins/payment/creditplus');

		$sUserID = Shopware()->Session()->get('sUserId');
		/** @var CustomerRepository $oCustomerRepository */
		$oCustomerRepository = Shopware()->Models()->getRepository('\Shopware\Models\Customer\Customer');
		/** @var Customer $oCustomer */
		$oCustomer = $oCustomerRepository->find($sUserID);
		$oBillingAddress = $oCustomer->getBilling();
		$oShippingAddress = $oCustomer->getShipping();

		// Fehler bei Rechnungsadresse: Keine angegeben oder eines der notwendigen Felder fehlt
		if ( !$oBillingAddress ) {
			$aErrorFlag['billing'] = true;
			$aErrorMessages[] = $oTranslation->get( 'missing_billing');
		} else {
			if ( $oBillingAddress->getFirstName() == '' ) {
				$aErrorFlag['billing'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_billing_firstname');
			}
			if ( $oBillingAddress->getLastName() == '' ) {
				$aErrorFlag['billing'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_billing_lastname');
			}
			if ( $oBillingAddress->getZipCode() == '' ) {
				$aErrorFlag['billing'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_billing_zipcode');
			}
			if ( $oBillingAddress->getCity() == '' ) {
				$aErrorFlag['billing'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_billing_city');
			}
			if ( $oBillingAddress->getStreet() == '' ) {
				$aErrorFlag['billing'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_billing_street');
			}
		}

		// Fehler bei Lieferadresse: Keine angegeben ist ok; wenn angegeben dann aber bitte vollständig
		if ( $oShippingAddress ) {
			if ( $oShippingAddress->getFirstName() == '' ) {
				$aErrorFlag['shipping'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_shipping_firstname');
			}
			if ( $oShippingAddress->getLastName() == '' ) {
				$aErrorFlag['shipping'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_shipping_lastname');
			}
			if ( $oShippingAddress->getZipCode() == '' ) {
				$aErrorFlag['shipping'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_shipping_zipcode');
			}
			if ( $oShippingAddress->getCity() == '' ) {
				$aErrorFlag['shipping'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_shipping_city');
			}
			if ( $oShippingAddress->getStreet() == '' ) {
				$aErrorFlag['shipping'] = true;
				$aErrorMessages[] = $oTranslation->get( 'missing_shipping_street');
			}
		}

		if ( count( $aErrorMessages ) ) {
			return array(
				"sErrorFlag"     => $aErrorFlag,
				"sErrorMessages" => $aErrorMessages
			);
		} else {
			return array();
		}
	}


	/**
	 * @inheritdoc
	 */
	public function savePaymentData($userId, \Enlight_Controller_Request_Request $request)
	{
		$lastPayment = $this->getCurrentPaymentDataAsArray($userId);

		/** @var \Shopware\Models\Payment\Repository $oPaymentRepository */
		$oPaymentRepository = Shopware()->Models()->getRepository('\Shopware\Models\Payment\Payment');
		$paymentMean = $oPaymentRepository->getActivePaymentsQuery(array('name' => 'sccp_financing'))->getOneOrNullResult(AbstractQuery::HYDRATE_ARRAY);

		$data = array(
			'use_billing_data' => 0,
			'bankname' => '',
			'iban' => '',
			'bic' => ''
		);

		if (!$lastPayment) {
			$date = new \DateTime();
			$data['created_at'] = $date->format('Y-m-d');
			$data['payment_mean_id'] = $paymentMean['id'];
			$data['user_id'] = $userId;
			Shopware()->Db()->insert("s_core_payment_data", $data);
		} else {
			$where = array(
				'payment_mean_id = ?' => $paymentMean['id'],
				'user_id = ?'  => $userId
			);

			Shopware()->Db()->update("s_core_payment_data", $data, $where);
		}
	}

	/**
	 * @inheritdoc
	 */
	public function getCurrentPaymentDataAsArray($userId)
	{
		/** @var \Shopware\Models\Customer\PaymentDataRepository $oPDRepository */
		$oPDRepository = Shopware()->Models()->getRepository('\Shopware\Models\Customer\PaymentData');

		$paymentData = $oPDRepository
			->getCurrentPaymentDataQueryBuilder($userId, 'sccp_financing')
			->getQuery()
			->getOneOrNullResult(AbstractQuery::HYDRATE_ARRAY);

		if (isset($paymentData)) {
			return $paymentData;
		}

		return null;
	}
}
