<?php
/**
 * Created by PhpStorm.
 * @author mihovil.bubnjar
 * @date 09.08.2016
 * @time 16:00
 */

namespace SicoCreditPlus\Models\RequestQueue;

use Doctrine\ORM\OptimisticLockException;
use Shopware\Components\Model\ModelRepository;
use Shopware\Models\Order\Order;

class Repository extends ModelRepository {
	
	/**
	 * @param string $sDate Date before which we want to find the entries
	 *
	 * @return null|RequestQueue[]
	 */
	public function findUnexecutedEntriesBeforeDate( $sDate ) {
		$oQB = $this->createQueryBuilder('rq');
		$oExpr = $oQB->expr();
		$oAnd = $oExpr->andX(
			$oExpr->eq('rq.alreadyExecuted', 0),
			$oExpr->lte('rq.executionDate', $oExpr->literal($sDate))
		);
		$oQB->where($oAnd);
		$oResult = $oQB->getQuery()->execute();
		return $oResult;
	}

	/**
	 * @param Order $oOrder
	 *
	 * @return null|RequestQueue[]
	 * @throws \Exception Wenn der Parameter $oOrder keine ID hat, wird ein Fehler ausgeworfen
	 */
	public function findUnexecutedEntriesForOrder( $oOrder ) {
		$oQB = $this->createQueryBuilder('rq');
		$oExpr = $oQB->expr();
		$oAnd = $oExpr->andX(
			$oExpr->eq('rq.alreadyExecuted', 0),
			$oExpr->eq('rq.order', $oOrder->getId())
		);
		$oQB->where($oAnd);
		$oResult = $oQB->getQuery();
		$sDQL = $oResult->getDQL();
		if ( substr(trim($sDQL), -1) == '=' ) {
			throw new \Exception('Parameter $oOrder scheint keine ID zu haben');
		}
		$oResult = $oResult->execute();
		return $oResult;
	}

	public function findGivenUpEntriesAfterDate( $sDate ) {
		$oQB = $this->createQueryBuilder('rq');
		$oExpr = $oQB->expr();
		$oAnd = $oExpr->andX(
			$oExpr->eq('rq.alreadyExecuted', 1),
			$oExpr->eq('rq.retryRound', 4),
			$oExpr->gte('rq.executionDate', $oExpr->literal($sDate))
		);
		$oQB->where($oAnd);
		$oResult = $oQB->getQuery()->execute();
		return $oResult;
	}

	/**
	 * @param RequestQueue $oRequestQueue
	 */
	public function append( $oRequestQueue ) {
		$this->getEntityManager()->persist($oRequestQueue);
		try {
			$this->getEntityManager()->flush( $oRequestQueue );
		} catch ( OptimisticLockException $oOLE ) {
			// Does not happen
		}
	}

	/**
	 * @param RequestQueue $oRequestQueue
	 *
	 * @throws OptimisticLockException
	 */
	public function update( $oRequestQueue ) {
		if ( $oRequestQueue->getId() ) {
			$this->getEntityManager()->flush($oRequestQueue);
		}
	}
}