/**
 * Shopware 5
 * Copyright (c) shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 *
 * @category   Shopware
 * @package    Order
 * @subpackage Model
 * @version    $Id$
 * @author SinkaCom AG
 */

/**
 * SinkaCom CreditPlus Data Model - Used for detail page of order to display Tab with additional information.
 *
 */
//{block name="backend/order/model/sico_credit_plus_data"}
Ext.define('Shopware.apps.Order.model.SicoCreditPlusData', {

	/**
	 * Extends the standard Ext Model
	 * @string
	 */
	extend: 'Ext.data.Model',

	/**
	 * The fields used for this model
	 * @array
	 */
	fields: [
		//{block name="backend/order/model/sico_credit_plus_data/fields"}{/block}
		{ name : 'id', type: 'int' }
	],

	/**
	 * Configure the data communication
	 * @object
	 */
	proxy: {
		/**
		 * Set proxy type to ajax
		 * @string
		 */
		type: 'ajax',

		/**
		 * Specific urls to call on CRUD action methods "create", "read", "update" and "destroy".
		 * @object
		 */
		api: {
			read: '{url controller="SicoCreditPlusOrder" action="getAdditionalData"}',
			create: '{url controller="SicoCreditPlusOrder" action="blank"}',
			update: '{url controller="SicoCreditPlusOrder" action="blank"}',
			destroy: '{url controller="SicoCreditPlusOrder" action="blank"}'
		},

		/**
		 * The Ext.data.reader.Reader to use to decode the server's
		 * response or data read from client. This can either be a Reader instance,
		 * a config object or just a valid Reader type name (e.g. 'json', 'xml').
		 * @object
		 */
		reader: {
			type: 'json',
			root: 'data'
		}
	},

	/**
	 * Define the associations of the CreditPlus Data model.
	 * One CreditPlus Data has a customer and a contract.
	 * @array
	 */
	associations:[
		{
			field: 'customer',
			relation: 'OneToOne',
			type: 'hasOne',
			model: 'Shopware.apps.Order.model.SicoCreditPlusCustomer',
			name: 'getCustomer',
			instanceName: 'getCustomerStore',
			associationKey: 'customer'
		},
		{
			field: 'contract',
			relation: 'OneToOne',
			type: 'hasOne',
			model: 'Shopware.apps.Order.model.SicoCreditPlusContract',
			name: 'getContract',
			instanceName: 'getContractStore',
			associationKey: 'contract'
		}
	],

	getCustomer: function() {
		if ( this.getCustomerStore && this.getCustomerStore.data ) {
			return this.getCustomerStore;
		}
		return null;
	},

	getContract: function() {
		if ( this.getContractStore && this.getContractStore.data ) {
			return this.getContractStore;
		}
		return null;
	}
});
//{/block}

