/**
 * Shopware 5
 * Copyright (c) shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 *
 * @category   Shopware
 * @package    Order
 * @subpackage View
 * @version    $Id$
 * @author SinkaCom AG
 */

/* {namespace name="frontend/plugins/payment/creditplus/backend"} */

/**
 * Shopware UI - Order detail page CreditPlus Data
 *
 */
//{block name="backend/order/view/detail/sico_credit_plus"}
Ext.define('Shopware.apps.Order.view.detail.SicoCreditPlus', {
	/**
	 * Define that the billing field set is an extension of the Ext.form.FieldSet
	 * @string
	 */
	extend: 'Ext.form.FieldSet',
	/**
	 * List of short aliases for class names. Most useful for defining xtypes for widgets.
	 * @string
	 */
	alias: 'widget.order-sico-credit-plus-field-set',
	/**
	 * Set css class for this component
	 * @string
	 */
	cls: Ext.baseCSSPrefix + 'sico-credit-plus-field-set',
	/**
	 * Layout for the component.
	 * @string
	 */
	layout: 'column',
	/**
	 * Contains all snippets for the view component
	 * @object
	 */
	snippets: {
		title: '{s name="CreditPlusTabTitle"}CreditPlus{/s}',
		blockTitles: {
			east: '{s name="CreditPlusTabBlockEast"}CreditPlus Daten{/s}',
			west: '{s name="CreditPlusTabBlockWest"}Bestellungsdaten{/s}',
			south: '{s name="CreditPlusTabBlockSouth"}Aktionen{/s}',
			billing: '{s name="CreditPlusTabBlockBilling"}Rechnungsanschrift{/s}',
			shipping: '{s name="CreditPlusTabBlockShipping"}Lieferanschrift{/s}'
		},
		customerData: {
			blockTitle: '{s name="CreditPlusTabCustomerData"}Kreditnehmer{/s}',
			firstName: '{s name="CreditPlusTabFirstName"}Vorname{/s}',
			lastName: '{s name="CreditPlusTabLastName"}Nachname{/s}',
			street: '{s name="CreditPlusTabStreet"}Straße{/s}',
			zipCode: '{s name="CreditPlusTabZipCode"}PLZ{/s}',
			city: '{s name="CreditPlusTabCity"}Stadt{/s}',
			country: '{s name="CreditPlusTabCountry"}Land{/s}',
			phone: '{s name="CreditPlusTabPhone"}Telefon{/s}',
			company: '{s name="CreditPlusTabCompany"}Firma{/s}'
		},
		contractData: {
			blockTitle: '{s name="CreditPlusTabContractData"}Vertragsdaten{/s}',
			referenceNumbers: '{s name="CreditPlusTabReferenceNumbers"}Referenznummern{/s}',
			productName: '{s name="CreditPlusTabProductName"}Produktname{/s}',
			financingState: '{s name="CreditPlusTabFinancingState"}Finanzierungsstatus{/s}',
			financingPrice: '{s name="CreditPlusTabFinancingPrice"}Preis{/s}',
			financingTime: '{s name="CreditPlusTabFinancingTime"}Laufzeit{/s}',
			shop: '{s name="CreditPlusTabReferenceNumberShop"}Shop{/s}',
			creditPlus: '{s name="CreditPlusTabReferenceNumberCreditPlus"}CreditPlus{/s}',
			missing: '{s name="CreditPlusTabContractMissing"}<p>Der Bezahlungsprozess wurde durch den Kunden noch nicht abgeschlossen.</p>{/s}'
		},
		orderData: {
			blockTitle: '{s name="CreditPlusTabOrderData"}Bestellungsdaten{/s}',
			orderNumber: '{s name="CreditPlusTabOrderNumber"}Bestellnummer{/s}',
			orderedOn: '{s name="CreditPlusTabOrderedOn"}Bestellt am{/s}',
			paidOn: '{s name="CreditPlusTabPaidOn"}Unterschrift durch Kunden am{/s}',
			paidOnNotYet: '{s name="CreditPlusTabPaidOnNotYet"}Noch nicht geschehen{/s}',
			deliveryDone: '{s name="CreditPlusTabDeliveryDone"}Ausgeliefert{/s}',
			deliveryDoneYes: '{s name="CreditPlusTabDeliveryDoneYes"}Ja{/s}',
			deliveryDoneNo: '{s name="CreditPlusTabDeliveryDoneNo"}Nein{/s}'
		},
		buttons: {
			sendNow: '{s name="CreditPlusTabSendNow"}Jetzt Warenversand bestätigen{/s}',
			cancelNow: '{s name="CreditPlusTabCancelNow"}Gesamt-/Reststorno{/s}'
		},
		cancel: {
			successTitle: '{s name="CreditPlusTabCancelSuccessTitle"}Erfolg{/s}',
			successMessage: '{s name="CreditPlusTabCancelSuccessMessage"}Die Bestellung wurde bei CreditPlus storniert.{/s}',
			errorTitle: '{s name="CreditPlusTabCancelErrorTitle"}Fehler{/s}',
			errorMessage: '{s name="CreditPlusTabCancelErrorMessage"}Leider konnte die Bestellung nicht storniert werden. Grund: [0]{/s}'
		},
		sendNow: {
			successTitle: '{s name="CreditPlusTabSendNowSuccessTitle"}Erfolg{/s}',
			successMessage: '{s name="CreditPlusTabSendNowSuccessMessage"}Die Bestellung wurde bei CreditPlus als versendet markiert.{/s}',
			errorTitle: '{s name="CreditPlusTabSendNowErrorTitle"}Fehler{/s}',
			errorMessage: '{s name="CreditPlusTabSendNowErrorMessage"}Leider konnte die Bestellung nicht versendet werden. Grund: [0]{/s}'
		}
	},

	/**
	 * The initComponent template method is an important initialization step for a Component.
	 * It is intended to be implemented by each subclass of Ext.Component to provide any needed constructor logic.
	 * The initComponent method of the class being created is called first,
	 * with each initComponent method up the hierarchy to Ext.Component being called thereafter.
	 * This makes it easy to implement and, if needed, override the constructor logic of the Component at any step in the hierarchy.
	 * The initComponent method must contain a call to callParent in order to ensure that the parent class' initComponent method is also called.
	 *
	 * @return void
	 */
	initComponent: function () {
		var me = this;
		me.title = me.snippets.title;
		me.items = me.createElements();
		me.callParent(arguments);
	},

	/**
	 * Creates the both containers for the field set
	 * to display the form fields in two columns.
	 *
	 * @return Ext.container.Container[] Contains the left and right container
	 */
	createElements:function () {
		var leftContainer, rightContainer, actionContainer, me = this;
		// No CreditPlus Order => tab is inactive anyways. Create fallback message just in case.
		if ( me.disabled ) {
			return [
				Ext.create('Ext.container.Container', {
					columnWidth: 1,
					border: false,
					layout: 'anchor',
					defaults: {
						anchor: '95%',
						labelWidth: 155,
						minWidth: 250,
						xtype: 'textfield'
					},
					items: {
						xtype: 'label',
						html: '<h1>Keine CreditPlus Bestellung</h1>'
					}
				})
			];
		}

		leftContainer = Ext.create('Ext.container.Container', {
			columnWidth: .5,
			border: false,
			layout: 'anchor',
			defaults: {
				anchor: '95%',
				labelWidth: 155,
				minWidth: 250,
				xtype: 'textfield'
			},
			items: me.createLeftElements()
		});

		rightContainer = Ext.create('Ext.container.Container', {
			columnWidth: .5,
			border: false,
			layout: 'anchor',
			defaults: {
				anchor: '95%',
				labelWidth: 155,
				minWidth: 250,
				xtype: 'textfield'
			},
			items: me.createRightElements()
		});

		actionContainer = Ext.create('Ext.container.Container', {
			columnWidth: 1,
			name: 'sico-credit-plus-actions',
			items: me.createActionButtons()
		});

		return [ leftContainer, rightContainer, actionContainer ];
	},

	/**
	 * Creates the left container of the CreditPlus field set.
	 *
	 * @return Object[] Contains the left components
	 */
	createLeftElements:function () {
		var me = this;

		return [
			{
				xtype: 'label',
				html: '<h2>'+me.snippets.blockTitles.west+'</h2>'
			},

			Ext.create('Ext.panel.Panel', {
				title: me.snippets.blockTitles.billing,
				bodyPadding: 10,
				margin: '5 0 5 0',
				items: [{
					xtype: 'container',
					renderTpl: me.createBillingShippingDataTemplate(),
					renderData: me.record.getBilling().first().data
				}]
			}),
			Ext.create('Ext.panel.Panel', {
				title: me.snippets.blockTitles.shipping,
				bodyPadding: 10,
				margin: '5 0 5 0',
				items: [{
					xtype: 'container',
					renderTpl: me.createBillingShippingDataTemplate(),
					renderData: me.record.getShipping().first().data
				}]
			}),
			Ext.create('Ext.panel.Panel', {
				title: me.snippets.orderData.blockTitle,
				bodyPadding: 10,
				margin: '5 0 5 0',
				flex: 1,
				items: [
					{
						xtype: 'container',
						renderTpl: me.createOrderDataTemplate(),
						renderData: me.record.raw
					}
				]
			})
		];
	},

	/**
	 * Creates the right container of the CreditPlus field set.
	 *
	 * @return Object[] Contains the right components
	 */
	createRightElements: function () {
		var me = this,
			oCPData = me.record.getSicoCreditPlusData(),
			aReturn = [];

		aReturn.push({
			xtype: 'label',
			html: '<h2>'+me.snippets.blockTitles.east+'</h2>'
		});
		if ( oCPData.getContract() && oCPData.getContract().get('cpReferenceNumber') ) {
			aReturn.push(
				Ext.create('Ext.panel.Panel', {
					title: me.snippets.customerData.blockTitle,
					bodyPadding: 10,
					margin: '5 0 5 0',
					flex: 1,
					items: [
						{
							xtype: 'container',
							renderTpl: me.createCustomerDataTemplate(),
							renderData: oCPData.getCustomer().data
						}
					]
				})
			);
			aReturn.push(
				Ext.create('Ext.panel.Panel', {
					title: me.snippets.contractData.blockTitle,
					bodyPadding: 10,
					margin: '5 0 5 0',
					flex: 1,
					items: [
						{
							xtype: 'container',
							renderTpl: me.createContractDataTemplate(),
							renderData: oCPData.getContract().data
						}
					]
				})
			);
		} else {
			aReturn.push({
				xtype: 'label',
				html: me.snippets.contractData.missing
			});
		}

		return aReturn;
	},

	/**
	 * Creates the action container of the CreditPlus field set.
	 *
	 * @return Object[] Contains the components
	 */
	createActionButtons:function () {
		var me = this,
			oContract = me.record.getSicoCreditPlusData().getContract();

		return [
			{
				xtype: 'label',
				html: '<h2>'+me.snippets.blockTitles.south+'</h2>'
			}, {
				xtype: 'button',
				text: me.snippets.buttons.sendNow,
				margin: '5 5 5 0',
				handler: function () {
					Ext.Ajax.request({
						url: '{url controller="SicoCreditPlusOrder" action="sendOrder"}',
						params: {
							id: me.record.get('id')
						},
						success: function(oRawResponse) {
							var oResponse = Ext.decode(oRawResponse.responseText);
							if ( oResponse.success ) {
								Shopware.Notification.createGrowlMessage(me.snippets.sendNow.successTitle, me.snippets.sendNow.successMessage, 'SicoCreditPlus');
								/* open send order dispatched mail dialog provided bei Leica technicians */
								Ext.Ajax.request({
									url: '{url controller="SicoCreditPlusOrder" action="getOrderDetails"}',
									params: {
										orderId: me.record.raw.id,
										stateId: me.record.raw.orderStatus.id
									},
									success: function(oMailData) {
										var oResponse = Ext.decode(oMailData.responseText);
										if ( oResponse.success === false ) { return; }
										var record = oResponse.data;
										if ( typeof record.mail === 'undefined' ) { return; }
										var mail = Ext.create('Shopware.apps.Order.model.Mail', record.mail);
										var order = Ext.create('Shopware.apps.Order.model.Order', record);
										var documentTypeStore = Ext.create('Shopware.apps.Order.store.DocType');
										var controller = Shopware.app.Application.getView('Shopware.apps.Order.view.mail.Window');
										controller.create({
											attached: [
												order.get('id')
											],
											record: order,
											listStore: null,
											documentTypeStore: documentTypeStore,
											mail: mail
										}).show();
									},
									failure: function(error) {
									}
								});
								/* open send order dispatched mail dialog provided bei Leica technicians */
							} else {
								Shopware.Notification.createGrowlMessage(me.snippets.sendNow.errorTitle, Ext.String.format(me.snippets.sendNow.errorMessage, oResponse.error), 'SicoCreditPlus');
							}
						},
						failure: function(error) {
							// Request timed out
						}
					});
				},
				disabled: ((oContract.get('finallyApproved')==false)||(oContract.get('deliveryDone')!=false))
			}, {
				xtype: 'button',
				text: me.snippets.buttons.cancelNow,
				margin: '5 5 5 0',
				handler: function () {
					Ext.Ajax.request({
						url: '{url controller="SicoCreditPlusOrder" action="cancelOrder"}',
						params: {
							id: me.record.get('id')
						},
						success: function(oRawResponse) {
							var oResponse = Ext.decode(oRawResponse.responseText);
							if ( oResponse.success ) {
								Shopware.Notification.createGrowlMessage(me.snippets.cancel.successTitle, me.snippets.cancel.successMessage, 'SicoCreditPlus');
							} else {
								Shopware.Notification.createGrowlMessage(me.snippets.cancel.errorTitle, Ext.String.format(me.snippets.cancel.errorMessage, oResponse.error), 'SicoCreditPlus');
							}
						},
						failure: function(error) {
							// Request timed out
						}
					});
				},
				disabled: (oContract.get('state') == '95')
			}
		];
	},

	/**
	 * Creates the XTemplate for an order status information panel.
	 *
	 * @return { Ext.XTemplate } generated Ext.XTemplate
	 */
	createCustomerDataTemplate: function() {
		return new Ext.XTemplate(
			'{literal}<tpl for=".">',
				'<div class="customer-info-pnl">',
					'<div class="base-info">',
						'<p>',
							'<span>{firstName} {lastName}</span>',
						'</p>',
						'<p>',
							'<span>{street}</span>',
						'</p>',
						'<p>',
							'<span>{zip}</span>&nbsp;',
							'<span>{city}</span>',
						'</p>',
					'</div>',
				'</div>',
			'</tpl>{/literal}'
		);
	},

	/**
	 * Creates the XTemplate for an order status information panel.
	 *
	 * @return { Ext.XTemplate } generated Ext.XTemplate
	 */
	createBillingShippingDataTemplate: function() {
		return new Ext.XTemplate(
			'{literal}<tpl for=".">',
				'<div class="customer-info-pnl" style="width: 50%; display: inline-block;">',
					'<div class="base-info">',
						'<p>',
							'<span>{firstName} {lastName}</span>',
						'</p>',
						'<p>',
							'<span>{street}</span>',
						'</p>',
						'<p>',
							'<span>{zipCode}</span>&nbsp;',
							'<span>{city}</span>',
						'</p>',
					'</div>',
				'</div>',
			'</tpl>{/literal}'
		);
	},

	/**
	 * Creates the XTemplate for an order status information panel.
	 *
	 * @return { Ext.XTemplate } generated Ext.XTemplate
	 */
	createContractDataTemplate: function() {
		var me = this;
		return new Ext.XTemplate(
			'{literal}<tpl for=".">',
				'<div class="customer-info-pnl">',
					'<div class="base-info">',
						'<table width="100%">',
							'<colgroup>' +
								'<col width="40%" />' +
								'<col width="60%" />' +
							'</colgroup>',
							'<tr>',
								'<td>'+me.snippets.contractData.referenceNumbers+'</td>',
								'<td>' +
									me.snippets.contractData.shop+': {dealerOrderNumber}<br />' +
									me.snippets.contractData.creditPlus+': {cpReferenceNumber}' +
								'</td>',
							'</tr>',
							'<tr><td colspan="2">&nbsp;</td></tr>',
							'<tr>',
								'<td>'+me.snippets.contractData.productName+'</td>',
								'<td>{conditionName}</td>',
							'</tr>',
							'<tr>',
								'<td>'+me.snippets.contractData.financingState+'</td>',
								'<td>{stateText}</td>',
							'</tr>',
							'<tr>',
								'<td>'+me.snippets.contractData.financingPrice+'</td>',
								'<td>{price}</td>',
							'</tr>',
							'<tr>',
								'<td>'+me.snippets.contractData.financingTime+'</td>',
								'<td>{creditDuration}</td>',
							'</tr>',
						'</table>',
					'</div>',
				'</div>',
			'</tpl>{/literal}'
		);
	},

	/**
	 * Creates the XTemplate for an order status information panel.
	 *
	 * @return { Ext.XTemplate } generated Ext.XTemplate
	 */
	createOrderDataTemplate: function() {
		var me = this;
		return new Ext.XTemplate(
			'{literal}<tpl for=".">' +
				'<div class="order-info-pnl">' +
					'<div class="base-info">' +
						'<table width="100%">' +
							'<colgroup>' +
								'<col width="40%" />' +
								'<col width="60%" />' +
							'</colgroup>' +
							'<tr>' +
								'<td>' + me.snippets.orderData.orderNumber+ '</td>' +
								'<td>{number}</td>' +
							'</tr>' +
							'<tr><td colspan="2">&nbsp;</td></tr>' +
							'<tr>' +
								'<td>'+me.snippets.orderData.orderedOn+'</td>' +
								'<td>'+Ext.Date.format(me.record.raw.orderTime, 'Y-m-d H:i:s')+'</td>' +
							'</tr>' +
							'<tr>' +
								'<td>'+me.snippets.orderData.deliveryDone+'</td>' +
								(me.record.raw.sicoCreditPlusData.contract.deliveryDone?'<td>'+me.snippets.orderData.deliveryDoneYes+'</td>':'<td>'+me.snippets.orderData.deliveryDoneNo+'</td>') +
							'</tr>' +
						'</table>' +
					'</div>' +
				'</div>' +
			'</tpl>{/literal}'
		);
	}
});
//{/block}