/**
 * Shopware 5
 * Copyright (c) shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 *
 * @category   Shopware
 * @package    Article
 * @subpackage Detail
 * @version    $Id$
 * @author SinkaCom AG AG
 */

/**
 * Shopware UI - Productgroup detail page based on Article detail page for inline edit list
 * The base component contains the configuration elements for the productgroup head data, like the productgroup name,
 * product type id and product class id.
 */
//{namespace name="frontend/plugins/payment/creditplus/backend"}
//{block name="backend/sico_credit_plus_productgroups/view/detail/base"}
Ext.define('Shopware.apps.SicoCreditPlusProductgroups.view.detail.Base', {
	/**
	 * Define that the billing field set is an extension of the Ext.form.FieldSet
	 * @string
	 */
	extend: 'Ext.form.FieldSet',

	/**
	 * The Ext.container.Container.layout for the fieldset's immediate child items.
	 * @object
	 */
	layout: 'column',

	/**
	 * List of short aliases for class names. Most useful for defining xtypes for widgets.
	 * @string
	 */
	alias: 'widget.sicoproductgroup-base-field-set',
	/**
	 * Set css class for this component
	 * @string
	 */
	cls: Ext.baseCSSPrefix + 'sicoproductgroup-base-field-set',

	/**
	 * Contains all snippets for the view component
	 * @object
	 */
	title: '{s name="SicoProductgroupWindowTitle"}Produktgruppe{/s}',
	snippets: {
		sicoName: '{s name="SicoProductgroupFieldSicoName"}Name{/s}',
		sicoProductTypeId: '{s name="SicoProductgroupFieldSicoProductTypeId"}Produkt Typ ID{/s}',
		sicoProductClassId: '{s name="SicoProductgroupFieldSicoProductClassId"}Produkt Klasse ID{/s}',
		regexNameValidation: '{s name="SicoProductgroupNameValidation"}Der Name darf nur aus Buchstaben, Leerzeichen, Zahlen, Umlauten und folgenden Sonderzeichen: -.,_ bestehen{/s}'
	},

	/**
	 * Contains the field set defaults.
	 */
	defaults: {
		labelWidth: 155
	},

	/**
	 * The initComponent template method is an important initialization step for a Component.
	 * It is intended to be implemented by each subclass of Ext.Component to provide any needed constructor logic.
	 * The initComponent method of the class being created is called first,
	 * with each initComponent method up the hierarchy to Ext.Component being called thereafter.
	 * This makes it easy to implement and, if needed, override the constructor logic of the Component at any step in the hierarchy.
	 * The initComponent method must contain a call to callParent in order to ensure that the parent class' initComponent method is also called.
	 *
	 * @return void
	 */
	initComponent: function () {
		var me = this;

		me.title = me.snippets.title;
		me.items = me.createElements();
		me.callParent(arguments);
	},

	/**
	 * Creates the both containers for the field set
	 * to display the form fields in two columns.
	 *
	 * @return [Array] Contains the left and right container
	 */
	createElements: function () {
		var leftContainer, rightContainer, me = this;

		leftContainer = Ext.create('Ext.container.Container', {
			columnWidth: 0.5,
			defaults: {
				labelWidth: 155,
				anchor: '100%'
			},
			padding: '0 20 0 0',
			layout: 'anchor',
			border: false,
			items: me.createLeftElements()
		});

		rightContainer = Ext.create('Ext.container.Container', {
			columnWidth: 0.5,
			layout: 'anchor',
			defaults: {
				labelWidth: 155,
				anchor: '100%'
			},
			border: false,
			items: me.createRightElements()
		});

		return [leftContainer, rightContainer];
	},


	/**
	 * Creates the field set items which displayed in the left column of the base field set
	 * @return Array
	 */
	createLeftElements: function () {
		var me = this;

		me.sicoName = Ext.create('Ext.form.field.Text', {
			name: 'sicoName',
			value: me.record.get('sicoName'),
			fieldLabel: me.snippets.sicoName,
			regex: /^[a-zA-Z0-9-_.,öäüÖÄÜß ]+$/,
			regexText: me.snippets.regexNameValidation,
			allowBlank: false,
			enableKeyEvents: true,
			checkChangeBuffer: 700,
			labelWidth: 155,
			anchor: '100%'
		});

		return [
			me.sicoName
		];
	},

	/**
	 * Creates the field set items which displayed in the right column of the base field set
	 * @return Array
	 */
	createRightElements: function () {
		var me = this;

		me.sicoProductTypeId = Ext.create('Ext.form.field.Text', {
			name: 'sicoProductTypeId',
			value: me.record.get('sicoProductTypeId'),
			fieldLabel: me.snippets.sicoProductTypeId,
			allowBlank: false,
			enableKeyEvents: true,
			checkChangeBuffer: 700,
			labelWidth: 155,
			anchor: '100%'
		});

		me.sicoProductClassId = Ext.create('Ext.form.field.Text', {
			name: 'sicoProductClassId',
			value: me.record.get('sicoProductClassId'),
			fieldLabel: me.snippets.sicoProductClassId,
			allowBlank: false,
			enableKeyEvents: true,
			checkChangeBuffer: 700,
			labelWidth: 155,
			anchor: '100%'
		});

		return [
			me.sicoProductTypeId,
			me.sicoProductClassId
		];
	}
});
//{/block}
