/**
 * Shopware 5
 * Copyright (c) shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 *
 * @category   Shopware
 * @package    SicoCreditPlus
 * @subpackage Productgroups
 * @version    $Id$
 * @author SinkaCom AG
 */

/**
 * Shopware UI - OfferedOptions for detail page
 * Based on the article detail pages price box, this handles the view around the offered options. 
 */
//{namespace name="frontend/plugins/payment/creditplus/backend"}
//{block name="backend/sico_credit_plus_productgroups/view/detail/offered_options"}
Ext.define('Shopware.apps.SicoCreditPlusProductgroups.view.detail.OfferedOptions', {
    /**
     * Define that the billing field set is an extension of the Ext.form.FieldSet
     * @string
     */
    extend:'Ext.form.FieldSet',
    /**
     * The Ext.container.Container.layout for the fieldset's immediate child items.
     * @object
     */
    layout: 'anchor',
    /**
     * List of short aliases for class names. Most useful for defining xtypes for widgets.
     * @string
     */
    alias: 'widget.sicoproductgroup-offeredoptions-field-set',
    /**
     * Set css class for this component
     * @string
     */
    cls: Ext.baseCSSPrefix + 'sicoproductgroup-offeredoptions-field-set',
    /**
     * Contains all snippets for the view component
     * @object
     */
    snippets:{
        title:'{s name="SicoProductgroupAreaTitleOfferedOptions"}Zinstabelle{/s}',
        any:'{s name="detail/prices/any"}Arbitrary{/s}',
        grid: {
            columns: {
                sicoMonths: '{s name="SicoOfferedOptionFieldSicoMonths"}Monate{/s}',
                sicoInterest: '{s name="SicoOfferedOptionFieldSicoInterest"}Effektiver Jahreszins{/s}',
                sicoInterestInvalid: '{s name="SicoOfferedOptionFieldSicoInterestInvalid"}Wählen Sie einen Wert zwischen 0 und 14,99.{/s}',
                sicoProdcode: '{s name="SicoOfferedOptionFieldSicoProdcode"}Produkt Code{/s}',
                sicoActive: '{s name="SicoOfferedOptionFieldSicoActive"}Aktiv{/s}',
				sicoActiveTrue: '{s name="SicoOfferedOptionFieldSicoActiveTrue"}Ja{/s}',
				sicoActiveFalse: '{s name="SicoOfferedOptionFieldSicoActiveFalse"}Nein{/s}',
                sicoRatefactor: '{s name="SicoOfferedOptionFieldSicoRatefactor"}Ratenfaktor{/s}',
                sicoRatefactorInvalid: '{s name="SicoOfferedOptionFieldSicoRatefactorInvalid"}Wählen Sie einen Wert der zwischen 0,00 und 1,00 liegt oder -1,00 für automatische Berechnung.{/s}'
            },
			delete: '{s name="RowDelete"}Löschen{/s}',
			mathematicallyCalculated:'{s name="SicoOfferedOptionMathematicallyCalculated"}Mathematisch berechnet{/s}'
        },
		toolbar: {
			add: '{s name="ButtonAdd"}Hinzufügen{/s}'
		},
		offeredOption: '{s name="SicoProductgroupFieldOfferedOptions"}Zinstabelle{/s}',
		deleteSuccessTitle: '{s name="SicoProductgroupFieldOfferedOptionsDeleteSuccessTitle"}Löschen erfolgreich{/s}',
		deleteSuccessMessage: '{s name="SicoProductgroupFieldOfferedOptionsDeleteSuccessMessage"}Der Eintrag der Zinstabelle wurde gelöscht.{/s}',
		deleteErrorTitle: '{s name="SicoProductgroupFieldOfferedOptionsDeleteErrorTitle"}Löschen fehlgeschlagen{/s}',
		deleteErrorMessage: '{s name="SicoProductgroupFieldOfferedOptionsDeleteErrorMessage"}Der Eintrag der Zinstabelle wurde nicht gelöscht. Öffnen Sie die Detailansicht erneut und prüfen Sie, ob dieser Eintrag noch existiert.{/s}',
		singleDeleteTitle: '{s name="SicoProductgroupFieldOfferedOptionsDeleteSingleTitle"}Löschen eines Eintrags{/s}',
		singleDeleteMessage: '{s name="SicoProductgroupFieldOfferedOptionsDeleteSingleMessage"}Möchten Sie den Eintrag [0] wirklich löschen?{/s}'
    },

    /**
	 * The initComponent template method is an important initialization step for a Component.
     * It is intended to be implemented by each subclass of Ext.Component to provide any needed constructor logic.
     * The initComponent method of the class being created is called first,
     * with each initComponent method up the hierarchy to Ext.Component being called thereafter.
     * This makes it easy to implement and, if needed, override the constructor logic of the Component at any step in the hierarchy.
     * The initComponent method must contain a call to callParent in order to ensure that the parent class' initComponent method is also called.
	 *
	 * @return void
	 */
    initComponent: function () {
        var me = this;
        me.title = me.snippets.title;
		me.items = me.createElements();
        me.registerEvents();
        me.callParent(arguments);
    },

    /**
     * Registers additional component events.
     */
    registerEvents: function() {
    	this.addEvents(
            /**
             * Fired when the user clicks the remove action column of the price grid
             *
             * @event
             * @param [array] The row record
             */
            'removeOfferedOption'
    	);
    },

    /**
     * Creates the elements for the description field set.
     * @return [] Contains all Ext.form.Fields for the description field set
     */
    createElements: function () {
        var me = this;
        return [
			me.createOfferedOptionsGrid()
		];
    },

    /**
     * Creates a grid for the offered options.
     *
     * @return Ext.grid.Panel
     */
    createOfferedOptionsGrid: function() {
		var me = this;
		return Ext.create('Ext.grid.Panel', {
			alias: 'widget.sicoproductgroup-offeredoptions-grid',
			cls: Ext.baseCSSPrefix + 'sicoproductgroup-offeredoptions-grid',
			height: 300,
			sortableColumns: false,
			selType: 'rowmodel',
			plugins: [
				Ext.create('Ext.grid.plugin.RowEditing', {
					clicksToEdit: 1
				})
			],
			defaults: {
				align: 'right',
				flex: 2
			},
			title: me.snippets.offeredOption,
			store: me.record.getOfferedOptions(),
			columns: me.getColumns(),
			dockedItems: me.getToolbar()
		});
	},

	/**
	 * Creates the grid toolbar with the add and delete button
	 *
	 * @return { Ext.toolbar.Toolbar } grid toolbar
	 */
	getToolbar: function () {
		var me = this;

		return Ext.create('Ext.toolbar.Toolbar', {
			dock:'top',
			ui: 'shopware-ui',
			items:[
				{
					iconCls: 'sprite-plus-circle-frame',
					text: me.snippets.toolbar.add,
					action: 'addOfferedOption',
					detailView: me,
					handler: me.addOfferedOption
				}
			]
		});
	},

    /**
     * Creates the elements for the description field set.
     * @return Array -  Contains all Ext.form.Fields for the description field set
     */
    getColumns: function () {
		var me = this;

		return [
			{
				header: me.snippets.grid.columns.sicoMonths,
				dataIndex: 'sicoMonths',
				flex: 1,
				editor: {
					xtype: 'numberfield',
					minValue: 0,
					maxValue: 360,
					decimalPrecision: 0,
					step: 1
				}
			}, {
				xtype: 'numbercolumn',
				header: me.snippets.grid.columns.sicoInterest,
				dataIndex: 'sicoInterest',
				flex: 1,
				editor: {
					xtype: 'numberfield',
					minValue: 0,
					maxValue: 14.99,
					decimalPrecision: 2,
					submitLocaleSeparator: false,
					invalidText: me.snippets.grid.columns.sicoInterestInvalid,
					step: 0.01
				},
				renderer: function (v) {
					if (Ext.isNumeric(v)) {
						return v;
					} else {
						return '';
					}
				}
			}, {
				header: me.snippets.grid.columns.sicoProdcode,
				dataIndex: 'sicoProdcode',
				flex: 1,
				editor: {
					xtype: 'textfield'
				}
			}, {
				xtype: 'booleancolumn',
				header: me.snippets.grid.columns.sicoActive,
				dataIndex: 'sicoActive',
				trueText: me.snippets.grid.columns.sicoActiveTrue,
				falseText: me.snippets.grid.columns.sicoActiveFalse,
				flex: 1,
				editor: {
					xtype: 'checkboxfield',
					uncheckedValue: 0,
					inputValue: 1
				}
			}, {
				xtype: 'numbercolumn',
				header: me.snippets.grid.columns.sicoRatefactor,
				dataIndex: 'sicoRatefactor',
				flex: 1,
				editor: {
					xtype: 'numberfield',
					minValue: -1.000000000,
					maxValue: 0.999999999,
					decimalPrecision: 9,
					submitLocaleSeparator: false,
					step: 0.001,
					validator: function(v) {
						if ( !Ext.isNumeric(v) && /[0-9-,]+/.test(v) ) {
							v = v.replace(',','.');
						}
						if ( !Ext.isNumeric(v) ) {
							return me.snippets.grid.columns.sicoRatefactorInvalid;
						}
						v = parseFloat(v);
						if ( v == -1.00 ) {
							return true;
						}
						if ( v <= 0.00 ) {
							return me.snippets.grid.columns.sicoRatefactorInvalid;
						}
						if ( v >= 1.00 ) {
							return me.snippets.grid.columns.sicoRatefactorInvalid;
						}
						return true;
					}
				},
				renderer: function (v) {
					if (Ext.isNumeric(v)) {
						if ( v == -1.00 ) {
							return me.snippets.grid.mathematicallyCalculated
						}
						return v;
					} else {
						return '';
					}
				}
			}, {
				xtype: 'actioncolumn',
				width: 25,
				items: [
					{
						iconCls: 'sprite-minus-circle-frame',
						action: 'delete',
						tooltip: me.snippets.grid.delete,
						handler: me.onDeleteSingleRow,
						sicoView: me,
						/**
						 * There must be at least one
						 * @param value
						 * @param metadata
						 * @param record
						 * @param rowIdx
						 * @return string
						 */
						getClass: function (value, metadata, record, rowIdx) {
							if (rowIdx === 0) {
								return 'x-hidden';
							}
						}
					}
				]
			}
		];
	},

	/**
	 * Event listener method which fires when the user use the
	 * delete icon at the end of an grid row to delete a single customer.
	 *
	 * @param { Shopware.apps.SicoCreditPlusProductgroups.view.detail.OfferedOptions } view - The view
	 * @param { integer } rowIndex - On which row position has been clicked
	 * @param { integer } colIndex - On which row position has been clicked
	 * @param { Object } item - On which row position has been clicked
	 * @param { Array } opts - On which row position has been clicked
	 * @param { Shopware.apps.SicoCreditPlusOfferedOptions.model.SicoCreditPlusOfferedOption } record - The deleted record
	 * @return void
	 */
	onDeleteSingleRow: function (view, rowIndex, colIndex, item, opts, record) {
		var me = this.items[0].sicoView,
			message = Ext.String.format(me.snippets.singleDeleteMessage,record.get('name')),
			title = me.snippets.singleDeleteTitle;

		me.doDeleteRows([record], title, message);
	},

	/**
	 * 
	 * @param { [Shopware.apps.SicoCreditPlusOfferedOptions.model.SicoCreditPlusOfferedOption] } aRecords - The OfferedOptions to delete
	 * @param { String } sTitle - 
	 * @param { String } sMessage
	 */
	doDeleteRows: function( aRecords, sTitle, sMessage) {
		var me = this;
		// we do not just delete - we are polite and ask the user if he is sure.
		Ext.MessageBox.confirm(sTitle, sMessage, function (response) {
			if ( response !== 'yes' ) {
				return;
			}
			Ext.each(aRecords,
				/**
				 * 
				 * @param { Shopware.apps.SicoCreditPlusOfferedOptions.model.SicoCreditPlusOfferedOption } oRecord
				 */
				function (oRecord) {
					oRecord.destroy({
						callback: function (data, operation) {
							if (operation.success === true) {
								Shopware.Notification.createGrowlMessage(me.snippets.deleteSuccessTitle, me.snippets.deleteSuccessMessage, 'OfferedOptions');
								me.record.getOfferedOptions().remove(oRecord);
							} else {
								Shopware.Notification.createGrowlMessage(me.snippets.deleteErrorTitle, me.snippets.deleteErrorMessage, 'OfferedOptions');
							}
						}
					});
				}
			);
		});
	},

	/**
	 * Adds an offeredOption to the currently show data entry
	 * @return void
	 */
	addOfferedOption: function() {
		var me = this.detailView;
		var oModel = Ext.ModelManager.getModel('Shopware.apps.SicoCreditPlusOfferedOptions.model.SicoCreditPlusOfferedOption');
		var newRecord = oModel.create({
			sicoMonths: 6,
			sicoProdgroup: me.record.get('id'),
			sicoRatefactor: -1.00,
			sicoProdcode: 'Standard',
			sicoActive: 0
		});
		me.record.getOfferedOptions().add(newRecord);
	}

});
//{/block}
